/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dispatch.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dispatch.dto.DispatchExpertQuery;
import com.tofly.dispatch.entity.DispatchExpert;
import com.tofly.dispatch.service.DispatchExpertService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 专家库管理
 *
 * @author admin
 * @date Thu Nov 10 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/expert")
@Api(tags="专家库管理接口")
public class DispatchExpertController {

    private final DispatchExpertService expertService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param dispatchExpert 专家库管理
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getExpertPage(Page page, @ModelAttribute DispatchExpertQuery dispatchExpert) {
      return ResultRespone.success(expertService.getPageData(page, dispatchExpert));
    }


    /**
     * 通过id查询专家库管理
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(expertService.getById(id));
    }

    /**
     * 新增专家库管理
     * @param dispatchExpert 专家库管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增专家库管理" )
    @ApiOperation(value = "新增专家库管理")
    @PostMapping
    public ResultRespone save(@RequestBody DispatchExpert dispatchExpert) {
      return ResultRespone.success(expertService.saveDispatchExpert(dispatchExpert,true));
    }

    /**
     * 修改专家库管理
     * @param dispatchExpert 专家库管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改专家库管理" )
    @ApiOperation(value = "修改专家库管理")
    @PutMapping
    public ResultRespone updateById(@RequestBody DispatchExpert dispatchExpert) {
        return ResultRespone.success(expertService.saveDispatchExpert(dispatchExpert,false));
    }

    /**
     * 通过id删除专家库管理
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除专家库管理" )
    @ApiOperation(value = "通过ID删除专家库管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(expertService.deleteByIds(id+""));
    }

    /**
     * 通过id批量删除专家库管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除专家库管理" )
    @ApiOperation(value = "通过ID批量删除专家库管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(expertService.deleteByIds(ids));
    }
}
