/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dispatch.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dispatch.dto.FacilitiesQuery;
import com.tofly.dispatch.entity.Facilities;
import com.tofly.dispatch.service.FacilitiesService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 应急设施
 *
 * @author admin
 * @date Wed Nov 09 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/facilities")
@Api(tags="应急设施基础数据接口")
public class FacilitiesController {

    private final  FacilitiesService facilitiesService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param facilities 应急设施
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getFacilitiesPage(Page page, @ModelAttribute FacilitiesQuery facilities) {
      return ResultRespone.success(facilitiesService.getPageData(page, facilities));
    }


    /**
     * 通过id查询应急设施
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(facilitiesService.getById(id));
    }

    /**
     * 新增应急设施
     * @param facilities 应急设施
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增应急设施" )
    @ApiOperation(value = "新增应急设施")
    @PostMapping
    public ResultRespone save(@RequestBody Facilities facilities) {
      return ResultRespone.success(facilitiesService.save(facilities));
    }

    /**
     * 修改应急设施
     * @param facilities 应急设施
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改应急设施" )
    @ApiOperation(value = "修改应急设施")
    @PutMapping
    public ResultRespone updateById(@RequestBody Facilities facilities) {
      return ResultRespone.success(facilitiesService.updateById(facilities));
    }

    /**
     * 通过id删除应急设施
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除应急设施" )
    @ApiOperation(value = "通过ID删除应急设施")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(facilitiesService.deleteByIds(id+""));
    }

    /**
     * 通过id批量删除应急设施
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除应急设施" )
    @ApiOperation(value = "通过ID批量删除应急设施")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(facilitiesService.deleteByIds(ids));
    }
}
