package com.tofly.filling.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.filling.entity.Filling;
import com.tofly.filling.entity.dto.FillingDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 填报记录(Filling)表服务接口
 *
 * @author HaiQiu
 * @since 2023-01-10 11:39:03
 */
public interface FillingService extends IService<Filling> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    Filling get(Serializable id);

    /**
     * 新增
     *
     * @param fillingDto 实体
     */
    void insert(FillingDto fillingDto);

    /**
     * 修改
     *
     * @param fillingDto 实体
     */
    void edit(FillingDto fillingDto);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page    分页
     * @param filling 查询wrapper
     * @param dates
     * @return 数据集合
     */
    Page<Filling> pageList(Page page, Filling filling, List<String> dates);

    /**
     * 数据导出
     *
     * @param page    分页
     * @param filling 查询模型
     * @param isAll   是否导出全部数据，true是，false否（默认否）
     */
    void export(Page page, Filling filling, boolean isAll);

    /**
     * 查询所有数据
     *
     * @param filling 查询条件
     * @return 查询所有数据
     */
    List<Filling> allList(Filling filling);
}

