package com.tofly.filling.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.core.util.DateUtil;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.filling.mapper.FillingMapper;
import com.tofly.filling.entity.Filling;
import com.tofly.filling.entity.dto.FillingDto;
import com.tofly.filling.service.FillingService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 填报记录(Filling)表服务实现类
 *
 * @author HaiQiu
 * @since 2023-01-10 11:39:03
 */
@Service("fillingService")
public class FillingServiceImpl extends ServiceImpl<FillingMapper, Filling> implements FillingService {


    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public Filling get(Serializable id) {
        return getById(id);
    }


    /**
     * 新增
     *
     * @param fillingDto 实体
     */
    @Override
    public void insert(FillingDto fillingDto) {
        Filling filling = convert(fillingDto);
        if (!save(filling)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param fillingDto 实体
     */
    @Override
    public void edit(FillingDto fillingDto) {
        Filling filling = convert(fillingDto);
        if (!updateById(filling)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param fillingDto dto实体
     */
    private Filling convert(FillingDto fillingDto) {
        Filling filling = new Filling();
        BeanUtils.copyProperties(fillingDto, filling);
        return filling;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        Filling filling = new Filling();
        ExcelUtils.createTemplate("填报记录", "填报记录",
                "填报记录", Filling.class, Collections.singletonList(filling));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<Filling> excelToList = ExcelUtils.getExcelToList("填报记录", file, Filling.class);
        if (CollectionUtils.isEmpty(excelToList)) {
            throw new RuntimeException("数据解析为空");
        }
        saveBatch(excelToList);
    }

    /**
     * 分页查询
     *
     * @param page    分页
     * @param filling 查询wrapper
     * @param dates
     * @return 数据集合
     */
    @Override
    public Page<Filling> pageList(Page page, Filling filling, List<String> dates) {
        LambdaQueryWrapper<Filling> wrapper = Wrappers.<Filling>lambdaQuery(filling);
        if (!CollectionUtils.isEmpty(dates)) {
            wrapper.between(Filling::getCreateTime, DateUtil.getDateFormat(dates.get(0)),
                    DateUtil.getDateFormat(dates.get(1)));
        }
        return page(page, wrapper);
    }

    /**
     * 数据导出
     *
     * @param page    分页
     * @param filling 查询模型
     * @param isAll   是否导出全部数据，true是，false否（默认否）
     */
    @Override
    public void export(Page page, Filling filling, boolean isAll) {
        Page<Filling> pageData = new Page<>();
        if (!isAll) {
            pageData = page(page, Wrappers.lambdaQuery(filling));
        } else {
            pageData.setRecords(list(Wrappers.lambdaQuery(filling)));
        }
        //导出数据
        ExcelUtils.getListToExcel(Filling.class, pageData.getRecords(), "填报记录");
    }

    /**
     * 查询所有数据
     *
     * @param filling 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<Filling> allList(Filling filling) {
        return list(Wrappers.lambdaQuery(filling));
    }
}

