package com.tofly.project.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.project.entity.ProjectRegistration;
import com.tofly.project.entity.dto.ProjectOtherSupervisionDto;
import com.tofly.project.entity.dto.ProjectRegistrationDto;
import com.tofly.project.entity.dto.ProjectSearchDto;
import com.tofly.project.service.ProjectRegistrationService;
import io.swagger.annotations.*;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import com.tofly.common.BaseController;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 工程登记(ProjectRegistration)表控制层
 *
 * @author HaiQiu
 * @since 2022-11-14 13:50:36
 */
@Api(tags = "工程登记")
@RestController
@RequestMapping("projectRegistration")
public class ProjectRegistrationController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private ProjectRegistrationService projectRegistrationService;

    /**
     * 分页查询数据
     *
     * @param page                分页对象
     * @param projectRegistration 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "工程登记-分页查询数据")
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone<Page<ProjectRegistration>> pageList(Page page, ProjectRegistration projectRegistration, ProjectSearchDto projectSearchDto) {
        return decide(projectRegistrationService.pageList(page, projectRegistration,projectSearchDto));
    }

    /**
     * 查询所有数据
     *
     * @param projectRegistration 查询实体
     * @return 所有数据
     */
    @ToFlyAppLog(title = "工程登记-查询所有数据")
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone<List<ProjectRegistration>> allList(ProjectRegistration projectRegistration) {
        return decide(projectRegistrationService.allList(projectRegistration));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ToFlyAppLog(title = "工程登记-通过主键查询单条数据")
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone<ProjectRegistration> get(@PathVariable Serializable id) {
        return decide(projectRegistrationService.get(id));
    }

    /**
     * 新增数据
     *
     * @param projectRegistrationDto 实体
     * @return 新增结果
     */
    @ToFlyAppLog(title = "工程登记-新增数据")
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@RequestPart @Validated ProjectRegistrationDto projectRegistrationDto,
                                @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        projectRegistrationService.insert(projectRegistrationDto,files);
        return decide("新增成功");
    }

    /**
     * 修改数据
     *
     * @param projectRegistrationDto 实体
     * @return 修改结果
     */
    @ToFlyAppLog(title = "工程登记-修改数据")
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@RequestPart @Validated ProjectRegistrationDto projectRegistrationDto,
                              @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        projectRegistrationService.edit(projectRegistrationDto,files);
        return decide("修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ToFlyAppLog(title = "工程登记-删除数据")
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        projectRegistrationService.delete(idList);
        return decide("删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ToFlyAppLog(title = "工程登记-模版下载")
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        projectRegistrationService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ToFlyAppLog(title = "工程登记-Excel数据导入")
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        projectRegistrationService.importData(file);
        return decide("导入成功");
    }


    /**
     * 数据导出
     *
     * @param page                分页
     * @param projectRegistration 查询模型
     * @param isAll               是否导出全部数据，true是，false否
     */
    @ToFlyAppLog(title = "工程登记-数据导出")
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page page, ProjectRegistration projectRegistration, Boolean isAll) {
        projectRegistrationService.export(page, projectRegistration, isAll);
    }
}

