package com.tofly.project.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

import org.hibernate.validator.constraints.*;

/**
 * 工程施工(ProjectConstruction)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-14 16:32:22
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "ProjectConstruction对象", description = "工程施工")
public class ProjectConstructionDto implements Serializable {

    private static final long serialVersionUID = -74298568242900870L;

    /**
     * 主键ID
     */
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * FLOW_PROJECT_REGISTRATION（工程登记ID）
     */
    @Digits(integer = 20, fraction = 0, message = "FLOW_PROJECT_REGISTRATION（工程登记ID）长度不能超过20")
    @ApiModelProperty(value = "FLOW_PROJECT_REGISTRATION（工程登记ID）")
    private Long projectId;

    /**
     * 施工单位
     */
    @Size(max = 200, message = "施工单位长度不能超过200")
    @ApiModelProperty(value = "施工单位")
    private String unit;

    /**
     * 施工类型（字典取值）
     */
    @Digits(integer = 20, fraction = 0, message = "施工类型（字典取值）长度不能超过20")
    @ApiModelProperty(value = "施工类型（字典取值）")
    private Long dataType;

    /**
     * 施工负责人
     */
    @Size(max = 100, message = "施工负责人长度不能超过100")
    @ApiModelProperty(value = "施工负责人")
    private String contacts;

    /**
     * 联系电话
     */
    @Size(max = 85, message = "联系电话长度不能超过85")
    @ApiModelProperty(value = "联系电话")
    private String contactsPhone;

    /**
     * 计划工期
     */
    @Size(max = 85, message = "计划工期长度不能超过85")
    @ApiModelProperty(value = "计划工期")
    private String plannedDuration;

    /**
     * 施工方（可以取字典，也可以0本公司，1第三方）
     */
    @Digits(integer = 20, fraction = 0, message = "施工方（可以取字典，也可以0本公司，1第三方）长度不能超过20")
    @ApiModelProperty(value = "施工方（可以取字典，也可以0本公司，1第三方）")
    private Long constructor;

    /**
     * 备注
     */
    @Size(max = 200, message = "备注长度不能超过200")
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 计划完工时间
     */
    @ApiModelProperty(value = "计划完工时间")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date plannedEndTime;

    /**
     * 计划开工时间
     */
    @ApiModelProperty(value = "计划开工时间")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date plannedStartTime;

    /**
     * 附件
     */
    @Size(max = 1000, message = "附件长度不能超过1000")
    @ApiModelProperty(value = "附件")
    private String annex;

    /**
     * 排序
     */
    @Digits(integer = 20, fraction = 0, message = "排序长度不能超过20")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 逻辑删除，1删除，0不删除
     */
    @ApiModelProperty(value = "逻辑删除，1删除，0不删除")
    private Boolean deleted;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    /**
     * 创建人
     */
    @Digits(integer = 20, fraction = 0, message = "创建人长度不能超过20")
    @ApiModelProperty(value = "创建人")
    private Long createUser;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    /**
     * 修改人
     */
    @Digits(integer = 20, fraction = 0, message = "修改人长度不能超过20")
    @ApiModelProperty(value = "修改人")
    private Long updateUser;

    /**
     * 编号
     */
    @Size(max = 100, message = "编号长度不能超过100")
    @ApiModelProperty(value = "编号")
    private String numberNo;

}

