package com.tofly.project.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.project.entity.ProjectConstruction;
import com.tofly.project.entity.dto.ProjectConstructionDto;

import com.tofly.project.entity.vo.ProjectConstructionVo;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 工程施工(ProjectConstruction)表服务接口
 *
 * @author HaiQiu
 * @since 2022-11-14 11:56:44
 */
public interface ProjectConstructionService extends IService<ProjectConstruction> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    ProjectConstruction get(Serializable id);

    /**
     * 新增
     *
     * @param projectConstructionDto 实体
     * @param files 附件
     */
    void insert(ProjectConstructionDto projectConstructionDto, List<MultipartFile> files);

    /**
     * 修改
     *
     * @param projectConstructionDto 实体
     * @param files
     */
    void edit(ProjectConstructionDto projectConstructionDto, List<MultipartFile> files);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page                分页
     * @param projectConstruction 查询wrapper
     * @return 数据集合
     */
    Page<ProjectConstructionVo> pageList(Page page, ProjectConstructionVo projectConstruction);

    /**
     * 数据导出
     *
     * @param page                分页
     * @param projectConstruction 查询模型
     * @param isAll               是否导出全部数据，true是，false否
     */
    void export(Page page, ProjectConstruction projectConstruction, Boolean isAll);

    /**
     * 查询所有数据
     *
     * @param projectConstruction 查询条件
     * @return 查询所有数据
     */
    List<ProjectConstruction> allList(ProjectConstruction projectConstruction);
}

