/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.stocks.controller;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.stocks.dto.DeviceHandleInfoDto;
import com.tofly.stocks.dto.DeviceHandleInfoQuery;
import com.tofly.stocks.service.DeviceHandleInfoService;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import java.util.Arrays;
import java.util.List;
/**
 * 设备处置信息表
 *
 * @author admin
 * @date Fri Nov 25 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/devicehandleinfo")
@Api(tags="设备处置信息表接口")
public class DeviceHandleInfoController {

    private final  DeviceHandleInfoService deviceHandleInfoService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param deviceHandleInfoQuery 设备处置信息表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getDeviceHandleInfoPage(Page page, @Validated DeviceHandleInfoQuery deviceHandleInfoQuery) {
      return ResultRespone.success(deviceHandleInfoService.getDataPage(page,deviceHandleInfoQuery));
    }


    /**
     * 通过id查询设备处置信息表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(deviceHandleInfoService.getById(id));
    }

    /**
     * 新增设备处置信息表
     * @param deviceHandleInfoDto 设备处置信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增设备处置信息表" )
    @ApiOperation(value = "新增设备处置信息表")
    @PostMapping
    public ResultRespone save(@Validated DeviceHandleInfoDto deviceHandleInfoDto,
    		@RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        return ResultRespone.success(deviceHandleInfoService.saveDeviceHandleInfo(deviceHandleInfoDto, files));
    }

    /**
     * 修改设备处置信息表
     * @param deviceHandleInfoDto 设备处置信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改设备处置信息表" )
    @ApiOperation(value = "修改设备处置信息表")
    @PutMapping
    public ResultRespone updateById(@Validated DeviceHandleInfoDto deviceHandleInfoDto,
    		@RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {    	
    	System.out.println("***********************"+deviceHandleInfoDto.getHandleRemark());
        return ResultRespone.success(deviceHandleInfoService.editDeviceHandleInfo(deviceHandleInfoDto, files));
    }
    


    /**
     * 通过id批量删除设备处置信息表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除设备处置信息表" )
    @ApiOperation(value = "通过ID批量删除设备处置信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(deviceHandleInfoService.deleteByIds(ids));
    }
    

          
}