/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.stocks.controller;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.stocks.dto.InstockInfoDto;
import com.tofly.stocks.dto.InstockInfoNewDto;
import com.tofly.stocks.dto.InstockInfoQuery;
import com.tofly.stocks.service.InstockInfoService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 库存入库信息表
 *
 * @author admin
 * @date Fri Nov 25 00:00:00 CST 2022
 */
@RestController
@AllArgsConstructor
@RequestMapping("/instockinfo")
@Api(tags="库存入库信息表接口")
public class InstockInfoController {

    private final  InstockInfoService instockInfoService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param instockInfoQuery 库存入库信息表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getInstockInfoPage(Page page, @Validated InstockInfoQuery instockInfoQuery) {
      return ResultRespone.success(instockInfoService.getDataPage(page, instockInfoQuery));
    }


    /**
     * 通过id查询库存入库信息表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(instockInfoService.getById(id));
    }

    /**
     * 新增库存入库信息表
     * @param instockInfoDto 库存入库信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增库存入库信息表" )
    @ApiOperation(value = "新增库存入库信息表")
    @PostMapping
    public ResultRespone save(@Validated InstockInfoDto instockInfoDto,
                              @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
      return ResultRespone.success(instockInfoService.saveInstockInfo(instockInfoDto,files));
    }

    /**
     * 修改库存入库信息表
     * @param instockInfoNewDto 库存入库信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改库存入库信息表" )
    @ApiOperation(value = "修改库存入库信息表")
    @PutMapping
    public ResultRespone updateById(@Validated InstockInfoNewDto instockInfoNewDto,
        @RequestPart(required = false) @ApiParam("附件列表") List<MultipartFile> files) {
        InstockInfoDto instockInfoDto=new InstockInfoDto();
        BeanUtils.copyProperties(instockInfoNewDto, instockInfoDto);
        try {
            instockInfoDto.setBuinessType(Long.parseLong(instockInfoNewDto.getBuinessType()));
        }catch (Exception e){

        }
        try {
            instockInfoDto.setPurchaseType(Long.parseLong(instockInfoNewDto.getPurchaseType()));
        }catch (Exception e){

        }
        try {
            instockInfoDto.setInstockType(Long.parseLong(instockInfoNewDto.getInstockType()));
        }catch (Exception e){

        }

            return ResultRespone.success(instockInfoService.editInstockInfo(instockInfoDto,files) );
        }

    /**
     * 通过id批量删除库存入库信息表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除库存入库信息表" )
    @ApiOperation(value = "通过ID批量删除库存入库信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(instockInfoService.deleteByIds(ids));
    }
}
