package com.tofly.base.config;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.rabbitmq.client.Channel;
import com.tofly.base.entity.Department;
import com.tofly.base.entity.User;
import com.tofly.base.service.DepartmentService;
import com.tofly.base.service.UserService;
import com.tofly.common.core.util.PasswordUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringEscapeUtils;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;


/**
 * 物联网监听数据配置
 */
@Component
@Slf4j
public class ReceiveHandler {

    public static final String YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    @Autowired
    private UserService userService;

    @Autowired
    private DepartmentService departmentService;

    @Value("${user.password}")
    private String intPassword;


    //监听QUEUE_INFORM_ZHDD队列
//    @RabbitListener(queues = {RabbitmqConfig.QUEUE_INFORM_ZHDD})
    @Transactional(rollbackFor = Exception.class)
    public void receive_zhdd(Object msg, Message message, Channel channel) {

        byte[] body = message.getBody();
        String encode = Base64.getEncoder().encodeToString(body);
        byte[] decode = Base64.getDecoder().decode(encode);
        String s = new String(decode);
        log.info("获取监听数据：{}", s);
        if (!StringUtils.isEmpty(s)) {
            //处理JSON格式数据,防止保存到数据库blob字段时,出现多个转义符 \
            //去除所有转义符
            s =StringEscapeUtils.unescapeJava(s);
           //去除首尾双引号
            s= s.substring(1, s.length() - 1);
            JSONObject jsonObject = JSON.parseObject(s);
            if (jsonObject != null) {
                List<Department> list = departmentService.list(Wrappers.<Department>query().lambda().isNotNull(Department::getOrganId));
                Map<String, List<Department>> departMap = new HashMap<>();
                if (null != list && list.size() > 0) {
                    departMap = list.stream().collect(Collectors.groupingBy(Department::getOrganId));
                }
                if (jsonObject.get("data") != null) {
                    JSONObject data = (JSONObject) jsonObject.get("data");
                    if (data != null) {
                        saveDepartment(departMap, data);
                    }
                }
                if (jsonObject.get("dataArray") != null) {
                    JSONArray dataArray = jsonObject.getJSONArray("dataArray");
                    if (dataArray != null) {
                        JSONArray jsonArray = JSONArray.parseArray(dataArray.toString());
                        for (Iterator<Object> iterator = jsonArray.iterator(); iterator.hasNext(); ) {
                            JSONObject data = (JSONObject) iterator.next();
                            saveDepartment(departMap, data);
                        }
                    }
                }
            }
        }

    }
    @Transactional(rollbackFor = Exception.class)
    void saveDepartment(Map<String, List<Department>> departMap, JSONObject data) {
        Object organId = data.get("organId");
        Object organCode = data.get("organCode");
        Object organName = data.get("organName");
        Object shortName = data.get("shortName");
        Object parentOrganCode = data.get("parentOrganCode");
        Object organState = data.get("organState");
        Department department = new Department();
        department.setCode(organCode != null ? organCode.toString() : null);
        department.setOrganId(organId != null ? organId.toString() : null);
        department.setName(organName != null ? organName.toString() : null);
        department.setStatusFlag("1");
        department.setShortName(shortName != null ? shortName.toString() : null);
        department.setCompanyId(1l);
        department.setParentOrganCode(parentOrganCode != null ? parentOrganCode.toString() : null);
        department.setOrganState(organState != null ? organState.toString() : null);
        if (departMap.get(department.getOrganId()) != null) {
            department.setId(departMap.get(department.getOrganId()).get(0).getId());
            departmentService.updateById(department);
        } else {
            departmentService.save(department);
        }
    }

    //监听QUEUE_INFORM_ZHDD队列
    @RabbitListener(queues = {RabbitmqConfig.QUEUE_INFORM_ZHDD02})
    @Transactional(rollbackFor = Exception.class)
    public void receive_zhdd02(Object msg, Message message, Channel channel) {

        byte[] body = message.getBody();
        String encode = Base64.getEncoder().encodeToString(body);
        byte[] decode = Base64.getDecoder().decode(encode);
        String s = new String(decode);
        log.info("获取监听数据：{}", s);
        if (!StringUtils.isEmpty(s)) {
            //处理JSON格式数据,防止保存到数据库blob字段时,出现多个转义符 \
            //去除所有转义符
            s =StringEscapeUtils.unescapeJava(s);
            //去除首尾双引号
            s= s.substring(1, s.length() - 1);
            JSONObject jsonObject = JSON.parseObject(s);
            if (jsonObject != null) {
                List<User> list = userService.list(Wrappers.<User>query().lambda().isNotNull(User::getUsername).isNotNull(User::getOrganId));
                List<Department> departments = departmentService.list(Wrappers.<Department>query().lambda().isNotNull(Department::getOrganId));
                Map<String, List<User>> departMap = new HashMap<>();
                Map<String, List<Department>> bmMap = new HashMap<>();
                if (null != list && list.size() > 0) {
                    departMap = list.stream().collect(Collectors.groupingBy(User::getUsername));
                }
                if (null != departments && departments.size() > 0) {
                    bmMap = departments.stream().collect(Collectors.groupingBy(Department::getOrganId));
                }
                if (jsonObject.get("dataArray") != null) {
                    JSONArray dataArray = jsonObject.getJSONArray("dataArray");
                    if (dataArray != null) {
                        JSONArray jsonArray = JSONArray.parseArray(dataArray.toString());
                        for (Iterator<Object> iterator = jsonArray.iterator(); iterator.hasNext(); ) {
                            JSONObject data = (JSONObject) iterator.next();
                            Object userId = data.get("userId");
                            Object organId = data.get("organId");
                            Object userName = data.get("userName");
                            Object mail = data.get("mail");
                            Object mobile = data.get("mobile");
                            Object userState = data.get("userState");
                            User user=new User();
                            user.setUsername(userId!=null?userId.toString():null);
                            user.setPassword(PasswordUtil.passwordEncode(intPassword));
                            user.setPhone(mobile!=null?mobile.toString():null);
                            user.setEmail(mail!=null?mail.toString():null);
                            user.setRealName(userName!=null?userName.toString():null);
                            user.setPwderrtimes((short) 0);
                            user.setUserState(userState!=null?userState.toString():null);
                            if(organId!=null){
                                String s1 = organId.toString();
                                user.setOrganId(s1);
                                if(bmMap.get(s1)!=null){
                                    List<Department> departments1 = bmMap.get(s1);
                                    if(departments1!=null && departments1.size()>0){
                                        user.setDepartmentId(departments1.get(0).getId());
                                    }
                                }
                            }
                            if (departMap.get(user.getUsername()) != null) {
                                user.setId(departMap.get(user.getUsername()).get(0).getId());
                                userService.updateById(user);
                            } else {
                                userService.save(user);
                            }
                        }
                    }
                }
            }
        }
    }



}
