/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.base.service.messge.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.base.entity.User;
import com.tofly.base.service.UserService;
import com.tofly.pojo.message.Usertask;
import com.tofly.base.mapper.UsertaskMapper;
import com.tofly.base.service.messge.UsertaskService;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.*;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.security.web.util.ThrowableAnalyzer;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.io.Serializable;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * 待办任务信息表
 *
 * @author admin
 * @date Wed Mar 31 00:00:00 CST 2021
 */
@Service("usertaskService")
@Slf4j
public class UsertaskServiceImpl extends ServiceImpl<UsertaskMapper, Usertask> implements UsertaskService {
    @Autowired
    private ThreadPoolTaskExecutor threadPoolTaskExecutor;
    @Autowired
    private UserService userService;

    @Override
    public Page<Usertask> getUserTaskPage(Page page, Usertask parDto) {
        // 按照时间范围
        QueryWrapper<Usertask> queryWrapper = new QueryWrapper();
        Date startTime = parDto.getStartTime();
        Date endTime = parDto.getEndTime();
        if (null != startTime && null != endTime) {
            queryWrapper.ge("START_TIME", startTime);
            queryWrapper.le("END_TIME", endTime);
            parDto.setStartTime(null);
            parDto.setEndTime(null);
        }
        queryWrapper.setEntity(parDto);
        // 待办事项按级别与任务派发时间降序排列
        queryWrapper.orderByDesc("TLEVEL").orderByDesc("CREATE_TIME");
        return this.page(page, queryWrapper);
    }

    @Override
    public boolean save(Usertask usertask) {
        super.save(usertask);

        pushDataTolc(usertask);

        //推送消息到
        return true;
    }

    @Override
    public boolean removeById(Serializable id) {

        pushDataDelTolC(id);
        return super.removeById(id);
    }

    private void pushDataTolc(Usertask usertask) {


        threadPoolTaskExecutor.execute(new Runnable() {
            @Override
            public void run() {
                //新增
                TaskAddDto dto = new TaskAddDto();
                dto.setTaskActivity("1");
                dto.setTaskName(usertask.getTknm());
                final User reveiveUser = userService.getById(usertask.getHandleUser());
                dto.setReceiveId(reveiveUser.getUsername());
                dto.setReceiveName(reveiveUser.getUsername());
                //处理状态：0待完成，1已完成,2未开始,3已超时
                //未开始（未开始）；待完成，已超时（正在进行）；已完成（已完成）
                String taskStatus = "1";

                switch (usertask.getIshandle()) {
                    case "0":
                    case "3":
                        taskStatus = "2";
                        break;
                    case "1":
                        taskStatus = "3";
                        break;
                    case "2":
                        taskStatus = "1";
                        break;
                }

                dto.setTaskStatus(taskStatus);
                //：0一般、1紧急
                dto.setTaskLevel("0".equals(usertask.getTlevel()) ? "3" : "1");

                dto.setTaskSource("tofly");

                dto.setStartTime(DateUtil.formatDateTime(usertask.getStartTime()));
                dto.setEndTime(DateUtil.formatDateTime(usertask.getEndTime()));
                final User createUser = userService.getById(usertask.getCreateUser());

                dto.setCreateName(createUser.getUsername());
                dto.setCreateId(String.valueOf(createUser.getUsername()));
                dto.setBatchId(String.valueOf(usertask.getId()));
                dto.setUrl(usertask.getRouteIp());
                String url = "http://172.16.0.16:30123/task-server/v1/task/pushListTask";
                RestTemplate restTemplate = new RestTemplate();
                // 设置请求头信息
                HttpHeaders headers = new HttpHeaders();
                headers.setContentType(MediaType.APPLICATION_JSON);
                dto.setAppKey("12ylhw8ykf9wltln3buwmm41");
                dto.setAppSecret("3j0kqonm7mobs7n0zwjsu36fr2p124rm");
                // 构建请求体数据
                String requestBody = JSON.toJSONString(dto);
                log.info("请求数据：" + requestBody);
                // 创建 HttpEntity 对象，包含请求头和请求体
                HttpEntity<String> requestEntity = new HttpEntity<>(requestBody, headers);
                ResponseEntity<String> responseEntity = restTemplate.exchange(url.toString(), HttpMethod.POST, requestEntity, String.class);
                final JSONObject jsonObject = JSON.parseObject(responseEntity.getBody());
                log.info("浪潮返回数据" + responseEntity.getBody());
                String data = (String) jsonObject.get("data");
                usertask.setLcId(data);
                baseMapper.updateById(usertask);
            }
        });


    }

    void pushDataDelTolC(Serializable id) {
        final Usertask byId = super.getById(id);

        final String lcId = byId.getLcId();
        if (!StringUtils.isNotBlank(lcId)) {

            return;
        }
        threadPoolTaskExecutor.execute(new Runnable() {
            @Override
            public void run() {
                //新增

                String url = "http://172.16.0.16:30123/task-server/v1/task/del/";
                url = url + lcId;
                RestTemplate restTemplate = new RestTemplate();
                // 设置请求头信息
                Map<String, String> paraMap = new HashMap<>();
                paraMap.put("AppKey", "12ylhw8ykf9wltln3buwmm41");
                paraMap.put("AppSecret", "3j0kqonm7mobs7n0zwjsu36fr2p124rm");

                // 创建 HttpEntity 对象，包含请求头和请求体
                //  HttpEntity<String> requestEntity = new HttpEntity<>(requestBody, headers);
                ResponseEntity<String> responseEntity = restTemplate.exchange(url.toString(), HttpMethod.DELETE, null, String.class, paraMap);
                final JSONObject jsonObject = JSON.parseObject(responseEntity.getBody());
                log.info("浪潮返回数据" + responseEntity.getBody());
                //删除数据推送
            }
        });
    }

    @Data
    class TaskAddDto {
        /**
         * 任务名称
         */
        private String taskName;
        /**
         * 任务状态（1、未开始，2：正在进行，3：已完成）
         * 默认状态1
         */
        private String taskStatus;
        /**
         * 紧急程度（紧急程度，1、高，2、中，3、低）
         */
        private String taskLevel;
        /**
         * 任务来源
         */
        private String taskSource;
        /**
         * 活动状态（1：收件箱，2：今天要做，3：下一步要做，4：以后要做）
         */
        private String taskActivity;
        /**
         * 开始时间
         */
        private String startTime;
        /**
         * 截止时间
         */
        private String endTime;
        /**
         * 创建人
         */
        private String createName;
        /**
         * 创建人id
         */
        private String createId;
        /**
         * 备注
         */
        private String remarks;
        /**
         * 紧急程度
         */
        private String taskDesc;
        /**
         * 负责人id(负责人userId)
         */
        private String receiveId;
        /**
         * 负责人名称
         */
        private String receiveName;
        /**
         * 审核界面url链接
         */
        private String url;
        /**
         * 任务批次id，同批次任务一个id
         */
        private String batchId;

        private String AppKey;
        private String AppSecret;

    }

    private Map<String, String> objectToMap(TaskAddDto taskAddDto) {
        Map<String, String> paraMap = new HashMap<>();
        paraMap.put("taskName", taskAddDto.getTaskName());
        /**
         *任务状态（1、未开始，2：正在进行，3：已完成）
         * 默认状态1
         */
        paraMap.put("taskStatus", taskAddDto.getTaskStatus());
        /**
         *紧急程度（紧急程度，1、高，2、中，3、低）
         */
        paraMap.put("taskLevel", taskAddDto.getTaskLevel());
        /**
         *任务来源
         */
        paraMap.put("taskSource", taskAddDto.getTaskSource());
        /**
         *活动状态（1：收件箱，2：今天要做，3：下一步要做，4：以后要做）
         */
        paraMap.put("taskActivity", taskAddDto.getTaskActivity());
        /**
         *开始时间
         */
        paraMap.put("startTime", taskAddDto.getStartTime());
        /**
         *截止时间
         */
        paraMap.put("endTime", taskAddDto.getEndTime());
        /**
         *创建人
         */
        paraMap.put("createName", taskAddDto.getCreateName());
        /**
         *创建人id
         */
        paraMap.put("createId", taskAddDto.getCreateId());
        /**
         *备注
         */
        paraMap.put("remarks", taskAddDto.getRemarks());
        /**
         *紧急程度
         */
        paraMap.put("taskDesc", taskAddDto.getTaskDesc());
        /**
         *负责人id(负责人userId)
         */
        paraMap.put("receiveId", taskAddDto.getReceiveId());
        /**
         *负责人名称
         */
        paraMap.put("receiveName", taskAddDto.getReceiveName());
        /**
         *审核界面url链接
         */
        paraMap.put("url", taskAddDto.getUrl());
        /**
         *任务批次id，同批次任务一个id
         */
        paraMap.put("batchId", taskAddDto.getBatchId());

        paraMap.put("AppKey", taskAddDto.getAppKey());
        paraMap.put("AppSecret", taskAddDto.getAppSecret());
        return paraMap;

    }
}
