package com.tofly.xrtygis.config;

import com.baomidou.mybatisplus.core.config.GlobalConfig;
import com.baomidou.mybatisplus.extension.spring.MybatisSqlSessionFactoryBean;
import com.tofly.mybatisplus.config.MapWrapperFactory;
import com.tofly.mybatisplus.config.MybatisKeyGenerator;
import com.tofly.mybatisplus.config.MybatisPlusConfigurer;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Lazy;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

@Configuration
@MapperScan(basePackages = {"com.tofly.*.pgWaterMapper"}, sqlSessionTemplateRef  = "thirdSqlSessionTemplate")
@Lazy
public class DruidConfigThird {

    @Autowired
    private MybatisPlusConfigurer mybatisPlusConfigurer;
    @Autowired
    private MybatisKeyGenerator mybatisKeyGenerator;

    @Bean(name = "thirdSqlSessionFactory")
    public SqlSessionFactory sqlSessionFactory(@Qualifier("thirdDataSource") DataSource dataSource) throws Exception {
        MybatisSqlSessionFactoryBean bean = new MybatisSqlSessionFactoryBean();
        bean.setDataSource(dataSource);

        bean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:pgWaterMapper/*.xml"));
        bean.setPlugins(mybatisPlusConfigurer.paginationInterceptor());
        bean.setDatabaseIdProvider(mybatisPlusConfigurer.databaseIdProvider());
        bean.setObjectWrapperFactory(new MapWrapperFactory());
        //设置全局配置
        GlobalConfig config = mybatisPlusConfigurer.globalConfig();
        config.setDbConfig(new GlobalConfig.DbConfig().setKeyGenerator(mybatisKeyGenerator));
        bean.setGlobalConfig(config);
        return bean.getObject();
    }

    @Bean(name = "thirdTransactionManager")
    public DataSourceTransactionManager transactionManager(@Qualifier("thirdDataSource") DataSource dataSource) {
        return new DataSourceTransactionManager(dataSource);
    }

    @Bean(name = "thirdSqlSessionTemplate")
    public SqlSessionTemplate sqlSessionTemplate(@Qualifier("thirdSqlSessionFactory") SqlSessionFactory sqlSessionFactory) {
        return new SqlSessionTemplate(sqlSessionFactory);
    }
}
