/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtygis.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtygis.entity.Poll;
import com.tofly.xrtygis.service.PollService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
/**
 * 污染物信息表
 *
 * @author admin
 * @date Wed Jun 23 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/poll")
@Api(tags="污染物信息表接口")
public class PollController {

    private final  PollService pollService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param poll 污染物信息表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPollPage(Page page, @ModelAttribute Poll poll) {
      return ResultRespone.success(pollService.page(page, Wrappers.query(poll)));
    }


    /**
     * 通过id查询污染物信息表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(pollService.getById(id));
    }

    /**
     * 新增污染物信息表
     * @param poll 污染物信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增污染物信息表" )
    @ApiOperation(value = "新增污染物信息表")
    @PostMapping
    public ResultRespone save(@RequestBody Poll poll) {
      return ResultRespone.success(pollService.save(poll));
    }

    /**
     * 修改污染物信息表
     * @param poll 污染物信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改污染物信息表" )
    @ApiOperation(value = "修改污染物信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Poll poll) {
      return ResultRespone.success(pollService.updateById(poll));
    }

    /**
     * 通过id删除污染物信息表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除污染物信息表" )
    @ApiOperation(value = "通过ID删除污染物信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(pollService.removeById(id));
    }

    /**
     * 通过id批量删除污染物信息表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除污染物信息表" )
    @ApiOperation(value = "通过ID批量删除污染物信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(pollService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
