/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtygis.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtygis.entity.Waterparams;
import com.tofly.xrtygis.service.WaterparamsService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

/**
 * 水质参数配置信息表
 *
 * @author admin
 * @date Wed May 26 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/waterparams")
@Api(tags = "水质参数配置信息表接口")
public class WaterparamsController {

    private final WaterparamsService waterparamsService;

    /**
     * 分页查询
     *
     * @param page        分页对象
     * @param waterparams 水质参数配置信息表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getWaterparamsPage(Page page, @ModelAttribute Waterparams waterparams) {
        return ResultRespone.success(waterparamsService.page(page, Wrappers.query(waterparams)));
    }

    @GetMapping("/getAllInfo")
    @ApiOperation(value = "查询全部信息")
    public List<Waterparams> getAllInfo(){
        return waterparamsService.list();
    }

    /**
     * 通过id查询水质参数配置信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(waterparamsService.getById(id));
    }

    /**
     * 新增水质参数配置信息表
     *
     * @param waterparams 水质参数配置信息表
     * @return ResultRespone
     */
    //(title = "新增水质参数配置信息表")
    @ApiOperation(value = "新增水质参数配置信息表")
    @PostMapping
    public ResultRespone save(@RequestBody Waterparams waterparams) {
        return ResultRespone.success(waterparamsService.save(waterparams));
    }

    /**
     * 修改水质参数配置信息表
     *
     * @param waterparams 水质参数配置信息表
     * @return ResultRespone
     */
    //(title = "修改水质参数配置信息表")
    @ApiOperation(value = "修改水质参数配置信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Waterparams waterparams) {
        return ResultRespone.success(waterparamsService.updateById(waterparams));
    }

    /**
     * 通过id删除水质参数配置信息表
     *
     * @param id id
     * @return ResultRespone
     */
    //(title = "通过id删除水质参数配置信息表")
    @ApiOperation(value = "通过ID删除水质参数配置信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(waterparamsService.removeById(id));
    }

    /**
     * 通过id批量删除水质参数配置信息表
     *
     * @param ids id
     * @return ResultRespone
     */
    //(title = "通过id批量删除水质参数配置信息表")
    @ApiOperation(value = "通过ID批量删除水质参数配置信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(waterparamsService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
