package com.tofly.xrtygis.custom.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.api.es.EsClient;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.DateUtil;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.es.GpsUpload;
import com.tofly.es.dto.EsQueryDto;
import com.tofly.xrtygis.callCenterMapper.PltCalllist1000Mapper;
import com.tofly.xrtygis.config.WaterFeign;
import com.tofly.xrtygis.custom.entity.WaterHistory;
import com.tofly.xrtygis.custom.entity.dto.*;
import com.tofly.xrtygis.custom.entity.marketing.CustomerDto;
import com.tofly.xrtygis.custom.entity.marketing.CustomerInfo;
import com.tofly.xrtygis.custom.entity.marketing.WaterVolume;
import com.tofly.xrtygis.custom.entity.scada.StatisticsScada;
import com.tofly.xrtygis.custom.entity.vo.*;
import com.tofly.xrtygis.custom.mapper.CustomDisplayMapper;
import com.tofly.xrtygis.custom.service.CustomDisplayService;
import com.tofly.xrtygis.custom.service.TheOnlineCheckService;
import com.tofly.xrtygis.custom.service.export.ExportCommonCodeService;
import com.tofly.xrtygis.custom.util.CallAPIUtil;
import com.tofly.xrtygis.custom.util.Date7Utils;
import com.tofly.xrtygis.em.Constant;
import com.tofly.xrtygis.em.NumberChangeEnum;
import com.tofly.xrtygis.entity.*;
import com.tofly.xrtygis.entity.vo.PipeMaterialStatisticalVo;
import com.tofly.xrtygis.pgWaterMapper.PgWaterMapper;
import com.tofly.xrtygis.service.*;
import com.tofly.xrtygis.util.CalendarHelper;
import lombok.AllArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.awt.geom.Point2D;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

import static com.tofly.xrtygis.custom.util.CommonMethod.listToPage;

/**
 * 自定义智慧综合展示相关
 *
 * @author 陈浩
 * @date 2021-3-25
 */
@Service
@AllArgsConstructor
public class CustomDisplayServiceImpl implements CustomDisplayService {

    private final ScadastaitemService scadastaitemService;
    private final LayerfieldService layerfieldService;
    private final CustomDisplayMapper customDisplayMapper;
    private final VideoconfigService videoconfigService;
    private final ScadastationService scadastationService;
    private final WatermeterService watermeterService;
    private final WorkratoitemService workratoitemService;
    private final PipelinktableService pipelinktableService;
    private final WaterTrendService waterTrendService;
    private final DensityconfigService densityconfigService;
    private final IndicatorCorrectionService indicatorCorrectionService;
    private final LeakAssessmentService leakAssessmentService;
    private final DmaregionService dmaregionService;
    private final TfDeviceArchiveManageService tfDeviceArchiveManageService;
    private final ExportCommonCodeService exportCommonCodeService;
    private final TheOnlineCheckService theOnlineCheckService;
    private final WarnService warnService;
    private final TfDeviceArchiveManageService deviceArchiveManageService;
    private final TfIndexAllocationService indexAllocationService;
    private final TfWaterInfoService waterInfoService;
    private final EsClient esClient;
    private final PltCalllist1000Mapper pltCalllist1000Mapper;
    private final PgWaterMapper pgWaterMapper;

    private final WaterFeign waterFeign;


    /**
     * 添加动态标识
     *
     * @param layerfield 动态标识
     * @return true or false
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertDynamicLogo(Layerfield layerfield) {
        Layerfield layer = layerfieldService.getOne(Wrappers.<Layerfield>lambdaQuery()
                .eq(Layerfield::getLayerEn, layerfield.getLayerEn())
                .eq(Layerfield::getCreateUser, SecurityUtils.getUserId()));

        layerfield.setCreateTime(new Date());

        if (layer != null) {
            layerfield.setId(layer.getId());
            layerfieldService.updateById(layerfield);
        } else {
            layerfieldService.save(layerfield);
        }
        return true;
    }

    /**
     * 删除动态标注
     *
     * @param id 动态标识id
     * @return true of false
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean deleteDynamicLogo(Long id) {
        return layerfieldService.removeById(id);
    }

    /**
     * 查看动态标注
     *
     * @param page page
     * @return page
     */
    @Override
    public Page viewDynamicLogo(Page page) {
        Long userId = SecurityUtils.getUserId();
        return layerfieldService.page(page, Wrappers.<Layerfield>lambdaQuery()
                .eq(Layerfield::getCreateUser, SecurityUtils.getUserId()));
    }

    /**
     * 获取多个测站指标
     *
     * @param stids 测站id集合
     * @return map
     */
    @Override
    public Map<Long, Map<Long, Object>> getSiteInfos(String stids) {
        if (stids.isEmpty()) {
            throw new ToflyDeniedException("请传入测站");
        }

        Map<Long, Map<Long, Object>> infoMap = new HashMap<>();

        //查询所传测站下的指标编号
        List<Scadastaitem> scadastaitems = scadastaitemService.list(Wrappers.<Scadastaitem>lambdaQuery()
                .eq(Scadastaitem::getState, 1)
                .select(Scadastaitem::getId, Scadastaitem::getItcd,Scadastaitem::getStationId)
                .in(Scadastaitem::getStationId, Arrays.asList(stids.split(","))));
        //根据测站的id对数据进行分组
        Map<Long, List<Scadastaitem>> collect = scadastaitems.stream().collect(Collectors.groupingBy(Scadastaitem::getStationId));

        List<String> itcdList = scadastaitems.stream().map(Scadastaitem::getItcd).collect(Collectors.toList());
        //将查询出来的指标数据按编号分组
        Map<String, List<IndexInfoDto>> realVal = customDisplayMapper.getRealtime(itcdList)
                .stream().collect(Collectors.groupingBy(IndexInfoDto::getItcd));
        Map<String, List<IndexInfoDto>> hisVal = customDisplayMapper.getMaxHistoryVal(itcdList)
                .stream().collect(Collectors.groupingBy(IndexInfoDto::getItcd));

        collect.forEach((k,l) ->{
            Map<Long, Object> itemMap = new HashMap<>();

            //遍历编号，查出对应实时指标
            for (Scadastaitem item : l) {

                RealValDto dto = new RealValDto();
                if (realVal.containsKey(item.getItcd())) {
                    Double real = realVal.get(item.getItcd()).get(0).getItval();
                    dto.setRealVal(real);
                    if (hisVal.containsKey(item.getItcd())) {
                        String mark;
                        Double his = hisVal.get(item.getItcd()).get(0).getItval();
                        if (his > real) {
                            mark = "0";
                        } else if (his < real) {
                            mark = "2";
                        } else {
                            mark = "1";
                        }
                        dto.setMark(mark);
                    } else {
                        dto.setMark("2");
                    }
                    itemMap.put(item.getId(), dto);
                } else {
                    itemMap.put(item.getId(), null);
                }
            }

            infoMap.put(k, itemMap);
        });

        return infoMap;
    }

    /**
     * 进取水趋势
     *
     * @param page      page
     * @param scid      测站id
     * @param code      0：按日查看  1：按月查看 2：同期比对
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return page
     */
    @Override
    public Page waterVolumeTrend(Page page, Long scid, String code, String startTime, String endTime) {

        Page<Map> resultPage = null;

        switch (code) {
            case "0":
                resultPage = customDisplayMapper.waterByHour(page, scid, startTime, endTime);
                break;
            case "1":
                resultPage = customDisplayMapper.waterByDay(page, scid, startTime, endTime);
                break;
            case "2":
                resultPage = customDisplayMapper.waterByMonth(page, scid, startTime, endTime);
                break;
            case "3":
                List<String> list = new ArrayList<>();
                for (String year : startTime.split(",")) {
                    for (String month : endTime.split(",")) {
                        list.add(year + month);
                    }
                }
                resultPage = customDisplayMapper.waterContrast(page, scid, list);
                break;
            default:
                throw new ToflyDeniedException("CODE ERROR");
        }
        return resultPage;
    }

    /**
     * 查询视频监控的分组名
     */
    @Override
    public List getGroupName() {
        return videoconfigService.list(Wrappers.<Videoconfig>lambdaQuery().select(Videoconfig::getGroupName));
    }

    /**
     * 获取视频监控信息
     *
     * @return list
     */
    @Override
    public List getAllVideo() {
        return videoconfigService.list();
    }

    /**
     * 模糊查询视频地址
     *
     * @param add 地址
     * @return list
     */
    @Override
    public List likeVideo(String add) {

        List<Videoconfig> list = videoconfigService.list(Wrappers.<Videoconfig>lambdaQuery()
                .like(Videoconfig::getAddress, add)
                .orderByAsc(Videoconfig::getAddress));

        if (list.isEmpty()) {
            throw new ToflyDeniedException("未找到相关信息");
        }

        return list;
    }

    /**
     * 修该监控视频大屏显示
     *
     * @param videoId 视频id
     * @return true of false
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateVideo(String videoId) {
        if (videoId == null || "".equals(videoId)) {
            throw new ToflyDeniedException("请选择监控视频");
        }
        List<String> videoIdList = Arrays.asList(videoId.trim().split(","));

        List<Videoconfig> videoconfigList = videoconfigService.list(Wrappers.<Videoconfig>lambdaQuery()
                .select(Videoconfig::getId, Videoconfig::getIsshow, Videoconfig::getAddress)
                .in(Videoconfig::getId, videoIdList));

        videoconfigList.forEach(video -> {
            video.setIsshow("1");
            videoconfigService.updateById(video);
        });

        return true;
    }

    /**
     * 停水区域查询
     *
     * @param valve 阀门编号
     * @return
     */
    @Override
    public Page waterStopArea(Page page, String valve) {
        return waterFeign.viewWaterCut(page, valve);
    }

    /**
     * 营销信息-获取客户用水量
     *
     * @param page        page
     * @param customerNum 客户编码
     * @return list
     */
    @Override
    public Page getUseWater(Page page, String customerNum) {
        return waterFeign.viewUseWater(customerNum);
    }

    /**
     * 营销信息-查看客户历史用水量
     *
     * @param page    page
     * @param khbh    客户编号
     * @param strTime 开始时间
     * @param endTime 结束时间
     * @return page
     */
    @Override
    public Page getUserHistroy(Page page, String khbh, String strTime, String endTime) {
        return waterFeign.useWaterHistory(page, khbh, strTime, endTime);
    }

    /**
     * 营销信息-客户用水量监控
     *
     * @param page
     * @return page
     */
    @Override
    public Page waterMonitoring(Page page) {
        return waterFeign.waterMonitoring(page);
    }

    /**
     * @param page    page
     * @param lon     区域各顶点的横坐标数组
     * @param lat     区域各顶点的纵坐标数组
     * @param strTime 开始时间
     * @param endTime 结束时间
     * @param code    0:查看全部   1：同期比对
     * @return map
     */
    @Override
    public Map<String, Object> getTableInArea(Page page, Double[] lon, Double[] lat, String strTime, String endTime, String code) {
        if (lon == null || lat == null) {
            throw new ToflyDeniedException("区域范围不能为空");
        }
        //查询所有表位过滤坐标为空的数据
        List<Watermeter> watermeterList = watermeterService.list().stream().filter(wm -> {
            return wm.getX() != null && wm.getY() != null;
        }).collect(Collectors.toList());

        if (watermeterList.isEmpty()) {
            throw new ToflyDeniedException("请先上传水表坐标");
        }

        //存放返回结果
        Map<String, Object> result = new HashMap<>();
        //存放点坐标的集合
        List<Point2D.Double> pointList = new ArrayList<>();

        for (int i = 0; i < lon.length; i++) {
            pointList.add(new Point2D.Double(lon[i], lat[i]));
        }
        //循环有坐标的水表
        Iterator<Watermeter> it = watermeterList.iterator();


        Point2D.Double point;
        //在范围内添加到一个新的集合，否则删除
        while (it.hasNext()) {
            Watermeter next = it.next();
            point = new Point2D.Double(next.getX(), next.getY());
            if (!IsPtInPoly(point, pointList)) {
                it.remove();
            }
        }
        //取出水表编号
        List<String> sid = watermeterList.stream().map(Watermeter::getSid).collect(Collectors.toList());

        if (sid.isEmpty()) {
            throw new ToflyDeniedException("未查询到相关信息");
        }

        String sidStr = sid.toString();
        String substring = sidStr.substring(1, sidStr.lastIndexOf("]"));

        WaterVolume waterVolume = new WaterVolume();
        waterVolume.setSBBH(substring);
        //传入时间为空区间设置为三年前
        if (strTime != null || endTime != null) {
            waterVolume.setCWYF_START(strTime);
            waterVolume.setCWYF_END(endTime);
        } else {
            waterVolume.setCWYF_START(DateUtil.getDateFormat(Date7Utils.addNextPeriod(-36, new Date()), "yyyyMM"));
            waterVolume.setCWYF_END(Date8Util.getYearMonth());
        }

        waterVolume.setACTION("query");

        Map map = null;

        try {
            //查询区域类所有水表信息
            map = CallAPIUtil.callAPIUtil(waterVolume, "api/Business/GetCustomerWater");
        } catch (Exception e) {
            e.printStackTrace();
        }

//        if (null != map.get("data")) {
//            List<CustomerWaterVo> data = JSONObject.parseArray(map.get("data").toString(), CustomerWaterVo.class);
//            //根据月份分组
//            Map<String, List<CustomerWaterVo>> collect = data.stream().collect(Collectors.groupingBy(CustomerWaterVo::getCWYF));
//            //拿出月份
//            List<String> cwyfList = new ArrayList(collect.keySet());
//            //存放区域水量
//            Map<String, Object> mapSum = new HashMap<>();
//
//            List pageList = new ArrayList();
//
//            for (String month : cwyfList) {
//                double sum = 0;
//                if ("0".equals(code)) {
//                    //计算截取区域的用水总和
//                    sum = collect.get(month).stream().mapToDouble(CustomerWaterVo::getYSL).sum();
//                } else {
//                    String mon = strTime.substring(4, 6);
//                    if (mon.equals(month.substring(4, 6))) {
//                        sum = collect.get(month).stream().mapToDouble(CustomerWaterVo::getYSL).sum();
//                    }
//                }
//                mapSum.put(month, sum);
//            }
//            //获取坐标
//            List<Map> coordinate = customDisplayMapper.getCoordinate(sid);
//
//            pageList.add(mapSum);
//
//            result.put("areaSum", listToPage(page, pageList));
//            result.put("coordinate", coordinate);
//        }
        return result;
    }

    /**
     * 通过管线编号查询客户信息
     *
     * @param page  page
     * @param pipes 管线编号
     * @return page
     */
    @Override
    public Page pipeToCustomers(Page page, String pipes) {

        String[] pipeArr = pipes.split(",");
        if (pipeArr.length == 0) {
            throw new ToflyDeniedException("请重新选择管线编号");
        }
        Map<String, Object> customer = new HashMap();
        for (String pipe : pipeArr) {
            Pipelinktable pipetable = pipelinktableService.getById(pipe);
            //关联表为空或者客户编号为空时返回空
            if (null == pipetable || null == pipetable.getTableId()) {
                customer.put(pipe, null);
                continue;
            }

            CustomerDto customerDto = new CustomerDto();
            customerDto.setYHBH(pipetable.getTableId());
            customerDto.setACTION("query");

            Map map = new HashMap();
            try {
                map = CallAPIUtil.callAPIUtil(customer, "api/Business/GetCustomerData");
            } catch (Exception e) {
                e.printStackTrace();
            }

            List<CustomerInfo> infos = JSONObject.parseArray(map.get("data").toString(), CustomerInfo.class);

            if (null == infos || infos.size() == 0) {
                customer.put(pipe, null);
                continue;
            }
            for (CustomerInfo info : infos) {
                Map<String, Double> coordinateInfo = customDisplayMapper.getCoordinateInfo(info.getSBBH());
                if (!coordinateInfo.isEmpty()) {
                    info.setX(coordinateInfo.get("x"));
                    info.setY(coordinateInfo.get("y"));
                }
            }
            customer.put(pipe, infos);
        }
        List<Map> pageList = new ArrayList<>();
        pageList.add(customer);
        return listToPage(page, pageList);
    }

    /**
     * 多因子关系成本图
     *
     * @param dto
     * @return
     */
    @Override
    public Map<String, Object> relationshipCost(RelationshipCostDto dto) {
        if (dto == null) {
            throw new ToflyDeniedException("数据不能为空");
        }

        if (dto.getCode() == null) {
            throw new ToflyDeniedException("范围不能为空");
        }
        Densityconfig config = densityconfigService.getById(1);

        if (null == config) {
            throw new ToflyDeniedException("请先配置浓度信息");
        }
        Map<String, Object> result = new HashMap<>();

        //原水数据
        if (null != dto.getPrimary()) {
            List<Map<String, Object>> primaryInfo = customDisplayMapper.getPrimaryInfo(Arrays.asList(dto.getPrimary().split(",")), dto.getStart(), dto.getEnd());
            result.put("primary", groupByData(primaryInfo, "createTime"));
        }
        //过程水数据
        if (null != dto.getProcess()) {
            List<Map<String, Object>> primaryInfo = customDisplayMapper.getPrimaryInfo(Arrays.asList(dto.getProcess().split(",")), dto.getStart(), dto.getEnd());
            result.put("process", groupByData(primaryInfo, "createTime"));
        }
        Map<String, List<Map<String, Object>>> map = new HashMap<>();
        Map<String, List> paramMap = new HashMap<>();
        //pac单耗
        if (null != dto.getPacOne() && null != config.getPacNd() && null != dto.getQslOne()) {
            paramMap.put("itcd1", Arrays.asList(dto.getPacOne().split(",")));
            paramMap.put("itcd2", Arrays.asList(dto.getQslOne().split(",")));
            map.put("pacOne", customDisplayMapper.getPacDensity(paramMap, dto.getStart(), dto.getEnd(), config.getPacNd()));
        }
        //pam单耗
        if (null != dto.getPamTwo() && null != config.getPamNd() && null != dto.getQslTwo()) {
            paramMap.put("itcd1", Arrays.asList(dto.getPamTwo().split(",")));
            paramMap.put("itcd2", Arrays.asList(dto.getQslTwo().split(",")));
            map.put("pamTwo", customDisplayMapper.getPamDensity(paramMap, dto.getStart(), dto.getEnd(), config.getPamNd()));
        }
        //高锰酸钾单耗
        if (null != dto.getGmsj() && null != config.getGmsjNd() && null != dto.getQslTwo()) {
            paramMap.put("itcd1", Arrays.asList(dto.getGmsj().split(",")));
            paramMap.put("itcd2", Arrays.asList(dto.getQslTwo().split(",")));
            map.put("gmsjTwo", customDisplayMapper.getGmsjDensity(paramMap, dto.getStart(), dto.getEnd(), config.getGmsjNd()));
        }
        //液氯单耗
        if (null != dto.getYl() && null != dto.getQslTwo()) {
            paramMap.put("itcd1", Arrays.asList(dto.getYl().split(",")));
            paramMap.put("itcd2", Arrays.asList(dto.getQslTwo().split(",")));
            map.put("yl", customDisplayMapper.getYlDensity(paramMap, dto.getStart(), dto.getEnd()));
        }
        if (null != dto.getPacTwo()) {
            map.put("pacTwo", customDisplayMapper.getPacTwo(Arrays.asList(dto.getPacTwo().split(",")), dto.getStart(), dto.getEnd()));
        }

        result.put("unit", map.size() > 0 ? handleUnitMap(map) : map);
        if (dto.getParam() != null) {
            String sql = ",round((ifnull(PACZFY_ONE,0) + ifnull(PACZFY_TWO,0)) / decode(GSL_TOTAL,0,null,GSL_TOTAL) * 1000,5)    paccb_total";
            String param = dto.getParam();
            if ("2".equals(dto.getCode())) {
                param += sql;
            }
            List<Map<String, Object>> costAndWater = customDisplayMapper.getCostAndWater(param, dto.getStart(), dto.getEnd());
            result.put("costAndWater", costAndWater != null ? costAndWater.stream().collect(Collectors.groupingBy(m -> m.get("upDate"))) : null);
        }
        List<Map<String, Object>> waterVolume = customDisplayMapper.getWaterVolume(dto.getCode(), dto.getStart(), dto.getEnd());
        result.put("waterVloum", waterVolume != null ? waterVolume.stream().collect(Collectors.groupingBy(m -> m.get("upDate"))) : null);

        return result;
    }

    /**
     * 多因子浓度配置
     *
     * @param config config
     * @return boolean
     */
    @Override
    public boolean concentrationConfig(Densityconfig config) {
        if (null == config) {
            throw new ToflyDeniedException("数据不允许为空");
        }
        if (null != densityconfigService.getById(1)) {
            densityconfigService.update(Wrappers.<Densityconfig>lambdaUpdate()
                    .eq(Densityconfig::getId, 1)
                    .set(Densityconfig::getGmsjNd, config.getGmsjNd())
                    .set(Densityconfig::getPacNd, config.getPacNd())
                    .set(Densityconfig::getPamNd, config.getPamNd()));
        } else {
            densityconfigService.save(config);
        }
        return true;
    }

    /**
     * 查看配置信息
     *
     * @return Densityconfig
     */
    @Override
    public Densityconfig viewConfig() {
        return densityconfigService.getById(1);
    }

    /**
     * 水表关联查询
     *
     * @param page page
     * @param khbh 客户编号
     * @return page
     */
    @Override
    public Page<Map> viewTable(Page page, String khbh, String khxm, String khdz) {
        return waterFeign.viewTable(page, khbh, khxm, khdz);
    }

    /**
     * 获得编码表详细消息
     *
     * @param pcid pcid
     * @return list
     */
    @Override
    public List<Map<String, Object>> getCodeInfo(String pcid) {
        return customDisplayMapper.getCodeInfo(pcid);
    }

    @Override
    public Page<Dmausertable> selectDmausertablePage(Page page, CustomDmausertableDto dmausertableDto) {
        Page<Dmausertable> dmausertablePage = customDisplayMapper.selectDmausertablePage(page, dmausertableDto);
        List<Dmausertable> dmausertableList = dmausertablePage.getRecords();
        List<Dmausertable> dmausertables = dmausertableList.stream()
                .filter(e-> Arrays.asList(e.getDmaIds().split(",")).contains(String.valueOf(dmausertableDto.getDmaId())))
                .collect(Collectors.toList());
        dmausertablePage.setRecords(dmausertables);
        return dmausertablePage;
//        return customDisplayMapper.selectDmausertablePage(page, dmausertableDto);
    }

    @Override
    public Page<WaterHistory> useWaterHistory(Page page, String yhbh, String startTime, String endTime) {
        if (yhbh == null){
            throw new ToflyDeniedException("用户编号不允许为空");
        }
        if (startTime == null || endTime == null){
            throw new ToflyDeniedException("时间不允许为空");
        }

        return customDisplayMapper.useWaterHistory(page, yhbh, startTime, endTime);
    }

    @Override
    public DmaQueryVo getDmaTree(String dmaName) {
        //查询父分区列表
        List<QueryVo> mainDmaList = customDisplayMapper.getAllMainDma(dmaName);
        DmaQueryVo dmaQueryVo = new DmaQueryVo();
        dmaQueryVo.setQueryVoList(getDmaChild(mainDmaList, dmaName));
        dmaQueryVo.setDmaCountList(customDisplayMapper.getDmaAllLevelCount());
        return dmaQueryVo;
    }

    @Override
    public List<Map<String,Object>> getDmaLevel() {
        List<Map<String,Object>> dmaLevelNameList = new ArrayList<>();
        List<String> dmaLevelList = customDisplayMapper.getDmaLevel();
        for(String dmaLevel : dmaLevelList){
            Map<String, Object> levelMap = new HashMap<>();
            if(StringUtils.isNotBlank(dmaLevel)){
                dmaLevel = dmaLevel.trim();
//                List<Dmaregion> dmaregionList = dmaregionService.list(new QueryWrapper<Dmaregion>()
//                        .eq("PARTITION_LEVEL",Integer.parseInt(dmaLevel)));
                levelMap.put("dmaregionList",customDisplayMapper.getDmaName(Integer.parseInt(dmaLevel)));
                levelMap.put("dmaLevelName",NumberChangeEnum.getCodeForValue(dmaLevel) + "级分区");
                levelMap.put("dmaLevel",dmaLevel);
            }
            dmaLevelNameList.add(levelMap);
        }
        return dmaLevelNameList;
    }

    @Override
    public List<Map<String, Object>> getDmaLevelAmount() {
        return customDisplayMapper.getDmaAllLevelCount();
    }

    /**
     * 递归子分区
     * @param dmaList 分区列表
     * @return
     */
    public List<QueryVo> getDmaChild(List<QueryVo> dmaList, String dmaName) {
        for(QueryVo dma:dmaList){
            List<QueryVo> dmaChildList = customDisplayMapper.getDmaChild(Long.parseLong(dma.getId()), dmaName);
            if(CollectionUtils.isNotEmpty(dmaChildList)){
                dma.setChildren(getDmaChild(dmaChildList, dmaName));
            }
            dma.setViewId((dma.getLabel() + "_" + dma.getId()));
        }
        return dmaList;
    }

    @Override
    public DmaAllIndexVo getDmaAllIndex(Long id) {
        if(id == 0){
            return null;
        }

        DmaAllIndexVo dmaAllIndexVo = new DmaAllIndexVo();
        String partitionLevel = null;

        Dmaregion dmaregion = dmaregionService.getById(id);
        if(dmaregion != null){
            dmaAllIndexVo.setId(dmaregion.getId());
            dmaAllIndexVo.setDmaName(dmaregion.getDmaName());
            dmaAllIndexVo.setRefStandard(dmaregion.getRefStandard());
            dmaAllIndexVo.setMonitorCount(dmaregion.getPointNumber());
            partitionLevel = String.valueOf(dmaregion.getPartitionLevel());
        }

        QueryAllScadaDto dto = new QueryAllScadaDto();
        dto.setDmaId(id);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        dto.setToDayTime(sdf.format(new Date()));

        dto.setModel("2");

        //计算入口压力
        dto.setDataType("0");
        List<StatisticsScada> inPressList = customDisplayMapper.getAllScadaIndex(dto);
        if(inPressList.size()!=0){
            dmaAllIndexVo.setInPress(inPressList.get(0).getAve());
        }else {
            dmaAllIndexVo.setInPress(0.0000);
        }

        //计算入口流量
        dto.setDataType("1");
        List<StatisticsScada> inFlowList = customDisplayMapper.getAllScadaIndex(dto);
        if(inFlowList.size()!=0){
            dmaAllIndexVo.setInFlow(inFlowList.get(0).getAve());
        }else {
            dmaAllIndexVo.setInFlow(0.0000);
        }

        dto.setModel("1");

        //计算出口压力
        dto.setDataType("0");
        List<StatisticsScada> outPressList = customDisplayMapper.getAllScadaIndex(dto);
        if(outPressList.size()!=0){
            dmaAllIndexVo.setOutPress(outPressList.get(0).getAve());
        }else {
            dmaAllIndexVo.setOutPress(0.0000);
        }

        //计算出口流量
        dto.setDataType("1");
        List<StatisticsScada> outFlowList = customDisplayMapper.getAllScadaIndex(dto);
        if(outFlowList.size()!=0){
            dmaAllIndexVo.setOutFlow(outFlowList.get(0).getAve());
        }else {
            dmaAllIndexVo.setOutFlow(0.0000);
        }

        //计算上月供水量上月漏损率漏点数量
        Map<String,Object> lastMonthStatistical = customDisplayMapper.getLastMonthStatistical(dto.getDmaId());
        if(lastMonthStatistical != null){
            dmaAllIndexVo.setLastMonthWater(new BigDecimal(String.valueOf(lastMonthStatistical.get("waterSupply")).trim()).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
            dmaAllIndexVo.setLastMonthUserWater(new BigDecimal(String.valueOf(lastMonthStatistical.get("userWater")).trim()).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
            dmaAllIndexVo.setLastMonthLeakageRate(Float.parseFloat(String.valueOf(lastMonthStatistical.get("leakageRate")).trim()));
            dmaAllIndexVo.setLeakCount((int) Double.parseDouble(String.valueOf(lastMonthStatistical.get("leakageVal")).trim()));
        }else {
            dmaAllIndexVo.setLastMonthWater(BigDecimal.ZERO);
            dmaAllIndexVo.setLastMonthUserWater(BigDecimal.ZERO);
            dmaAllIndexVo.setLastMonthLeakageRate(0.00f);
            dmaAllIndexVo.setLeakCount(0);
        }

        //获取分区漏损评估
        Float leakageRate = dmaAllIndexVo.getLastMonthLeakageRate();
        if(leakageRate != null){
            LeakAssessment leakAssessment = leakAssessmentService.getOne(new QueryWrapper<LeakAssessment>()
                    .eq(partitionLevel != null,"DMA_LEVLE",partitionLevel)
                    .ge("LEAK_UP",leakageRate)
                    .le("LEAK_DOWN",leakageRate));
            if(leakAssessment != null){
                dmaAllIndexVo.setDmaLeakageAssess(leakAssessment.getLeakAssessmentName());
            }
        }

        //获取分区户数
        //dmaAllIndexVo.setUserNum(customDisplayMapper.getUserByDmaId(id));

        return dmaAllIndexVo;
    }

    @Override
    public List<StatisticsScada> getDmaFlowPress(String dataType, Long dmaId, String startTime, String endTime) {
        return customDisplayMapper.getDmaFlowPress(dataType, dmaId, startTime, endTime);
    }

//    @Override
//    public List<StatisticsScada> getDmaPress(Long dmaId, String startTime, String endTime) {
//        return customDisplayMapper.getDmaPress(dmaId, startTime, endTime);
//    }

    @Override
    public List<Map<String, Object>> getDmaCensusParam(Long dmaId, String startTime, String endTime) {
        return customDisplayMapper.getDmaCensusParam(dmaId, startTime, endTime);
    }

    @Override
    public boolean exportReportForm(HttpServletResponse response, Page page, WisdomDisplayDto wisdomDisplayDto) {
        List<Map<String,Object>> reportFormList = null;
        String type = wisdomDisplayDto.getType();
        if(StringUtils.isBlank(type)){
            throw new ToflyDeniedException("类型不能为空");
        }
        switch (type) {
            case "1":
                reportFormList = customDisplayMapper.getYearReportForm(page, wisdomDisplayDto).getRecords();
                break;
            case "2":
                reportFormList = customDisplayMapper.getQuarterReportForm(page, wisdomDisplayDto).getRecords();
                break;
            case "3":
                reportFormList = customDisplayMapper.getMonthReportForm(page, wisdomDisplayDto).getRecords();
                break;
        }
        if(reportFormList != null){
            exportCommonCodeService.exportCode(response, "exportWaterReportFormService", reportFormList);
        }
        return true;
    }

    @Override
    public Page<Map<String, Object>> getYearReportForm(Page page, WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getYearReportForm(page, wisdomDisplayDto);
    }

    @Override
    public Page<Map<String, Object>> getQuarterReportForm(Page page, WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getQuarterReportForm(page, wisdomDisplayDto);
    }

    @Override
    public Page<Map<String, Object>> getMonthReportForm(Page page, WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getMonthReportForm(page, wisdomDisplayDto);
    }

    @Override
    public Page<Map<String, Object>> getLeakageRate(Page page, WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getLeakageRate(page, wisdomDisplayDto);
    }

    @Override
    public List<Map<String, Object>> getLeakageRateCensus(WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getLeakageRateCensus(wisdomDisplayDto);
    }

    @Override
    public List<Map<String,Object>> getLeakRateStatus(WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getLeakRateStatus(wisdomDisplayDto);
    }

    @Override
    public Page<Map<String, Object>> getNrw(Page page, WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getNrw(page, wisdomDisplayDto);
    }

    @Override
    public List<Map<String,Object>> getNrwStatus(WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.getNrwStatus(wisdomDisplayDto);
    }

    @Override
    public Page<Map<String, Object>> getWaterBalance(Page page, WisdomDisplayDto wisdomDisplayDto) {
        String type = wisdomDisplayDto.getType();
        if(StringUtils.isBlank(type)){
            throw new ToflyDeniedException("类型不能为空");
        }
        if(type.equals("1")){
            return customDisplayMapper.getWaterBalanceM(page, wisdomDisplayDto);
        }else if(type.equals("2")){
            return customDisplayMapper.getWaterBalanceY(page, wisdomDisplayDto);
        }else {
            throw new ToflyDeniedException("类型传入数据有误");
        }
    }

    @Override
    public Page<Map<String, Object>> getObserveFlow(Page page, WisdomDisplayDto wisdomDisplayDto) {
        Integer statisticsType = wisdomDisplayDto.getStatisticsType();
        if(statisticsType == null){
            throw new ToflyDeniedException("类型不能为空");
        }
        switch (statisticsType) {
            case 2:
                return customDisplayMapper.getObserveFlowM(page, wisdomDisplayDto);
            case 1:
                return customDisplayMapper.getObserveFlowD(page, wisdomDisplayDto);
            case 0:
                return customDisplayMapper.getObserveFlowH(page, wisdomDisplayDto);
            default:
                throw new ToflyDeniedException("类型传入数据有误");
        }
    }

    @Override
    public Map<String, Object> getLeakOverview() {
        Map<String, Object> leakOverviewMap = customDisplayMapper.getLeakOverview();
        if(leakOverviewMap == null){
            return null;
        }
        leakOverviewMap.put("monitorCount" , customDisplayMapper.getMonitorCount());
        return leakOverviewMap;
    }

    @Override
    public List<Map<String, Object>> getSupplyWaterYear() {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        Calendar c = Calendar.getInstance();
        c.add(Calendar.MONTH, -1);
        //上个月时间
        String endDate = sdf.format(c.getTime());

        c.add(Calendar.YEAR, -1);
        c.add(Calendar.MONTH, +1);
        //去年这个月时间
        String startDate = sdf.format(c.getTime());

        WisdomDisplayDto wisdomDisplayDto = new WisdomDisplayDto();
        wisdomDisplayDto.setStartDate(startDate);
        wisdomDisplayDto.setEndDate(endDate);
        return customDisplayMapper.getSupplyWaterYear(wisdomDisplayDto);
    }

    @Override
    public List<Map<String, Object>> getLeakNrwYear(WisdomDisplayDto wisdomDisplayDto) {
        List<Map<String, Object>> vos = new ArrayList<>();
        if(wisdomDisplayDto.getStartYear() != null && wisdomDisplayDto.getEndYear() != null){
            int startYear = Integer.parseInt(wisdomDisplayDto.getStartYear());
            int endYear = Integer.parseInt(wisdomDisplayDto.getEndYear());
            for(int i = startYear;i < endYear + 1;i++){
                String censusMonth;
                for(int j = 1;j < 13;j++){
                    if(j<10){
                        censusMonth = i + "-0" + j;
                    }else {
                        censusMonth = i + "-" + j;
                    }
                    Map<String,Object> map = customDisplayMapper.getDmaStatisticalMonthOne(censusMonth);
                    if(map == null){
                        Map<String, Object> vo = new HashMap<>();
                        vo.put("statisticalDate",censusMonth);
                        vo.put("waterSupply",0);
                        vo.put("leakageRate",0);
                        vo.put("poorProductionRate",0);
                        vos.add(vo);
                    }else {
                        vos.add(map);
                    }
                }
            }
            return vos;
        }else {
            return customDisplayMapper.getDmaStatisticalMonth(wisdomDisplayDto);
        }
    }

    @Override
    public List<Map<String, Object>> getDmaStatisticalMonth(WisdomDisplayDto wisdomDisplayDto) {
        List<Map<String, Object>> list = customDisplayMapper.getDmaStatisticalMonth(wisdomDisplayDto);
        for(Map<String, Object> one : list){
            if(one.get("statisticalDate") == null){
                continue;
            }
            String nowDate = String.valueOf(one.get("statisticalDate"));
            String year = nowDate.substring(0,4);
            String month = nowDate.substring(5,7);
            //去年下个月时间
            String lastYearNextMonth =getTrueDate(String.valueOf((Integer.parseInt(year) - 1)),String.valueOf(Integer.parseInt(month) + 1));
            //去年当月时间
            String lastYearThisMonth = (Integer.parseInt(year) - 1) + "-" + month;
            //当年上个月时间
            String thisYearLastMonth =getTrueDate(year,String.valueOf(Integer.parseInt(month) - 1));

            //计算IIL漏损指数
            WisdomDisplayDto wis = new WisdomDisplayDto();
            wis.setStartMonth(lastYearNextMonth);
            wis.setEndMonth(nowDate);
            Map<String, Object> statisticalMap = customDisplayMapper.getStatisticalGroupYear(wis);
            if(statisticalMap.get("lossWaterVolume") != null && statisticalMap.get("outflowWaterVolume") != null){
                double outflowWaterVolume = Double.parseDouble(String.valueOf(statisticalMap.get("outflowWaterVolume")));
                double lossWaterVolume = Double.parseDouble(String.valueOf(statisticalMap.get("lossWaterVolume")));
                if(outflowWaterVolume != 0 && lossWaterVolume != 0){
                    BigDecimal b1 = new BigDecimal(Double.toString(lossWaterVolume));
                    BigDecimal b2 = new BigDecimal(Double.toString(outflowWaterVolume));
                    BigDecimal ILILeakageIndex = b2.divide(b1, 2, RoundingMode.HALF_UP);
                    one.put("ILILeakageIndex",ILILeakageIndex);
                }
            }
            if(!one.containsKey("ILILeakageIndex")){
                one.put("ILILeakageIndex",BigDecimal.ZERO);
            }

            //计算供水量同比增长率
            wis.setStartMonth(null);
            wis.setEndMonth(null);
            wis.setCensusMonth(lastYearThisMonth);
            Map<String, Object> lastYearThisMonthMap = customDisplayMapper.getStatisticalGroupYear(wis);
            float yearOnYearMarginRate = getIndexRate("waterSupply", one, lastYearThisMonthMap);
            float yearOnYearWaterSupply = getMainIndexRate("waterSupply", one, lastYearThisMonthMap);
            BigDecimal yearOnYearMarginValueWaterSupply = getIndexValue("waterSupply", one, lastYearThisMonthMap);
            one.put("yearOnYearMarginRate",yearOnYearMarginRate);
            one.put("yearOnYearWaterSupply",yearOnYearWaterSupply);
            one.put("yearOnYearMarginValueWaterSupply",yearOnYearMarginValueWaterSupply);
            //计算售水量同比增长率
            float yearOnYearMarginRateUserWater = getIndexRate("userWater", one, lastYearThisMonthMap);
            float yearOnYearUserWater = getMainIndexRate("userWater", one, lastYearThisMonthMap);
            BigDecimal yearOnYearMarginValueUserWater = getIndexValue("userWater", one, lastYearThisMonthMap);
            one.put("yearOnYearMarginRateUserWater",yearOnYearMarginRateUserWater);
            one.put("yearOnYearUserWater",yearOnYearUserWater);
            one.put("yearOnYearMarginValueUserWater",yearOnYearMarginValueUserWater);

            //计算供水量环比增长率
            wis.setCensusMonth(thisYearLastMonth);
            Map<String, Object> thisYearLastMonthMap = customDisplayMapper.getStatisticalGroupYear(wis);
            float linkRatioMarginRate = getIndexRate("waterSupply", one, thisYearLastMonthMap);
            float linkRatioWaterSupply = getMainIndexRate("waterSupply", one, thisYearLastMonthMap);
            one.put("linkRatioMarginRate",linkRatioMarginRate);
            one.put("linkRatioWaterSupply",linkRatioWaterSupply);
            //计算售水量环比增长率
            float linkRatioMarginRateUserWater = getIndexRate("userWater", one, thisYearLastMonthMap);
            float linkRatioUserWater = getMainIndexRate("userWater", one, thisYearLastMonthMap);
            one.put("linkRatioMarginRateUserWater",linkRatioMarginRateUserWater);
            one.put("linkRatioUserWater",linkRatioUserWater);
        }
        return list;
    }

    @Override
    public Map<String, Object> getLevelDmaLeak() {
        Map<String, Object> dmaLeak = new HashMap<>();

        dmaLeak.put("oneLeakList",getLevelDmaLeakList(1));
        dmaLeak.put("twoLeakList",getLevelDmaLeakList(2));
        dmaLeak.put("threeLeakList",getLevelDmaLeakList(3));

        dmaLeak.put("dmaCount",customDisplayMapper.getDmaCount());
        dmaLeak.put("LeakCount",customDisplayMapper.getLeakageCount());

        return dmaLeak;
    }

    @Override
    public List<Map<String, Object>> viewDmaCensusParam(WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.viewDmaCensusParam(wisdomDisplayDto);
    }

    @Override
    public WaterCruxIndexVo viewWaterCruxIndex(WisdomDisplayDto wisdomDisplayDto) {
        WaterCruxIndexVo waterCruxIndexVo = new WaterCruxIndexVo();
//        SimpleDateFormat sdfMonth = new SimpleDateFormat("yyyy-MM");
//        SimpleDateFormat sdfymd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        //计算总供水用户数量
        waterCruxIndexVo.setSupplyWaterUsers(theOnlineCheckService.getTableUserListNum());

//        //计算总制水量
//        Map<String,Object> totalIndexMap = customDisplayMapper.getStatisticalGroupYear(new WisdomDisplayDto());
//        if(totalIndexMap != null && totalIndexMap.get("waterSupply") != null){
//            waterCruxIndexVo.setTotalWaterSupply(new BigDecimal(String.valueOf(totalIndexMap.get("waterSupply"))).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//        }else {
//            waterCruxIndexVo.setTotalWaterSupply(BigDecimal.ZERO);
//        }

        //计算管网总长度
        List<PipeMaterialStatisticalVo> pipeMaterialStatisticalVos = pgWaterMapper.getPipeMaterialStatistical();
        BigDecimal pipeLength = BigDecimal.ZERO;
        for(PipeMaterialStatisticalVo one : pipeMaterialStatisticalVos){
            if(one.getNum() != null){
                pipeLength = pipeLength.add(one.getNum());
            }
        }
        pipeLength = pipeLength.setScale(2, RoundingMode.HALF_UP);
        waterCruxIndexVo.setPipeLength(pipeLength);

//        //计算设计供水能力
//        waterCruxIndexVo.setDesignProcessingCapacity(customDisplayMapper.getDesignProcessCapacity(null).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//        waterCruxIndexVo.setOtDesignProcessingCapacity(customDisplayMapper.getDesignProcessCapacity("一二期").divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//        waterCruxIndexVo.setTDesignProcessingCapacity(customDisplayMapper.getDesignProcessCapacity("三期").divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//
//        //计算今年供水量
//        Calendar cal = Calendar.getInstance();
//        SimpleDateFormat sdfYear = new SimpleDateFormat("yyyy");
//        String startDateYW = sdfYear.format(cal.getTime()) + "-01";
//        String endDateYW = sdfMonth.format(cal.getTime());
//        WisdomDisplayDto wi = new WisdomDisplayDto();
//        wi.setStartMonth(startDateYW);
//        wi.setEndMonth(endDateYW);
//        Map<String, Object> yy = customDisplayMapper.getStatisticalGroupYear(wi);
//        if(yy != null && yy.get("waterSupply") != null){
//            waterCruxIndexVo.setYearSupplyWater(new BigDecimal(String.valueOf(yy.get("waterSupply"))).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//        }else {
//            waterCruxIndexVo.setYearSupplyWater(BigDecimal.ZERO);
//        }
//        if(yy != null && yy.get("userWater") != null){
//            waterCruxIndexVo.setYearUserWater(new BigDecimal(String.valueOf(yy.get("userWater"))).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//        }else {
//            waterCruxIndexVo.setYearUserWater(BigDecimal.ZERO);
//        }
//
//        //计算上月供水量
//        Map<String,Object> lastMonthIndex = customDisplayMapper.getLeakOverview();
//        if(lastMonthIndex != null){
//            waterCruxIndexVo.setLastMonthSupplyWater(new BigDecimal(String.valueOf(lastMonthIndex.get("waterSupply"))).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//        }else {
//            waterCruxIndexVo.setLastMonthSupplyWater(BigDecimal.ZERO);
//        }
//        //计算上月售水量
//        if(lastMonthIndex != null){
//            waterCruxIndexVo.setLastMonthUserWater(new BigDecimal(String.valueOf(lastMonthIndex.get("billingWaterVolume"))).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//        }else {
//            waterCruxIndexVo.setLastMonthUserWater(BigDecimal.ZERO);
//        }
//
//        //计算昨日供水量
//        waterCruxIndexVo.setYesterdaySupplyWater(customDisplayMapper.getYesterdaySupplyWater().divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//
//        //计算昨日取水量
//        waterCruxIndexVo.setYesterdayUserWater(customDisplayMapper.getYesterdayUserWater().divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//
//        WisdomDisplayDto wisLastMonth = new WisdomDisplayDto();
//        Calendar cc = Calendar.getInstance();
//        cc.add(Calendar.MONTH, -1);
//        wisLastMonth.setCensusMonth(sdfMonth.format(cc.getTime()));
//        //计算同比增长
//        List<Map<String, Object>> lastMonthDmaIndex = this.getDmaStatisticalMonth(wisLastMonth);
//        for(Map<String, Object> one : lastMonthDmaIndex){
//            //计算供水量同比增长率
//            if(one.get("yearOnYearMarginRate") != null){
//                waterCruxIndexVo.setYearOnYearRateWaterSupply(Float.parseFloat(String.valueOf(one.get("yearOnYearMarginRate"))));
//            }
//            //计算供水量同比增长值
//            if(one.get("yearOnYearMarginValueWaterSupply") != null){
//                waterCruxIndexVo.setYearOnYearValueWaterSupply(new BigDecimal(String.valueOf(one.get("yearOnYearMarginValueWaterSupply"))).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//            }
//            //计算售水量同比增长率
//            if(one.get("yearOnYearMarginRateUserWater") != null){
//                waterCruxIndexVo.setYearOnYearRateUserWater(Float.parseFloat(String.valueOf(one.get("yearOnYearMarginRateUserWater"))));
//            }
//            //计算售水量同比增长值
//            if(one.get("yearOnYearMarginValueUserWater") != null){
//                waterCruxIndexVo.setYearOnYearValueUserWater(new BigDecimal(String.valueOf(one.get("yearOnYearMarginValueUserWater"))).divide(BigDecimal.valueOf(10000),2,RoundingMode.HALF_UP));
//            }
//        }
//        if(waterCruxIndexVo.getYearOnYearRateWaterSupply() == null){
//            waterCruxIndexVo.setYearOnYearRateWaterSupply(0.00f);
//        }
//        if(waterCruxIndexVo.getYearOnYearValueWaterSupply() == null){
//            waterCruxIndexVo.setYearOnYearValueWaterSupply(BigDecimal.ZERO);
//        }
//        if(waterCruxIndexVo.getYearOnYearRateUserWater() == null){
//            waterCruxIndexVo.setYearOnYearRateUserWater(0.00f);
//        }
//        if(waterCruxIndexVo.getYearOnYearValueUserWater() == null){
//            waterCruxIndexVo.setYearOnYearValueUserWater(BigDecimal.ZERO);
//        }
//
//        //计算电耗同比增长
//        wisLastMonth.setType("1");
//        wisLastMonth.setCensusYear(sdfYear.format(new Date()));
//        List<Map<String, Object>> powerLossCensusList = this.viewLossCensus(wisLastMonth);
//        for(Map<String, Object> lossCensusOne : powerLossCensusList){
//            if(lossCensusOne.get("yearOnYearMarginRate") != null){
//                waterCruxIndexVo.setYearOnYearRatePower(Float.parseFloat(String.valueOf(lossCensusOne.get("yearOnYearMarginRate"))));
//            }
//            if(lossCensusOne.get("yearOnYearMarginValue") != null){
//                waterCruxIndexVo.setYearOnYearValuePower(new BigDecimal(String.valueOf(lossCensusOne.get("yearOnYearMarginValue"))));
//            }
//        }
//        if(waterCruxIndexVo.getYearOnYearRatePower() == null){
//            waterCruxIndexVo.setYearOnYearRatePower(0.00f);
//        }
//        if(waterCruxIndexVo.getYearOnYearValuePower() == null){
//            waterCruxIndexVo.setYearOnYearValuePower(BigDecimal.ZERO);
//        }
//
//        //计算药耗同比增长
//        wisLastMonth.setType("2");
//        wisLastMonth.setCensusYear(sdfYear.format(new Date()));
//        List<Map<String, Object>> drugLossCensusList = this.viewLossCensus(wisLastMonth);
//        for(Map<String, Object> lossCensusOne : drugLossCensusList){
//            if(lossCensusOne.get("yearOnYearMarginRate") != null){
//                waterCruxIndexVo.setYearOnYearRateDrug(Float.parseFloat(String.valueOf(lossCensusOne.get("yearOnYearMarginRate"))));
//            }
//            if(lossCensusOne.get("yearOnYearMarginValue") != null){
//                waterCruxIndexVo.setYearOnYearValueDrug(new BigDecimal(String.valueOf(lossCensusOne.get("yearOnYearMarginValue"))));
//            }
//        }
//        if(waterCruxIndexVo.getYearOnYearRateDrug() == null){
//            waterCruxIndexVo.setYearOnYearRateDrug(0.00f);
//        }
//        if(waterCruxIndexVo.getYearOnYearValueDrug() == null){
//            waterCruxIndexVo.setYearOnYearValueDrug(BigDecimal.ZERO);
//        }
//
//        //计算安全生产天数
//        List<TfWaterInfo> waterInfoList = customDisplayMapper.getWaterInfo();
//        if(waterInfoList.size() != 0){
//            Date blowInTime = waterInfoList.get(0).getBlowInTime();
//            Date nowTime = new Date();
//            if(blowInTime != null){
//                waterCruxIndexVo.setSafeMakeDays(differentDays(blowInTime, nowTime));
//            }
//        }
//
//        //计算视频监控数量
//        waterCruxIndexVo.setCameraMonitorAmount(customDisplayMapper.getCameraAmount());

        //计算制水厂数量
        waterCruxIndexVo.setWaterPlantAmount(customDisplayMapper.getWaterAmount());

//        //计算加压站数量
//        waterCruxIndexVo.setPressStationAmount(customDisplayMapper.getPipeDevice(null,"5",null));
//
//        //计算远控阀门数量
//        waterCruxIndexVo.setRemoteControlValveNum(customDisplayMapper.getPipeDevice("2","8",null));
//
//        //计算管网水质监测点
//        waterCruxIndexVo.setPipeWaterMonitorAmount(customDisplayMapper.getPipeDevice("2","6",null));
//
//        //计算管网压力监测点
//        waterCruxIndexVo.setPipePressMonitorAmount(customDisplayMapper.getPipeDevice("2","1",null));
//
//        //计算管网流量监测点
//        waterCruxIndexVo.setFlowMonitorAmount(customDisplayMapper.getPipeDevice("2","3",null));
//
//        //计算噪音检测仪
//        waterCruxIndexVo.setNoiseCheckAmount(customDisplayMapper.getNoiseCheckAmount());
//
//        //计算设备完好率
//        int runningDevice = customDisplayMapper.getPipeDevice(null,null,"1");
//        int freeDevice = customDisplayMapper.getPipeDevice(null,null,"2");
//        int allDevice = customDisplayMapper.getPipeDevice(null,null,null);
//        int sumGoodDevice = runningDevice + freeDevice;
//        if(allDevice != 0 && sumGoodDevice != 0){
//            float deviceIntactRate = new BigDecimal((float)sumGoodDevice / allDevice).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.HALF_UP).floatValue();
//            waterCruxIndexVo.setDeviceIntactRate(deviceIntactRate);
//        }else {
//            waterCruxIndexVo.setDeviceIntactRate(0.00f);
//        }
//
//        //计算近一年的漏损率和产销差和近三年的漏损率
//        Calendar c = Calendar.getInstance();
//        c.add(Calendar.MONTH, -1);
//        //上个月时间
//        String endDate = sdfMonth.format(c.getTime());
//        Date endDateFormat = null;
//        try {
//            endDateFormat = sdfymd.parse(sdfymd.format(c.getTime()));
//        } catch (ParseException e) {
//            e.printStackTrace();
//        }
//
//        c.add(Calendar.YEAR, -1);
//        c.add(Calendar.MONTH, +1);
//        //去年这个月时间
//        String startDate = sdfMonth.format(c.getTime());
//        Date startDateFormat = null;
//        try {
//            startDateFormat = sdfymd.parse(sdfymd.format(c.getTime()));
//        } catch (ParseException e) {
//            e.printStackTrace();
//        }
//
//        c.add(Calendar.YEAR, -2);
//        //三年前这个月时间
//        String startDateThree = sdfMonth.format(c.getTime());
//        List<Map<String,Double>> nearYearStaticalList = customDisplayMapper.getNearYearStatistical(startDate, endDate);
//        if(nearYearStaticalList.size() == 0){
//            waterCruxIndexVo.setNearYearLeakRate(0.00);
//            waterCruxIndexVo.setNearYearNrw(0.00);
//        }else {
//            Map<String,Double> leakNrwAvg = getLeakNrwAvg(nearYearStaticalList);
//            waterCruxIndexVo.setNearYearLeakRate(leakNrwAvg.get("leakageRate"));
//            waterCruxIndexVo.setNearYearNrw(leakNrwAvg.get("poorProductionRate"));
//        }
//        List<Map<String,Double>> nearThreeYearStaticalList = customDisplayMapper.getNearYearStatistical(startDateThree, endDate);
//        if(nearThreeYearStaticalList.size() == 0){
//            waterCruxIndexVo.setNearThreeYearLeakRate(0.00);
//        }else {
//            Map<String,Double> leakNrwAvg = getLeakNrwAvg(nearThreeYearStaticalList);
//            waterCruxIndexVo.setNearThreeYearLeakRate(leakNrwAvg.get("leakageRate"));
//        }
//
//        //计算水质合格率
//        int qualityWaterCount = customDisplayMapper.getWaterQualityDevice("0");
//        int AllWaterCount = customDisplayMapper.getWaterQualityDevice(null);
//        if(qualityWaterCount != 0 && AllWaterCount != 0){
//            float deviceIntactRate = new BigDecimal((float)qualityWaterCount / AllWaterCount).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.HALF_UP).floatValue();
//            waterCruxIndexVo.setWaterQualityRate(deviceIntactRate);
//        }else {
//            waterCruxIndexVo.setWaterQualityRate(0.00f);
//        }
//
//        //计算管网总表数量
//        waterCruxIndexVo.setPipeTableAmount(customDisplayMapper.getPipeTableAmount());
//
//        //计算ILI漏损指数
//        if(wisdomDisplayDto.getStartDate() == null && wisdomDisplayDto.getEndDate() == null){
//            Calendar c1 = Calendar.getInstance();
//            c1.add(Calendar.MONTH, -1);
//            wisdomDisplayDto.setEndMonth(sdfMonth.format(c1.getTime()));
//
//            c1.add(Calendar.YEAR, -1);
//            c1.add(Calendar.MONTH, +1);
//            //去年这个月时间
//            wisdomDisplayDto.setStartMonth(sdfMonth.format(c1.getTime()));
//        }
//        Map<String, Object> statisticalMap = customDisplayMapper.getStatisticalGroupYear(wisdomDisplayDto);
//        if(statisticalMap != null && statisticalMap.get("lossWaterVolume") != null && statisticalMap.get("outflowWaterVolume") != null){
//            double outflowWaterVolume = Double.parseDouble(String.valueOf(statisticalMap.get("outflowWaterVolume")));
//            double lossWaterVolume = Double.parseDouble(String.valueOf(statisticalMap.get("lossWaterVolume")));
//            if(outflowWaterVolume != 0 && lossWaterVolume != 0){
//                BigDecimal b1 = new BigDecimal(Double.toString(lossWaterVolume));
//                BigDecimal b2 = new BigDecimal(Double.toString(outflowWaterVolume));
//                BigDecimal ILILeakageIndex = b2.divide(b1, 2, RoundingMode.HALF_UP);
//                waterCruxIndexVo.setILILeakageIndex(ILILeakageIndex);
//            }
//        }
//        if(waterCruxIndexVo.getILILeakageIndex() == null){
//            waterCruxIndexVo.setILILeakageIndex(BigDecimal.ZERO);
//        }
//
//        //计算巡检相关数据
//        WisdomDisplayDto wis = new WisdomDisplayDto();
//        //计算巡检总人数
//        waterCruxIndexVo.setInspectPersonNum(customDisplayMapper.getInspectPersonNum());
//        //计算当月统计数据
//        wis.setCensusMonth(sdfMonth.format(new Date()));
//        Map<String, Object> monthInspectData = customDisplayMapper.viewInspectCensus(wis);
//        if(monthInspectData != null && monthInspectData.get("orderNum") != null){
//            waterCruxIndexVo.setCmOrderNum(Integer.parseInt(String.valueOf(monthInspectData.get("orderNum"))));
//            if(monthInspectData.get("completeOrderNum") != null){
//                BigDecimal completeOrderNum = new BigDecimal(String.valueOf(monthInspectData.get("completeOrderNum")));
//                BigDecimal orderNum = new BigDecimal(String.valueOf(monthInspectData.get("orderNum")));
//                if(!orderNum.equals(BigDecimal.ZERO) && !completeOrderNum.equals(BigDecimal.ZERO)){
//                    waterCruxIndexVo.setCmCompleteOrderRate(Float.parseFloat(String.valueOf(completeOrderNum.divide(orderNum,4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)))));
//                }else {
//                    waterCruxIndexVo.setCmCompleteOrderRate(0.00f);
//                }
//            }
//        }
//        if(monthInspectData != null && monthInspectData.get("handleOrderNum") != null){
//            waterCruxIndexVo.setCmHandleOrderNum(Integer.parseInt(String.valueOf(monthInspectData.get("handleOrderNum"))));
//        }
//        Long monthAllOrderData = customDisplayMapper.viewOrderCensus(wis);
//        if(monthAllOrderData != null && monthAllOrderData != 0L) {
//            //获得当月工单总数
//            waterCruxIndexVo.setCmOrderNum(monthAllOrderData);
//            //设置为归档
//            wis.setProcessNode(45L);
//            Long monthCompleteOrderData = customDisplayMapper.viewOrderCensus(wis);
//            if(monthCompleteOrderData != null && monthCompleteOrderData != 0L) {
//                //获得当月正在处理工单数
//                waterCruxIndexVo.setCmHandleOrderNum(monthAllOrderData - monthCompleteOrderData);
//                Float completeOrderRate = BigDecimal.valueOf(monthCompleteOrderData).divide(BigDecimal.valueOf(monthAllOrderData),4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
//                //获得当月工单完成率
//                waterCruxIndexVo.setCmCompleteOrderRate(completeOrderRate);
//            }else {
//                waterCruxIndexVo.setCmHandleOrderNum(monthAllOrderData);
//                waterCruxIndexVo.setCmCompleteOrderRate(00.00f);
//            }
//        }else {
//            waterCruxIndexVo.setCmOrderNum(0L);
//            waterCruxIndexVo.setCmHandleOrderNum(0L);
//            waterCruxIndexVo.setCmCompleteOrderRate(00.00f);
//        }
//        //计算近十二个月统计数据
//        wis.setStartMonth(startDate);
//        wis.setEndMonth(endDate);
//        wis.setCensusMonth(null);
//        wis.setProcessNode(null);
//        Map<String, Object> yearInspectData = customDisplayMapper.viewInspectCensus(wis);
//        if(yearInspectData != null && yearInspectData.get("orderNum") != null){
//            waterCruxIndexVo.setYearOrderNum(Integer.parseInt(String.valueOf(yearInspectData.get("orderNum"))));
//            if(yearInspectData.get("completeOrderNum") != null){
//                BigDecimal completeOrderNum = new BigDecimal(String.valueOf(yearInspectData.get("completeOrderNum")));
//                BigDecimal orderNum = new BigDecimal(String.valueOf(yearInspectData.get("orderNum")));
//                if(!orderNum.equals(BigDecimal.ZERO) && !completeOrderNum.equals(BigDecimal.ZERO)){
//                    waterCruxIndexVo.setYearCompleteOrderRate(Float.parseFloat(String.valueOf(completeOrderNum.divide(orderNum,4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)))));
//                }else {
//                    waterCruxIndexVo.setYearCompleteOrderRate(0.00f);
//                }
//            }
//        }
//        if(yearInspectData != null && yearInspectData.get("inspectNum") != null){
//            waterCruxIndexVo.setYearInspectNum(Integer.parseInt(String.valueOf(yearInspectData.get("inspectNum"))));
//        }
//        if(yearInspectData != null && yearInspectData.get("planTotalDistance") != null){
//            waterCruxIndexVo.setYearInspectDistance(new BigDecimal(String.valueOf(yearInspectData.get("planTotalDistance"))));
//        }
//        if(yearInspectData != null && yearInspectData.get("findTroubleNum") != null){
//            waterCruxIndexVo.setYearFindTroubleNum(Integer.parseInt(String.valueOf(yearInspectData.get("findTroubleNum"))));
//        }
//        Long yearAllOrderData = customDisplayMapper.viewOrderCensus(wis);
//        if(yearAllOrderData != null) {
//            //获得近一年工单总数
//            waterCruxIndexVo.setYearOrderNum(yearAllOrderData);
//            //设置为归档
//            wis.setProcessNode(45L);
//            Long yearCompleteOrderData = customDisplayMapper.viewOrderCensus(wis);
//            if(yearCompleteOrderData != null) {
//                Float completeOrderRate = BigDecimal.valueOf(yearCompleteOrderData).divide(BigDecimal.valueOf(yearAllOrderData),4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
//                //获得近一年工单完成率
//                waterCruxIndexVo.setYearCompleteOrderRate(completeOrderRate);
//            }else {
//                waterCruxIndexVo.setYearCompleteOrderRate(00.00f);
//            }
//        }else {
//            waterCruxIndexVo.setYearOrderNum(0L);
//            waterCruxIndexVo.setYearCompleteOrderRate(00.00f);
//        }
//        Long yearInspectNum = customDisplayMapper.viewInspectPlanCensus(wis);
//        if(yearInspectNum != null) {
//            //获得近一年巡检次数
//            waterCruxIndexVo.setYearInspectNum(yearInspectNum);
//        }else {
//            waterCruxIndexVo.setYearInspectNum(0L);
//        }
//        Long yearTroubleNum = customDisplayMapper.viewTroubleCensus(wis);
//        if(yearTroubleNum != null && yearTroubleNum != 0) {
//            //获得近一年发现隐患次数
//            waterCruxIndexVo.setYearFindTroubleNum(yearTroubleNum);
//            wis.setTroubleState("2");
//            Long yearHandleTroubleNum = customDisplayMapper.viewTroubleCensus(wis);
//            if(yearHandleTroubleNum != null && yearHandleTroubleNum != 0L) {
//                waterCruxIndexVo.setYearHandleTroubleNum(yearHandleTroubleNum);
//                waterCruxIndexVo.setYearNotHandleTroubleNum(yearTroubleNum - yearHandleTroubleNum);
//            }else {
//                waterCruxIndexVo.setYearHandleTroubleNum(0L);
//                waterCruxIndexVo.setYearNotHandleTroubleNum(yearTroubleNum);
//            }
//        }else {
//            waterCruxIndexVo.setYearFindTroubleNum(0L);
//        }
        // 计算近一年巡检公里数
        // 根据时间范围 查询对应巡检用户的 巡检里程
//        EsQueryDto pram = new EsQueryDto();
//        pram.setStartDate(startDateFormat);
//        pram.setEndDate(endDateFormat);
//        List<Long> inspectPersonList = customDisplayMapper.getInspectPersonList();
//        BigDecimal yearInspectDistance = BigDecimal.ZERO;
//        if(inspectPersonList != null && inspectPersonList.size() != 0){
//            StringBuilder inspectPersonText = new StringBuilder();
//            inspectPersonList.forEach(inspectPersonText::append);
//            pram.setUserIds(inspectPersonText.toString());
//            List<GpsUpload> all = esClient.getAll(pram).getResult();
//            if(all != null && all.size() != 0){
//                // 用户分组
//                Map<Long, List<GpsUpload>> collect = all.stream().collect(Collectors.groupingBy(GpsUpload::getUserId));
//                collect.forEach((k, v) -> {
//                    // 计算个人巡检里程
//                    BigDecimal totalMileage = v.stream().map(GpsUpload::getDistance).reduce(BigDecimal.ZERO, BigDecimal::add).divide(BigDecimal.valueOf(1000), 4, RoundingMode.HALF_UP);
//                    yearInspectDistance.add(totalMileage);
//                });
//            }
//        }
//        waterCruxIndexVo.setYearInspectDistance(yearInspectDistance);
//
//        Calendar c2 = Calendar.getInstance();
//
//        WisdomDisplayDto wisHotLine = new WisdomDisplayDto();
//        wisHotLine.setCensusYear(sdfYear.format(c2.getTime()));
//        //计算今年热线来电数量
//        wisHotLine.setCallType(1);
//        waterCruxIndexVo.setThisYearHotLineInNum(pltCalllist1000Mapper.viewNewHotLineNum(wisHotLine));
//        //计算今年热线去电数量
//        wisHotLine.setCallType(2);
//        waterCruxIndexVo.setThisYearHotLineOutNum(pltCalllist1000Mapper.viewNewHotLineNum(wisHotLine));
//        //计算今年热线好评率
//        waterCruxIndexVo.setHotLinePraiseRate(getHotLineRate(sdfYear.format(c2.getTime()),"1"));
//        //计算上一年热线来电数量
//        c2.add(Calendar.YEAR, -1);
//        wisHotLine.setCensusYear(sdfYear.format(c2.getTime()));
//        wisHotLine.setCallType(1);
//        waterCruxIndexVo.setLastYearHotLineInNum(pltCalllist1000Mapper.viewNewHotLineNum(wisHotLine));
//        //计算上一年热线去电数量
//        wisHotLine.setCallType(2);
//        waterCruxIndexVo.setLastYearHotLineOutNum(pltCalllist1000Mapper.viewNewHotLineNum(wisHotLine));
//        //计算上一年热线好评率
//        waterCruxIndexVo.setLastYearHotLinePraiseRate(getHotLineRate(sdfYear.format(c2.getTime()),"1"));

        return waterCruxIndexVo;
    }

    @Override
    public Page<Map<String, Object>> reportFormCensus(Page page, WisdomDisplayDto wisdomDisplayDto) {
        String censusDateType = wisdomDisplayDto.getCensusDateType();
        //String deviceType = wisdomDisplayDto.getDeviceType();
        if(StringUtils.isBlank(censusDateType)){
            throw new ToflyDeniedException("时间统计类型不能为空");
        }
//        if(StringUtils.isBlank(deviceType)){
//            throw new ToflyDeniedException("设备类型不能为空");
//        }
        List<TfDeviceArchiveManage> deviceArchiveManages = deviceArchiveManageService.list();
        List<TfIndexAllocation> indexAllocations = indexAllocationService.list();
        switch (censusDateType) {
            case "1":
                wisdomDisplayDto.setStatisticsType(3);
                break;
            case "2":
                wisdomDisplayDto.setStatisticsType(2);
                break;
            case "3":
                wisdomDisplayDto.setStatisticsType(1);
                break;
            default:
                throw new ToflyDeniedException("时间统计类型传入数据有误");
        }
        Page<Map<String, Object>> resPage = customDisplayMapper.reportFormCensus(page, wisdomDisplayDto);
        resPage.getRecords().forEach(info -> {
            if (info.get("variableCode") == null) {
                return;
            }
            String variableCode = String.valueOf(info.get("variableCode"));
            List<TfIndexAllocation> indexAllocationsFinal = indexAllocations.stream().filter(e -> e.getVariableCode().equals(variableCode)).collect(Collectors.toList());
            if (CollUtil.isEmpty(indexAllocationsFinal)) {
                return;
            }
            info.put("unit", indexAllocationsFinal.get(0).getUnit());
            info.put("variableName", indexAllocationsFinal.get(0).getVariableName());
            info.put("displayName", indexAllocationsFinal.get(0).getDisplayName());
            info.put("type", indexAllocationsFinal.get(0).getType());
            info.put("sortNumber", indexAllocationsFinal.get(0).getSortNumber());
            info.put("accuracy", indexAllocationsFinal.get(0).getAccuracy());
            if (info.get("diff") != null) {
                double diff = Double.parseDouble(String.valueOf(info.get("diff")));
                int accuracy = Integer.parseInt(String.valueOf(info.get("accuracy")));
                BigDecimal diffVal = new BigDecimal(diff);
                info.put("diff", diffVal.setScale(accuracy, RoundingMode.HALF_UP).doubleValue());
            }
            if (info.get("ave") != null) {
                double ave = Double.parseDouble(String.valueOf(info.get("ave")));
                int accuracy = Integer.parseInt(String.valueOf(info.get("accuracy")));
                BigDecimal aveVal = new BigDecimal(ave);
                info.put("ave", aveVal.setScale(accuracy, RoundingMode.HALF_UP).doubleValue());
            }
            List<TfDeviceArchiveManage> deviceArchiveManageFinal = deviceArchiveManages.stream()
                    .filter(e -> Objects.equals(e.getId(), indexAllocationsFinal.get(0).getDeviceId()))
                    .collect(Collectors.toList());
            if (CollUtil.isEmpty(deviceArchiveManageFinal)) {
                return;
            }
            info.put("code", deviceArchiveManageFinal.get(0).getCode());
            info.put("name", deviceArchiveManageFinal.get(0).getName());
        });
        return resPage;
    }

    @Override
    public List<Map<String, Object>> currencyViewIndexH(WisdomDisplayDto wisdomDisplayDto) {
        List<Map<String, Object>> indexList = customDisplayMapper.currencyViewIndexH(wisdomDisplayDto);
        for (Map<String, Object> index : indexList) {
            if(index.get("reportId") == null){
                index.put("reportUpper",0.00);
                index.put("reportLower",0.00);
                continue;
            }
            List<Long> reportIdList = Arrays.stream(String.valueOf(index.get("reportId")).split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                    .collect(Collectors.toList());
            List<Map<String,Object>> scadaReportList = customDisplayMapper.getScadaReportById(reportIdList);
            Map<String,Double> scadaReportMap = getReportUpperLower(scadaReportList);
            if(scadaReportMap == null){
                index.put("reportUpper",0.00);
                index.put("reportLower",0.00);
            }else {
                index.put("reportUpper",scadaReportMap.get("reportUpper"));
                index.put("reportLower",scadaReportMap.get("reportLower"));
            }
        }
        return indexList;
    }

    @Override
    public List<Map<String, Object>> currencyViewIndexAvgH(WisdomDisplayDto wisdomDisplayDto) {
        return customDisplayMapper.currencyViewIndexAvgH(wisdomDisplayDto);
    }

    @Override
    public List<Map<String, Object>> censusDeviceStaCount() {
        return customDisplayMapper.censusDeviceStaCount();
    }

    @Override
    public List<Map<String, Object>> censusIndexStaCount() {
        return customDisplayMapper.censusIndexStaCount();
    }

    @Override
    public List<Map<String, Object>> viewDeviceTree(TfDeviceArchiveManage deviceArchiveManage) {
        List<Map<String, Object>> deviceTree = new ArrayList<>();
        List<String> deviceTypeList = customDisplayMapper.getAllDeviceType(deviceArchiveManage);
        for(String deviceType : deviceTypeList){
            if(StringUtils.isBlank(deviceType)){
                continue;
            }
            Map<String, Object> deviceMap = new HashMap<>();

            deviceMap.put("name",customDisplayMapper.getDeviceTypeName(deviceType));

            deviceArchiveManage.setDeviceType(deviceType);
            List<TfDeviceArchiveManage> deviceArchiveManages = deviceArchiveManageService.list(new QueryWrapper<TfDeviceArchiveManage>()
                    .eq("DEVICE_TYPE",deviceType)
                    .eq(StringUtils.isNotBlank(deviceArchiveManage.getType()),"TYPE",deviceArchiveManage.getType())
                    .like(StringUtils.isNotBlank(deviceArchiveManage.getName()),"NAME",deviceArchiveManage.getName()));
            //判断设备监测状态
            for(TfDeviceArchiveManage one : deviceArchiveManages){
                List<TfIndexAllocation> indexAllocations = indexAllocationService.list(new QueryWrapper<TfIndexAllocation>()
                        .eq("DEVICE_ID",one.getId()));
                for(TfIndexAllocation indexAllocation : indexAllocations){
                    if(indexAllocation.getIsAlarm().equals("1")){
                        one.setDeviceMonitorStatus("1");
                        one.setDeviceMonitorStatusName("报警");
                    }
                }
                if(one.getDeviceMonitorStatus() == null){
                    one.setDeviceMonitorStatus("0");
                    one.setDeviceMonitorStatusName("正常");
                }
            }
            deviceMap.put("deviceList",deviceArchiveManages);

            deviceTree.add(deviceMap);

        }
        return deviceTree;
    }

    @Override
    public List<Map<String, Object>> pressQualityRateH(WisdomDisplayDto wisdomDisplayDto) {
        List<Map<String, Object>> pressQualityRates = new ArrayList<>();
        if(wisdomDisplayDto.getCensusDay() == null){
            throw new ToflyDeniedException("统计天censusDay不能为空");
        }
        SimpleDateFormat sdfm = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        for(int i = 0;i < 24;i++){
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH");

            Calendar calendar = Calendar.getInstance();
            try {
                calendar.setTime(sdfm.parse(wisdomDisplayDto.getCensusDay() + " " +"00:00:00"));
            } catch (ParseException e) {
                e.printStackTrace();
            }
            if(i != 0 ){
                calendar.set(Calendar.HOUR, +i);
            }
            String time = sdf.format(calendar.getTime());

            wisdomDisplayDto.setCensusHour(time);
            wisdomDisplayDto.setStatisticsType(0);
            List<Map<String, Object>> indexList = customDisplayMapper.currencyViewIndexH(wisdomDisplayDto);
            Map<String, Object> pressIndexMap = new HashMap<>();
            pressIndexMap.put("time",time);
            int sum = 0;
            int qualitySum = 0;
            float qualityRate;
            if(indexList != null && indexList.size() != 0){
                sum = indexList.size();
                for(Map<String, Object> index : indexList){
                    if(index.get("reportId") == null || index.get("ave") == null){
                        continue;
                    }
                    List<Long> reportIdList = Arrays.stream(String.valueOf(index.get("reportId")).split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                        .collect(Collectors.toList());
                    List<Map<String,Object>> scadaReportList = customDisplayMapper.getScadaReportById(reportIdList);
                    Map<String,Double> scadaReportMap = getReportUpperLower(scadaReportList);
                    if(scadaReportMap == null){
                        continue;
                    }
                    Double reportUpper = scadaReportMap.get("reportUpper");
                    Double reportLower = scadaReportMap.get("reportLower");
                    Double indexAvg = Double.parseDouble(String.valueOf(index.get("ave")));
                    if( indexAvg.compareTo(reportUpper) < 0 && indexAvg.compareTo(reportLower) > 0 ){
                        qualitySum ++;
                    }
                }
            }
            if(sum != 0){
                qualityRate = BigDecimal.valueOf(qualitySum).divide(BigDecimal.valueOf(sum),4,RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
            }else {
                qualityRate = 100;
            }
            pressIndexMap.put("qualityRate",qualityRate);
            pressQualityRates.add(pressIndexMap);
        }
        return pressQualityRates;
    }

    @Override
    public List<Map<String, Object>> viewLossCensus(WisdomDisplayDto wisdomDisplayDto) {
        if(wisdomDisplayDto.getCensusYear() == null){
            throw new ToflyDeniedException("统计年不能为空");
        }
        List<Map<String,Object>> lossCensusDatas =  customDisplayMapper.viewLossCensus(wisdomDisplayDto);
        String month;
        int nowYear;
        for(Map<String,Object> one : lossCensusDatas){
            if(one.get("fillingDate") == null){
                continue;
            }
            month = String.valueOf(one.get("fillingDate")).substring(5,7);
            List<String> censusMonths = new ArrayList<>();
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy");
            nowYear = Integer.parseInt(sdf.format(new Date()));

            //获取近四年同月日期
            for(int i = 0;i < 4;i++){
                censusMonths.add(String.valueOf(nowYear - i).trim() + "-" +month);
            }

            wisdomDisplayDto.setCensusYear(null);
            wisdomDisplayDto.setCensusMonths(censusMonths);
            List<Map<String,Object>> avgLossDatas =  customDisplayMapper.viewLossCensus(wisdomDisplayDto);
            BigDecimal sum = BigDecimal.ZERO;
            int total = 0;
            //计算近四年数据平均值
            for(Map<String,Object> avgData : avgLossDatas){
                if(avgData.get("expendAmount") != null){
                    sum = sum.add(new BigDecimal(String.valueOf(avgData.get("expendAmount"))));
                    total ++;
                }
            }
            BigDecimal avg;
            if(!sum.equals(BigDecimal.ZERO) && total != 0){
                avg = sum.divide(BigDecimal.valueOf(total), 2, RoundingMode.HALF_UP);
                one.put("avg",avg);
            }

            String nowDate = String.valueOf(one.get("fillingDate"));
            String yoyYear = nowDate.substring(0,4);
            String yoyMonth = nowDate.substring(5,7);
            //去年当月时间
            String lastYearThisMonth = (Integer.parseInt(yoyYear) - 1) + "-" + yoyMonth;

            //计算能耗同比增长率
            WisdomDisplayDto wis = new WisdomDisplayDto();
            wis.setCensusMonth(lastYearThisMonth);
            Map<String, Object> lastYearThisMonthMap = customDisplayMapper.viewLossCensusOne(wis);
            float yearOnYearExpendAmount = getMainIndexRate("expendAmount", one, lastYearThisMonthMap);
            float yearOnYearMarginRate = getIndexRate("expendAmount", one, lastYearThisMonthMap);
            BigDecimal yearOnYearMarginValue = getIndexValue("expendAmount", one, lastYearThisMonthMap);
            one.put("yearOnYearExpendAmount",yearOnYearExpendAmount);
            one.put("yearOnYearMarginRate",yearOnYearMarginRate);
            one.put("yearOnYearMarginValue",yearOnYearMarginValue);
        }
        return lossCensusDatas;
    }

//    @Override
//    public List<Map<String, Object>> viewGpsCalculateByMonth(WisdomDisplayDto wisdomDisplayDto) {
//        List<Map<String, Object>> gpsCalculates = customDisplayMapper.viewGpsCalculateByMonth(wisdomDisplayDto);
//        for(Map<String, Object> gpsCalculate : gpsCalculates){
//            if(gpsCalculate.get("orderNum") != null && gpsCalculate.get("completeOrderNum") != null){
//                BigDecimal completeOrderNum = new BigDecimal(String.valueOf(gpsCalculate.get("completeOrderNum")));
//                BigDecimal orderNum = new BigDecimal(String.valueOf(gpsCalculate.get("orderNum")));
//                if(!orderNum.equals(BigDecimal.ZERO) && !completeOrderNum.equals(BigDecimal.ZERO)){
//                    gpsCalculate.put("completeOrderRate",Float.parseFloat(String.valueOf(completeOrderNum.divide(orderNum,4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)))));
//                }
//            }
//            if(!gpsCalculate.containsKey("completeOrderRate")){
//                gpsCalculate.put("completeOrderRate", 0.00);
//            }
//        }
//        return gpsCalculates;
//    }

    @Override
    public List<Map<String, Object>> viewOrderByMonth(WisdomDisplayDto wisdomDisplayDto) {
        List<Map<String, Object>> orderList = customDisplayMapper.viewOrderByMonth(wisdomDisplayDto);
        for(Map<String, Object> order : orderList){
            if(order.get("djsj") == null){
                continue;
            }
            WisdomDisplayDto wis = new WisdomDisplayDto();
            wis.setCensusMonth(String.valueOf(order.get("djsj")));
            Long monthAllOrderData = customDisplayMapper.viewOrderCensus(wis);
            if(monthAllOrderData != null && monthAllOrderData != 0L) {
                //设置为归档
                wis.setProcessNode(45L);
                Long monthCompleteOrderData = customDisplayMapper.viewOrderCensus(wis);
                if(monthCompleteOrderData != null && monthCompleteOrderData != 0L) {
                    order.put("completeOrderNum",monthCompleteOrderData);
                    Float completeOrderRate = BigDecimal.valueOf(monthCompleteOrderData).divide(BigDecimal.valueOf(monthAllOrderData),4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
                    order.put("completeOrderRate",completeOrderRate);
                }else {
                    order.put("completeOrderNum",0L);
                    order.put("completeOrderRate",00.00f);
                }
            }else {
                order.put("completeOrderRate",00.00f);
            }
        }
        return orderList;
    }

    @Override
    public List<Map<String, Object>> viewInspectPlanByMonth(WisdomDisplayDto wisdomDisplayDto) throws Exception {
        List<Map<String, Object>> inspectPlanList = customDisplayMapper.viewInspectPlanByMonth(wisdomDisplayDto);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
        SimpleDateFormat sdfymd = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar calendar = Calendar.getInstance();
        int dayNum;
        for (Map<String, Object> inspectPlan : inspectPlanList) {
            if(inspectPlan.get("inspectDate") == null){
                inspectPlan.put("inspectDistance",BigDecimal.ZERO);
                continue;
            }
            String inspectDate = String.valueOf(inspectPlan.get("inspectDate"));
            calendar.setTime(sdf.parse(inspectDate));
            //获取此年此月最大天数
            dayNum = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
            Date startDate = null;
            //此月第一天
            try {
                startDate = sdfymd.parse(inspectDate + "-01 00:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
            Date endDate = null;
            //此月最后一天
            try {
                if(dayNum > 0 && dayNum <10){
                    endDate = sdfymd.parse(inspectDate + "-0" + dayNum + " 23:59:59");
                }else {
                    endDate = sdfymd.parse(inspectDate + "-" + dayNum + " 23:59:59");
                }
            } catch (ParseException e) {
                e.printStackTrace();
            }
            // 计算此年此月巡检公里数
            // 根据时间范围 查询对应巡检用户的 巡检里程
            EsQueryDto pram = new EsQueryDto();
            pram.setStartDate(startDate);
            pram.setEndDate(endDate);
            List<Long> inspectPersonList = customDisplayMapper.getInspectPersonList();
            BigDecimal inspectDistance = BigDecimal.ZERO;
            if(inspectPersonList != null && inspectPersonList.size() != 0){
                StringBuilder inspectPersonText = new StringBuilder();
                inspectPersonList.forEach(inspectPersonText::append);
                pram.setUserIds(inspectPersonText.toString());
                List<GpsUpload> all = esClient.getAll(pram).getResult();
                if(all != null && all.size() != 0){
                    // 用户分组
                    Map<Long, List<GpsUpload>> collect = all.stream().collect(Collectors.groupingBy(GpsUpload::getUserId));
                    collect.forEach((k, v) -> {
                        // 计算个人巡检里程
                        BigDecimal totalMileage = v.stream().map(GpsUpload::getDistance).reduce(BigDecimal.ZERO, BigDecimal::add).divide(BigDecimal.valueOf(1000), 4, RoundingMode.HALF_UP);
                        inspectDistance.add(totalMileage);
                    });
                }
            }
            inspectPlan.put("inspectDistance",inspectDistance);
        }
        return inspectPlanList;
    }

    @Override
    public List<Map<String, Object>> viewNewHotLineByMonth(WisdomDisplayDto wisdomDisplayDto) {
        List<Map<String, Object>> list = pltCalllist1000Mapper.viewNewHotLineCountByMonth(wisdomDisplayDto);
        for(Map<String, Object> one : list){
            if(one.get("time") == null){
                continue;
            }
            float favorableRate = getHotLineRate(String.valueOf(one.get("time")),"2");
            one.put("favorableRate",favorableRate);
        }
        return list;
    }

    @Override
    public List<WaterStationInfoVo> waterStationInfo() {
        List<TfWaterInfo> waterInfoList = waterInfoService.list();
        List<WaterStationInfoVo> res = new ArrayList<>();
        List<Map<String,Object>> todayStationWaterList = customDisplayMapper.getStationWater(cn.hutool.core.date.DateUtil.format(new Date(), "yyyy-MM-dd") + " 00:00:00",
                cn.hutool.core.date.DateUtil.format(new Date(), "yyyy-MM-dd") + " 23:59:59",
                0);
        List<Map<String,Object>> yearStationWaterList = customDisplayMapper.getStationWater(cn.hutool.core.date.DateUtil.format(new Date(), "yyyy") + "-01-01 00:00:00",
                cn.hutool.core.date.DateUtil.format(new Date(), "yyyy-MM-dd") + " 23:59:59",
                1);
        waterInfoList.forEach(info -> {
            WaterStationInfoVo vo = new WaterStationInfoVo();
            vo.setWaterStationId(info.getId());
            vo.setWaterStationName(info.getName());
            vo.setDesignWaterCapacity(BigDecimal.valueOf(info.getDesignProcessingCapacity()).divide(BigDecimal.valueOf(10000), 2, RoundingMode.HALF_UP));
            if (CollUtil.isNotEmpty(todayStationWaterList)) {
                List<Map<String,Object>> todayStationWaterListFinal = todayStationWaterList.stream()
                        .filter(e -> String.valueOf(e.get("waterStationId")).equals(String.valueOf(info.getId())))
                                .collect(Collectors.toList());
                vo.setTodayWater(CollUtil.isNotEmpty(todayStationWaterListFinal) ?
                        new BigDecimal(String.valueOf(todayStationWaterListFinal.get(0).get("indexValue"))).divide(BigDecimal.valueOf(10000), 2, RoundingMode.HALF_UP):BigDecimal.ZERO);
            }
            if (CollUtil.isNotEmpty(yearStationWaterList)) {
                List<Map<String,Object>> yearStationWaterListFinal = yearStationWaterList.stream()
                        .filter(e -> String.valueOf(e.get("waterStationId")).equals(String.valueOf(info.getId())))
                        .collect(Collectors.toList());
                vo.setThisYearWater(CollUtil.isNotEmpty(yearStationWaterListFinal) ?
                        new BigDecimal(String.valueOf(yearStationWaterListFinal.get(0).get("indexValue"))).divide(BigDecimal.valueOf(10000), 2, RoundingMode.HALF_UP):BigDecimal.ZERO);
            }
            res.add(vo);
        });
        return res;
    }

    @Override
    public List<PlanTypeDoneVo> getPlanTypeDone(String startTime, String endTime) {
        List<Map<String,Object>> planTypeList = customDisplayMapper.getPlanTypeAll();
        List<PlanTypeDoneVo> res = new ArrayList<>();
        List<PlanTypeDoneVo> planTypeDoneVos = customDisplayMapper.getPlanTypeDone(startTime, endTime);
        planTypeList.forEach(info -> {
            PlanTypeDoneVo vo = new PlanTypeDoneVo();
            vo.setPlanTypeName(String.valueOf(info.get("name")));
            vo.setPlanTypeId(Long.parseLong(String.valueOf(info.get("id"))));
            List<PlanTypeDoneVo> planTypeDoneVosFinal = planTypeDoneVos.stream()
                    .filter(e -> Objects.equals(e.getPlanTypeId(), vo.getPlanTypeId()))
                    .collect(Collectors.toList());
            if (CollUtil.isNotEmpty(planTypeDoneVosFinal)) {
                vo.setDoneCount(planTypeDoneVosFinal.get(0).getDoneCount());
                vo.setAllCount(planTypeDoneVosFinal.get(0).getAllCount());
            }
            res.add(vo);
        });
        return res;
    }

    @Override
    public List<Map<String, Object>> getUserCount(String startMonth, String endMonth) {
        List< Map<String, Object>> list = new ArrayList<>();

        if(!StringUtil.isNotEmpty(startMonth) && !StringUtil.isNotEmpty(endMonth)){
            Calendar calendar=Calendar.getInstance();
            Date endTime=calendar.getTime();
            startMonth=CalendarHelper.getYearAndMonth(endTime,-12,"yyyyMM");
            endMonth= CalendarHelper.getFormatDate(endTime,"yyyyMM");
        }

        List<String> listDate = CalendarHelper.getMonthBetweenDate(startMonth, endMonth, "");
        listDate.forEach(item->{
            Map<String, Object> vo = customDisplayMapper.getUserCount(item);
            if(vo == null){
                vo = new HashMap<>();
            }
            vo.put("cwrq",item);
            list.add(vo);
        });

        return list;
    }


    /* ***************************************************自定义方法******************************************** */


    /**
     * 判断点是否在多边形内
     *
     * @param point 检测点
     * @param pts   多边形的顶点
     * @return 点在多边形内返回true, 否则返回false
     */
    public static boolean IsPtInPoly(Point2D.Double point, List<Point2D.Double> pts) {
        int N = pts.size();
        boolean boundOrVertex = true; //如果点位于多边形的顶点或边上，也算做点在多边形内，直接返回true
        int intersectCount = 0;
        double precision = 2e-10; //浮点类型计算时候与0比较时候的容差
        Point2D.Double p1, p2;
        Point2D.Double p = point; //当前点

        p1 = pts.get(0);
        for (int i = 1; i <= N; ++i) {
            if (p.equals(p1)) {
                return boundOrVertex;
            }
            p2 = pts.get(i % N);
            if (p.x < Math.min(p1.x, p2.x) || p.x > Math.max(p1.x, p2.x)) {
                p1 = p2;
                continue;
            }

            if (p.x > Math.min(p1.x, p2.x) && p.x < Math.max(p1.x, p2.x)) {
                if (p.y <= Math.max(p1.y, p2.y)) {
                    if (p1.x == p2.x && p.y >= Math.min(p1.y, p2.y)) {
                        return boundOrVertex;
                    }

                    if (p1.y == p2.y) {
                        if (p1.y == p.y) {
                            return boundOrVertex;
                        } else {
                            ++intersectCount;
                        }
                    } else {
                        double xinters = (p.x - p1.x) * (p2.y - p1.y) / (p2.x - p1.x) + p1.y;
                        if (Math.abs(p.y - xinters) < precision) {
                            return boundOrVertex;
                        }

                        if (p.y < xinters) {
                            ++intersectCount;
                        }
                    }
                }
            } else {
                if (p.x == p2.x && p.y <= p2.y) {
                    Point2D.Double p3 = pts.get((i + 1) % N);
                    if (p.x >= Math.min(p1.x, p3.x) && p.x <= Math.max(p1.x, p3.x)) {
                        ++intersectCount;
                    } else {
                        intersectCount += 2;
                    }
                }
            }
            p1 = p2;
        }

        if (intersectCount % 2 == 0) {
            return false;
        } else {
            return true;
        }
    }

    public static Map<Object, List<Map<String, Object>>> groupByData(List<Map<String, Object>> list, String key) {
        Map<Object, List<Map<String, Object>>> result = null;
        if (list != null) {
            result = list
                    .stream().collect(Collectors.groupingBy(yl -> {
                        return yl.get(key);
                    }));
        }
        return result;
    }

    private Map<Object, List<Object>> handleUnitMap(Map<String, List<Map<String, Object>>> map) {
        Map<Object, List<Object>> result = new HashMap<>();

        List<Map<String, Object>> pacOne = map.get("pacOne");
        List<Map<String, Object>> pamTwo = map.get("pamTwo");
        List<Map<String, Object>> gmsjTwo = map.get("gmsjTwo");
        List<Map<String, Object>> yl = map.get("yl");
        List<Map<String, Object>> pacTwo = map.get("pacTwo");

        if (pacOne != null && pacOne.size() > 0) {
            handleMap(result, pacOne, "time", "pacOne");
        }
        if (pamTwo != null && pamTwo.size() > 0) {
            handleMap(result, pamTwo, "time", "pam");
        }
        if (gmsjTwo != null && gmsjTwo.size() > 0) {
            handleMap(result, gmsjTwo, "time", "gmsj");
        }
        if (yl != null && yl.size() > 0) {
            handleMap(result, yl, "time", "yl");
        }
        if (pacTwo != null && pacTwo.size() > 0) {
            handleMap(result, pacTwo, "time", "pacTwo");
        }

        return result;
    }

    private void handleMap(Map<Object, List<Object>> result, List<Map<String, Object>> unit, String key, String name) {
        unit.forEach(m -> {
            m.put("name", name);
            if (result.containsKey(m.get(key))) {
                result.get(m.get(key)).add(m);
            } else {
                List<Object> list = new ArrayList<>();
                list.add(m);
                result.put(m.get(key), list);
            }
        });
    }

    /**
     * 获取不同级别分区得漏损信息
     * @param dmaLevel
     * @return
     */
    public List<Map<String,Object>> getLevelDmaLeakList(Integer dmaLevel) {
        List<Map<String,Object>> dmaList = customDisplayMapper.getDmaByLevel(dmaLevel);
        for (Map<String,Object> one : dmaList){
            Long dmaId = Long.parseLong(String.valueOf(one.get("id")));
            Calendar calendar = Calendar.getInstance();
            calendar.set(Calendar.YEAR, calendar.get(Calendar.YEAR) - 1);
            String startMonth = new SimpleDateFormat( "yyyy-MM").format(calendar.getTime());
            calendar.set(Calendar.YEAR, calendar.get(Calendar.YEAR) + 1);
            calendar.set(Calendar.MONTH, calendar.get(Calendar.MONTH) - 1);
            String endMonth = new SimpleDateFormat( "yyyy-MM").format(calendar.getTime());

            WisdomDisplayDto wis = new WisdomDisplayDto();
            wis.setDmaIdOne(dmaId);
            wis.setStartMonth(startMonth);
            wis.setEndMonth(endMonth);
            if(dmaLevel == 3){
                one.put("data",customDisplayMapper.getLevelDmaLeak(wis));
            }else {
                one.put("leakageRate",customDisplayMapper.getLevelDmaLeakAvg(wis));
            }
        }

        return dmaList;
    }

    /**
     * date2比date1多的天数
     * @param date1
     * @param date2
     * @return
     */
    private static int differentDays(Date date1,Date date2) {
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(date1);

        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(date2);
        int day1 = cal1.get(Calendar.DAY_OF_YEAR);
        int day2 = cal2.get(Calendar.DAY_OF_YEAR);

        int year1 = cal1.get(Calendar.YEAR);
        int year2 = cal2.get(Calendar.YEAR);
        if (year1 != year2) {//同一年
            int timeDistance = 0;
            for (int i = year1; i < year2; i++) {
                if (i % 4 == 0 && i % 100 != 0 || i % 400 == 0)    //闰年
                {
                    timeDistance += 366;
                } else    //不是闰年
                {
                    timeDistance += 365;
                }
            }

            return timeDistance + (day2 - day1);
        } else {// 不同年
            System.out.println("判断day2 - day1 : " + (day2 - day1));
            return day2 - day1;
        }
    }

    /**
     * 计算某月字符串是否符合标准时间规范
     *
     * @param
     * @return BigDecimal
     */
    public String getTrueDate(String year, String month) {
        if(Arrays.asList(month.split(",")).size() == 1){
            month = "0" + month;
        }
        if(month.equals("13")){
            return (Integer.parseInt(year) + 1) + "-01";
        }else if(month.equals("0")){
            return (Integer.parseInt(year) - 1) + "-12";
        }
        return year + "-" +month;
    }

    /**
     * 计算同比环比指标差率
     *
     * @param
     * @return BigDecimal
     */
    public float getIndexRate(String indexName, Map<String, Object> mainMap, Map<String, Object> partMap) {
        if(mainMap == null || partMap == null){
            return 0.00f;
        }
        if(partMap.get(indexName) != null && mainMap.get(indexName) != null){
            BigDecimal nowIndex = new BigDecimal(String.valueOf(mainMap.get(indexName)));
            BigDecimal lastIndex = new BigDecimal(String.valueOf(partMap.get(indexName)));
            BigDecimal diff;
            diff = nowIndex.subtract(lastIndex);
            if(!lastIndex.equals(BigDecimal.ZERO) && !diff.equals(BigDecimal.ZERO)){
                return diff.divide(lastIndex, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
            }
        }
        return 0.00f;
    }

    /**
     * 计算同比环比指标差值
     *
     * @param
     * @return BigDecimal
     */
    public BigDecimal getIndexValue(String indexName, Map<String, Object> mainMap, Map<String, Object> partMap) {
        if(mainMap == null || partMap == null){
            return BigDecimal.ZERO;
        }
        if(partMap.get(indexName) != null && mainMap.get(indexName) != null){
            BigDecimal nowIndex = new BigDecimal(String.valueOf(mainMap.get(indexName)));
            BigDecimal lastIndex = new BigDecimal(String.valueOf(partMap.get(indexName)));
            return nowIndex.subtract(lastIndex);

        }
        return BigDecimal.ZERO;
    }

    /**
     * 计算同比环比率
     *
     * @param
     * @return BigDecimal
     */
    public float getMainIndexRate(String indexName, Map<String, Object> mainMap, Map<String, Object> partMap) {
        if(mainMap == null || partMap == null){
            return 0.00f;
        }
        if(partMap.get(indexName) != null && mainMap.get(indexName) != null){
            BigDecimal nowIndex = new BigDecimal(String.valueOf(mainMap.get(indexName)));
            BigDecimal lastIndex = new BigDecimal(String.valueOf(partMap.get(indexName)));
            if(!nowIndex.equals(BigDecimal.ZERO) && !lastIndex.equals(BigDecimal.ZERO)){
                return nowIndex.divide(lastIndex, 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100)).floatValue();
            }
        }
        return 0.00f;
    }

    /**
     * 获取漏损率和产销差平均值
     * @param nearYearStaticalList
     * @return
     */
    public Map<String,Double> getLeakNrwAvg(List<Map<String,Double>> nearYearStaticalList) {
        Map<String,Double> leakNrwAvgMap = new HashMap<>();
        double leakRate = 0.0;
        double poorProductionRate = 0.0;
        int leakRateCount = 0;
        int poorProductionRateCount = 0;
        for(Map<String,Double> one : nearYearStaticalList){
            if(one.get("leakageRate") != null){
                leakRate += Double.parseDouble(String.valueOf(one.get("leakageRate")));
                leakRateCount ++;
            }
            if(one.get("poorProductionRate") != null){
                poorProductionRate += Double.parseDouble(String.valueOf(one.get("poorProductionRate")));
                poorProductionRateCount ++;
            }
        }
        if(leakRateCount != 0){
            leakRate = new BigDecimal(leakRate / leakRateCount).setScale(2, RoundingMode.HALF_UP).doubleValue();
        }
        if(poorProductionRateCount != 0){
            poorProductionRate = new BigDecimal(poorProductionRate / poorProductionRateCount).setScale(2, RoundingMode.HALF_UP).doubleValue();
        }

        leakNrwAvgMap.put("leakageRate",leakRate);
        leakNrwAvgMap.put("poorProductionRate",poorProductionRate);
        return leakNrwAvgMap;
    }

    /**
     * 计算热线好评率(type:1年;2月)
     * @param
     * @return
     */
    public float getHotLineRate(String time,String type) {
        if(StringUtils.isBlank(time) || StringUtils.isBlank(type)){
            return 0.00f;
        }
        WisdomDisplayDto wisdomDisplayDto = new WisdomDisplayDto();
        if(type.equals("1")){
            wisdomDisplayDto.setCensusYear(time);
        }else if(type.equals("2")){
            wisdomDisplayDto.setCensusMonth(time);
        }else {
            return 0.00f;
        }
        List<String> statusList = new ArrayList<>();
        statusList.add("1");
        statusList.add("2");
        wisdomDisplayDto.setStatusList(statusList);
        List<String> goodScores = pltCalllist1000Mapper.getConnectIds(wisdomDisplayDto);
        Long goodNum = 0L;
        if(goodScores.size() != 0){
            wisdomDisplayDto.setConnectIds(goodScores);
            goodNum = pltCalllist1000Mapper.viewNewHotLineNum(wisdomDisplayDto);
        }
        statusList.add("3");
        statusList.add("4");
        wisdomDisplayDto.setStatusList(statusList);
        List<String> scores = pltCalllist1000Mapper.getConnectIds(wisdomDisplayDto);
        Long allNum = 0L;
        if(scores.size() != 0){
            wisdomDisplayDto.setConnectIds(scores);
            allNum = pltCalllist1000Mapper.viewNewHotLineNum(wisdomDisplayDto);
        }
        if(goodNum == 0 || allNum == 0){
            return 0.00f;
        }
        return new BigDecimal((float)goodNum / allNum).multiply(BigDecimal.valueOf(100)).setScale(2, RoundingMode.HALF_UP).floatValue();
    }

    /**
     * 获取指标报警的最低等级上下限
     * @param scadaReportList 报警信息列表
     * @return
     */
    private Map<String,Double> getReportUpperLower(List<Map<String,Object>> scadaReportList){
        if(scadaReportList == null || scadaReportList.size() == 0){
            return null;
        }
        Map<String,Double> res = new HashMap<>();
        Double reportLower = null;
        Double reportUpper = null;
        double mid;
        for (Map<String, Object> scadaReport : scadaReportList) {
            if(scadaReport.get("reportUpper") != null && scadaReport.get("reportLower") != null){
                if(reportLower == null){
                    reportLower = Double.parseDouble(String.valueOf(scadaReport.get("reportLower")));
                }else {
                    mid = Double.parseDouble(String.valueOf(scadaReport.get("reportLower")));
                    if(reportLower.compareTo(mid) > 0){
                        reportLower = mid;
                    }
                }
                if(reportUpper == null){
                    reportUpper = Double.parseDouble(String.valueOf(scadaReport.get("reportUpper")));
                }else {
                    mid = Double.parseDouble(String.valueOf(scadaReport.get("reportUpper")));
                    if(reportUpper.compareTo(mid) > 0){
                        reportUpper = mid;
                    }
                }
            }
        }
        if(reportLower == null || reportUpper == null){
            return null;
        }
        res.put("reportLower",reportLower);
        res.put("reportUpper",reportUpper);
        return res;
    }

}
