/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.rushrepair;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.rushrepair.IRushrepairService;
import com.tofly.pojo.rushrepair.Rushrepair;
import com.tofly.pojo.rushrepair.dto.RushrepairDto;
import com.tofly.pojo.rushrepair.vo.RushrepairVo;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.*;

/**
 * 抢修上报信息表
 *
 * @author sunjian
 * @date Tue Mar 21 00:00:00 CST 2023
 */
@RestController
@RequestMapping("/rushrepair")
@Api(tags = "抢修上报信息表接口")
@Slf4j
public class RushrepairController extends BaseController<Rushrepair, RushrepairVo> {

    @Resource
    private IRushrepairService IRushrepairService;

    @Autowired
    public RushrepairController(IRushrepairService IRushrepairService) {
        init(IRushrepairService);
    }

    /**
     * 分页查询
     *
     * @param page          分页对象
     * @param rushrepairDto 抢修上报信息表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getRushrepairPage(Page page, @ModelAttribute RushrepairDto rushrepairDto) {
        ResultRespone<Page<RushrepairVo>> resultRespone = new ResultRespone<>();
        try {
            Page<RushrepairVo> rushRepairPage = IRushrepairService.getRushRepairPage(page, rushrepairDto);
            resultRespone.setResult(rushRepairPage);
        } catch (BusinessException businessException) {
            resultRespone.setCode(businessException.getRetCd());
            resultRespone.setMessage(businessException.getMsgDes());
        }
        return resultRespone;
    }


    /**
     * 通过id查询抢修上报信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        ResultRespone<RushrepairVo> resultRespone = new ResultRespone<>();
        try {
            RushrepairVo rushRepairById = IRushrepairService.getRushRepairById(id);
            resultRespone.setResult(rushRepairById);
        } catch (BusinessException businessException) {
            resultRespone.setMessage(businessException.getMsgDes());
            resultRespone.setCode(businessException.getRetCd());
        }
        return resultRespone;
    }

    /**
     * 新增抢修上报信息表
     *
     * @param rushrepairDto 抢修上报信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增抢修上报信息表")
    @ApiOperation(value = "新增抢修上报信息表")
    @PostMapping
    public ResultRespone save(@Valid RushrepairDto rushrepairDto,
                              @RequestParam(value = "files", required = false) @ApiParam("抢修附件列表") List<MultipartFile> files,
                              @RequestParam(value = "auditFiles", required = false) @ApiParam("抢修审核附件列表") List<MultipartFile> auditFiles,
                              BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                IRushrepairService.savaBuildInfo(files, auditFiles, rushrepairDto);
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;
    }

    @ApiOperation(value = "新增基本信息对象Feign调用")
    @PostMapping("/saveFeign")
    public ResultRespone<Object> saveRushRepairFeign(@RequestBody RushrepairDto rushrepairDto,
                                                     @RequestParam(value = "files", required = false) @ApiParam("抢修附件列表") List<MultipartFile> files,
                                                     @RequestParam(value = "auditFiles", required = false) @ApiParam("抢修审核附件列表") List<MultipartFile> auditFiles) {
        ResultRespone<Object> rest = new ResultRespone<>();
        try {
            log.info("新增工单热线传入对象 {}", JSON.toJSONString(rushrepairDto));

            Rushrepair rushrepair = IRushrepairService.savaBuildInfo(files, auditFiles, rushrepairDto);
            rest.setResult(rushrepair.getId());
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        } catch (Exception e) {
            e.printStackTrace();
        }

        return rest;
    }

    /**
     * 修改抢修上报信息表
     *
     * @param rushrepairDto 抢修上报信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改抢修上报信息表")
    @ApiOperation(value = "修改抢修上报信息表")
    @PutMapping
    public ResultRespone updateById(@Valid @RequestBody RushrepairDto rushrepairDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return super.updateById(JSON.parseObject(JSON.toJSONString(rushrepairDto), Rushrepair.class));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;
    }

    /**
     * 通过id批量删除抢修上报信息表（自动删除审核表信息）
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除抢修上报信息表（自动删除审核表信息）")
    @ApiOperation(value = "通过ID批量删除抢修上报信息表（自动删除审核表信息）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        ResultRespone<Object> resultRespone = new ResultRespone<>();
        try {
            return IRushrepairService.removeByIdList(ids);
        } catch (BusinessException e) {
            resultRespone.setCode(e.getRetCd());
            resultRespone.setMessage(e.getMsgDes());
        }
        return resultRespone;
    }

    /**
     * 查询抢修单sid信息
     */
    @ToFlyAppLog(title = "查询抢修单sid-layerid信息")
    @ApiOperation(value = "查询抢修单sid-layerid信息")
    @GetMapping("/getSidSelect")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "typeid", value = "抢修类型id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始日期", required = false, dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束日期", required = false, dataType = "String"),
    })
    public ResultRespone getSidSelect(String typeid, String startTime, String endTime) {
        Map map = new HashMap();
        map.put("startTime", startTime);
        map.put("endTime", endTime);
        return ResultRespone.success(IRushrepairService.getSidSelect(typeid, map));
    }

}
