/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.rushrepair;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.rushrepair.IRushrepairauditService;
import com.tofly.pojo.rushrepair.Rushrepairaudit;
import com.tofly.pojo.rushrepair.dto.RushrepairauditDto;
import com.tofly.pojo.rushrepair.vo.RushrepairauditVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.Optional;

/**
 * 抢修信息审核表
 *
 * @author sunjian
 * @date Tue Mar 21 00:00:00 CST 2023
 */
@RestController
@RequestMapping("/rushrepairaudit")
@Api(tags = "抢修信息审核表接口")
public class RushrepairauditController extends BaseController<Rushrepairaudit, RushrepairauditVo> {

    @Resource
    private IRushrepairauditService IRushrepairauditService;

    @Autowired
    public RushrepairauditController(IRushrepairauditService IRushrepairauditService) {
        init(IRushrepairauditService);
    }

    /**
     * 分页查询
     *
     * @param page               分页对象
     * @param rushrepairauditDto 抢修信息审核表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getRushrepairauditPage(Page page, @ModelAttribute RushrepairauditDto rushrepairauditDto) {
        ResultRespone<Page<RushrepairauditVo>> resultRespone = new ResultRespone<>();
        try {
            Page<RushrepairauditVo> rushRepairPage = IRushrepairauditService.getRushRepairAuditPage(page, rushrepairauditDto);
            resultRespone.setResult(rushRepairPage);
        } catch (BusinessException businessException) {
            resultRespone.setCode(businessException.getRetCd());
            resultRespone.setMessage(businessException.getMsgDes());
        }
        return resultRespone;
    }


    /**
     * 通过id查询抢修信息审核表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        ResultRespone<RushrepairauditVo> resultRespone = new ResultRespone<>();
        try {
            return super.getById(id);
        } catch (BusinessException businessException) {
            resultRespone.setMessage(businessException.getMsgDes());
            resultRespone.setCode(businessException.getRetCd());
        }
        return resultRespone;
    }

    /**
     * 新增抢修信息审核表
     *
     * @param rushrepairauditDto 抢修信息审核表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增抢修信息审核表")
    @ApiOperation(value = "新增抢修信息审核表")
    @PostMapping
    public ResultRespone save(@Valid @RequestBody RushrepairauditDto rushrepairauditDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return super.save(JSON.parseObject(JSON.toJSONString(rushrepairauditDto), Rushrepairaudit.class));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;
    }

    /**
     * 修改抢修信息审核表
     *
     * @param rushrepairauditDto 抢修信息审核表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改抢修信息审核表")
    @ApiOperation(value = "修改抢修信息审核表")
    @PutMapping
    public ResultRespone updateById(@Valid @RequestBody RushrepairauditDto rushrepairauditDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return super.updateById(JSON.parseObject(JSON.toJSONString(rushrepairauditDto), Rushrepairaudit.class));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;
    }

    /**
     * 通过id批量删除抢修信息审核表
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除抢修信息审核表")
    @ApiOperation(value = "通过ID批量删除抢修信息审核表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        ResultRespone<Object> resultRespone = new ResultRespone<>();
        try {
            return super.removeByIdList(ids);
        } catch (BusinessException e) {
            resultRespone.setCode(e.getRetCd());
            resultRespone.setMessage(e.getMsgDes());
        }
        return resultRespone;
    }
}
