/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.rushrepair;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.rushrepair.IRushrepairmaterialService;
import com.tofly.pojo.rushrepair.Rushrepairmaterial;
import com.tofly.pojo.rushrepair.dto.RushrepairmaterialDto;
import com.tofly.pojo.rushrepair.vo.RushrepairmaterialVo;
import com.tofly.pojo.rushrepair.vo.RushrepairVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.annotation.Resource;
import javax.validation.Valid;

import java.util.Optional;

/**
 * 抢修耗材
 *
 * @author sunjian
 * @date Thu Apr 13 00:00:00 CST 2023
 */
@RestController
@RequestMapping("/rushrepairmaterial")
@Api(tags="抢修耗材接口")
public class RushrepairmaterialController extends BaseController<Rushrepairmaterial, RushrepairmaterialVo> {

    @Resource
    private IRushrepairmaterialService rushrepairMaterialServiceI;

    @Autowired
    public RushrepairmaterialController(IRushrepairmaterialService rushrepairMaterialServiceI) {
        init(rushrepairMaterialServiceI);
    }

    /**
    * 分页查询
    * @param page 分页对象
    * @param rushrepairMaterialDto 抢修耗材
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getRushrepairMaterialPage(Page page, @ModelAttribute RushrepairmaterialDto rushrepairMaterialDto) {
        ResultRespone<Page<RushrepairmaterialVo>> resultRespone = new ResultRespone<>();
        try {
            Page<RushrepairmaterialVo> rushrepairMaterialVoPage = rushrepairMaterialServiceI.getRushrepairMaterialPage(page, rushrepairMaterialDto);
            resultRespone.setResult(rushrepairMaterialVoPage);
        } catch (BusinessException businessException) {
            resultRespone.setCode(businessException.getRetCd());
            resultRespone.setMessage(businessException.getMsgDes());
        }
        return resultRespone;
    }


    /**
     * 通过id查询抢修耗材
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
        ResultRespone<RushrepairVo> resultRespone = new ResultRespone<>();
        try {
            return super.getById(id);
        } catch (BusinessException businessException) {
            resultRespone.setMessage(businessException.getMsgDes());
            resultRespone.setCode(businessException.getRetCd());
        }
        return resultRespone;
    }

    /**
     * 新增抢修耗材
     * @param rushrepairMaterialDto 抢修耗材
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增抢修耗材" )
    @ApiOperation(value = "新增抢修耗材")
    @PostMapping
    public ResultRespone<Object> save(@Valid @RequestBody RushrepairmaterialDto rushrepairMaterialDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return super.save(JSON.parseObject(JSON.toJSONString(rushrepairMaterialDto), Rushrepairmaterial.class));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;
    }

    /**
     * 修改抢修耗材
     * @param rushrepairMaterialDto 抢修耗材
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改抢修耗材" )
    @ApiOperation(value = "修改抢修耗材")
    @PutMapping
    public ResultRespone updateById(@RequestBody RushrepairmaterialDto rushrepairMaterialDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                return super.updateById(JSON.parseObject(JSON.toJSONString(rushrepairMaterialDto), Rushrepairmaterial.class));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;
    }

    /**
     * 通过id批量删除抢修耗材
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除抢修耗材" )
    @ApiOperation(value = "通过ID批量删除抢修耗材")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        ResultRespone<Object> resultRespone = new ResultRespone<>();
        try {
            return super.removeByIdList(ids);
        } catch (BusinessException e) {
            resultRespone.setCode(e.getRetCd());
            resultRespone.setMessage(e.getMsgDes());
        }
        return resultRespone;
    }
}
