/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.rushrepair;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.gpsboot.service.rushrepair.IRushrepairtypeService;
import com.tofly.pojo.rushrepair.Rushrepairtype;
import com.tofly.pojo.rushrepair.vo.RushrepairtypeVo;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.annotation.Resource;
import java.util.Arrays;
/**
 * 抢修类型
 *
 * @author sunjian
 * @date Fri Jul 21 00:00:00 CST 2023
 */
@RestController
@RequestMapping("/rushrepairtype")
@Api(tags="抢修类型接口")
@Slf4j
public class RushrepairtypeController extends BaseController<Rushrepairtype, RushrepairtypeVo> {

    @Resource
    private IRushrepairtypeService rushrepairtypeService;

    @Autowired
    public RushrepairtypeController(IRushrepairtypeService rushrepairtypeServiceI) {
        init(rushrepairtypeServiceI);
    }
    /**
    * 分页查询
    * @param page 分页对象
    * @param rushrepairtype 抢修类型
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getRushrepairtypePage(Page page, @ModelAttribute Rushrepairtype rushrepairtype) {
      return ResultRespone.success(rushrepairtypeService.page(page, Wrappers.query(rushrepairtype)));
    }


    /**
     * 通过id查询抢修类型
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(rushrepairtypeService.getById(id));
    }

    /**
     * 新增抢修类型
     * @param rushrepairtype 抢修类型
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增抢修类型" )
    @ApiOperation(value = "新增抢修类型")
    @PostMapping
    public ResultRespone save(@RequestBody Rushrepairtype rushrepairtype) {
      return ResultRespone.success(rushrepairtypeService.save(rushrepairtype));
    }

    /**
     * 修改抢修类型
     * @param rushrepairtype 抢修类型
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改抢修类型" )
    @ApiOperation(value = "修改抢修类型")
    @PutMapping
    public ResultRespone updateById(@RequestBody Rushrepairtype rushrepairtype) {
      return ResultRespone.success(rushrepairtypeService.updateById(rushrepairtype));
    }

    /**
     * 通过id删除抢修类型
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除抢修类型" )
    @ApiOperation(value = "通过ID删除抢修类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(rushrepairtypeService.removeById(id));
    }

    /**
     * 通过id批量删除抢修类型
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除抢修类型" )
    @ApiOperation(value = "通过ID批量删除抢修类型")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(rushrepairtypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
