/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.trouble;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.trouble.ToublerangeService;

import com.tofly.pojo.trouble.Toublerange;
import com.tofly.pojo.trouble.Toubletype;
import com.tofly.pojo.trouble.dto.ToublerangeDto;

import com.tofly.pojo.trouble.vo.ToublerangeVo;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;
import java.util.Optional;

/**
 * 隐患等级表
 *
 * @author admin
 * @date Fri Aug 06 00:00:00 CST 2021
 */
@RestController
@RequestMapping("/toublerange")
@Api(tags="隐患等级表接口")
public class ToublerangeController extends BaseController<Toublerange, ToublerangeVo> {

    @Autowired
    private ToublerangeService service;

    @Autowired
    public ToublerangeController(ToublerangeService service) {
        init(service);
    }

    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ToFlyAppLog(title = "分页查询隐患等级信息")
    public ResultRespone<Page<ToublerangeVo>> getToublerangePage(Page page, @ModelAttribute ToublerangeDto parDto) {
        return super.getInfoPage(page, JSON.parseObject(JSON.toJSONString(parDto), Toublerange.class));
    }

    @ApiOperation(value = "新增基本信息对象")
    @PostMapping("/save")
    @ToFlyAppLog(title = "新增隐患等级信息")
    public ResultRespone<Object> saveToublerange(@Valid @RequestBody ToublerangeDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        return Optional.ofNullable(super.validRest(rest, result)).orElseGet(() -> super.save(JSON.parseObject(JSON.toJSONString(parDto), Toublerange.class)));

    }

    @ApiOperation(value = "修改基本信息")
    @PutMapping("/updata")
    @ToFlyAppLog(title = "修改隐患等级信息")
    public ResultRespone<Object> updateByIdToublerange(@Valid @RequestBody ToublerangeDto parDto, BindingResult result) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                super.updateById(JSON.parseObject(JSON.toJSONString(parDto), Toublerange.class));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }

        }
        return rest;

    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @ToFlyAppLog(title = "通过ID查询隐患等级信息")
    public ResultRespone<Toublerange> getByIdToublerange(@PathVariable Long id) {
        return super.getById(id);
    }


    @ApiOperation(value = "通过ID批量删除基本信息对象")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    @ToFlyAppLog(title = "通过ID批量删除隐患等级信息")
    public ResultRespone<Object> removeByIdListToublerange(String ids) {
        ResultRespone<Object> rest = new ResultRespone<>();
        try {
            service.deleteToublerangeInfo(ids);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }

}
