/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.trouble;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;

import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.pojo.trouble.dto.TroubleDto;
import com.tofly.pojo.trouble.vo.TroubleVo;

import com.tofly.pojo.trouble.vo.detail.TroubleDetailVo;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.trouble.TroubleService;
import com.tofly.pojo.trouble.Trouble;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 隐患上报信息表
 *
 * @author admin
 * @date Thu Mar 18 00:00:00 CST 2021
 */
@RestController
@RequestMapping("/trouble")
@Api(tags = "隐患上报信息表接口")
public class TroubleController extends BaseController<Trouble, TroubleVo> {
    @Autowired
    private TroubleService service;

    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    @ToFlyAppLog(title = "分页查询隐患上报信息")
    public ResultRespone<Page<Trouble>> getTroublePage(Page page, @ModelAttribute TroubleDto parDto) {
        ResultRespone<Page<Trouble>> rest = new ResultRespone<>();
        Page<Trouble> buildPage = service.getBuildPage(page, parDto);
        rest.setResult(buildPage);
        return rest;
    }

    @ApiOperation(value = "新增基本信息对象")
    @PostMapping(value = "/save")
    @ToFlyAppLog(title = "新增隐患上报信息")
    public ResultRespone<Object> saveTrouble(@Valid TroubleDto parDto, BindingResult result,@RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回
        service.savaBuildInfo(fileList, parDto);

        return rest;
    }

    @ApiOperation(value = "修改基本信息")
    @PutMapping("/updata")
    @ToFlyAppLog(title = "修改隐患上报信息")
    public ResultRespone<Object> updateByIdTrouble(@Valid TroubleDto parDto, BindingResult result,@RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        // 数据校验 判空返回

            try {
                service.updateTroubleInfo(fileList, parDto);
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }catch (Exception exception){
                exception.printStackTrace();
            }

        return rest;

    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @ToFlyAppLog(title = "通过ID查询隐患上报信息")
    public ResultRespone<Trouble> getByIdTrouble(@PathVariable Long id) {
        return ResultRespone.success(service.getById(id));
    }

    @GetMapping("/getTroubleDetail")
    @ApiOperation(value = "通过ID,flag查询隐患上报详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
            @ApiImplicitParam(name = "flag", value = "是否展示部门下所有数据 是则填1 个人不填为空", required = false, dataType = "String")
    })
    @ToFlyAppLog(title = "通过ID,flag查询隐患上报详情")
    public ResultRespone<TroubleDetailVo> getTroubleDetail(Long id,String flag) {
        ResultRespone<TroubleDetailVo> rest = new ResultRespone<>();
        try {
            TroubleDetailVo info = service.getTroubleInfoDetail(id,flag);
            rest.setResult(info);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }catch (Exception exception){
            exception.printStackTrace();
        }
        return rest;
    }

    @ApiOperation(value = "通过ID批量删除基本信息对象")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    @ToFlyAppLog(title = "通过ID批量删除隐患上报信息")
    public ResultRespone<Object> removeByIdListTrouble(String ids) {
        ResultRespone<Object> rest = new ResultRespone<>();
        List<Long> removeId = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        try {

            service.deleteTroubleInfo(removeId);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }catch (Exception exception){
            exception.printStackTrace();
        }
        return rest;
    }
}
