/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.gpsboot.controller.valvewell;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.constant.ResponeStatus;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.controller.BaseController;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.pojo.plan.Valvewell;
import com.tofly.gpsboot.service.valvewell.ValvewellService;
import com.tofly.pojo.plan.dto.ValvewellDto;
import com.tofly.pojo.plan.vo.ValvewellVo;

import com.tofly.pojo.statement.dto.StatementDto;
import com.tofly.pojo.statement.vo.InspectionStatisticsVo;
import com.tofly.pojo.valewell.dto.valewellStaDto;
import com.tofly.pojo.valewell.vo.valewellStaVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 阀门井信息表
 *
 * @author admin
 * @date Wed Mar 29 00:00:00 GMT+08:00 2023
 */
@RestController
@RequestMapping("/valvewell")
@Api(tags = "阀门井信息表接口")
public class ValvewellController extends BaseController<Valvewell, ValvewellVo> {

    @Autowired
    private ValvewellService valvewellService;

    @Autowired
    public ValvewellController(ValvewellService service) {
        init(service);
    }


    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getValvewellPage(Page page, @ModelAttribute ValvewellDto parDto) {
        ResultRespone<Page<ValvewellVo>> rest = new ResultRespone<>();
        try {
            Page<ValvewellVo> valvewellVoPage = valvewellService.getValvewellPage(page, parDto);

            rest.setResult(valvewellVoPage);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone<ValvewellVo> getValvewellInfo(@PathVariable Long id) {
        ResultRespone<ValvewellVo> rest = new ResultRespone<>();
        try {
            ValvewellVo baseInfo = valvewellService.getValvewellInfo(id);
            if (null == baseInfo) {
                rest.setMessage(Constant.DATA_NONENTITY);
                rest.setCode(Constant.DATA_NONENTITY_CODE);
            }

            rest.setResult(baseInfo);
        } catch (Exception e) {
            rest.setCode(ResponeStatus.FAIL.getCode());
        }
        return rest;
    }


    @ToFlyAppLog(title = "新增阀门井信息表")
    @ApiOperation(value = "新增阀门井信息表")
    @PostMapping("/save")
    public ResultRespone savaValvewellInfo(@Valid ValvewellDto parDto, BindingResult result, @RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                rest.setResult(valvewellService.savaValvewellInfo(fileList, parDto));
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }

        }
        return rest;
    }

    @ToFlyAppLog(title = "修改阀门井信息表")
    @ApiOperation(value = "修改阀门井信息表")
    @PutMapping("/updata")
    public ResultRespone updateValvewellInfo(@Valid ValvewellDto parDto, BindingResult result, @RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        ResultRespone<Object> rest = new ResultRespone<>();
        if (!Optional.ofNullable(super.validRest(rest, result)).isPresent()) {
            try {
                valvewellService.updateValvewellInfo(fileList, parDto);
            } catch (BusinessException e) {
                rest.setCode(e.getRetCd());
                rest.setMessage(e.getMsgDes());
            }
        }
        return rest;
    }


    @ToFlyAppLog(title = "通过id批量删除阀门井信息表")
    @ApiOperation(value = "通过ID批量删除阀门井信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        ResultRespone<Object> rest = new ResultRespone<>();
        List<Long> removeId = Arrays.stream(ids.split(Constant.CSV)).map(s -> Long.parseLong(s.trim())).collect(Collectors.toList());
        try {
            valvewellService.deleteValvewellInfo(removeId);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }

    @ApiOperation(value = "阀门巡检台账")
    @PostMapping("/getValvewellSta")
    @ToFlyAppLog(title = "阀门巡检次数台账")
    public ResultRespone<Object> getValvewellSta(@RequestBody valewellStaDto dto) {
        ResultRespone<Object> rest = new ResultRespone<>();
        try {
            List<valewellStaVo> valvewellSta = valvewellService.getValvewellSta(dto);
            rest.setResult(valvewellSta);
        } catch (BusinessException e) {
            rest.setCode(e.getRetCd());
            rest.setMessage(e.getMsgDes());
        }
        return rest;
    }

}
