package com.tofly.gpsboot.service.audit.impl;

import com.tofly.api.GdglinfoClient;
import com.tofly.api.UserTaskClient;
import com.tofly.common.core.util.StringUtil;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.common.system.*;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.audit.IAuditRushRepairAuditServer;
import com.tofly.gpsboot.service.comm.IFileInfoService;
import com.tofly.gpsboot.service.mq.IAuditMessageService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairAndMaterialService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairAndSidService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairauditService;
import com.tofly.gpsboot.util.DateTimeUtil;
import com.tofly.pojo.message.Usertask;
import com.tofly.pojo.plan.dto.ReportAuditDto;
import com.tofly.pojo.rushrepair.Rushrepair;
import com.tofly.pojo.rushrepair.Rushrepairaudit;
import com.tofly.pojo.rushrepair.dto.RushrepairDto;
import com.tofly.pojo.rushrepair.dto.RushrepairauditDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

/**
 * 抢修 审核
 *
 * @author sunjian
 * @date 2023/3/24
 */
@Service("AUDIT_TYPE_RUSH_REPAIR_AUDIT")
public class AuditRushRepairAuditServerImpl implements IAuditRushRepairAuditServer {

    @Autowired
    private IRushrepairService IRushrepairService;
    @Autowired
    private IRushrepairauditService IRushrepairauditService;
    @Autowired
    private IAuditMessageService iAuditMessageService;
    @Autowired
    private UserTaskClient userTaskClient;
    /**
     * 获取文件服务
     */
    @Autowired
    private IFileInfoService iFileInfoService;
    @Value("${xrty.ftp.rushRepairAudit}")
    private String rushRepairAuditFilePath;

    @Autowired
    private IRushrepairAndMaterialService rushrepairAndMaterialService;

    @Autowired
    private IRushrepairAndSidService rushrepairAndSidService;

    @Autowired
    private GdglinfoClient gdglinfoClient;

    /**
     * 审核
     *
     * @param dto 审核数据
     * @throws BusinessException
     */
    @Override
    public void audit(ReportAuditDto dto) throws BusinessException {

        //region 对象初始化
        Long audit_id = dto.getId();
        //审核对象
        Rushrepairaudit info_audit = IRushrepairauditService.getById(audit_id);
        if (null == info_audit) {
            throw new BusinessException(Constant.RUSH_REPAIR_AUDIT_NULL_CODE, Constant.RUSH_REPAIR_AUDIT_NULL);
        }
        Long id = info_audit.getRushrepairId();
        //主表对象
        Rushrepair info = IRushrepairService.getById(id);
        //主表dto对象
        RushrepairDto infodto = dto.getRushrepairDto();
        if (AuditSateEnum.AUDIT_YES.getCode().equals(info_audit.getAuditResult())) {
            throw new BusinessException(Constant.AUDIT_NOT_UNCOMMITTED_CODE, Constant.AUDIT_NOT_UNCOMMITTED);
        }
        //endregion

        //region 设置传入参数
        String auditResult = dto.getAuditResult();
        Long auditId = dto.getAuditId();
        Long auditDeptId = dto.getAuditDeptId();
        String auditNotes = dto.getAuditNotes();
        Date date = DateTimeUtil.convertLDTToDate(LocalDateTime.now());
        String rushRepairType = info.getState();
        String ischeck = null == infodto ? null : infodto.getIsCheck();
        String isnoprocess = null == infodto ? null : infodto.getIsNoprocess();
        Long rushRepairNextAuditId = null == infodto ? null : infodto.getRushRepairNextAuditId();
        Long rushRepairNextAuditDeptId = null == infodto ? null : infodto.getRushRepairNextAuditDeptId();
        //endregion

        //region 设置主、审核表对象
        //修改主表对象
        info.setHandleLgtd(null == infodto ? info.getHandleLgtd() : infodto.getHandleLgtd());
        info.setHandleLttd(null == infodto ? info.getHandleLttd() : infodto.getHandleLttd());
        info.setCheckLgtd(null == infodto ? info.getCheckLgtd() : infodto.getCheckLgtd());
        info.setCheckLttd(null == infodto ? info.getCheckLttd() : infodto.getCheckLttd());
        info.setAssistpersonDept(null == infodto ? info.getAssistpersonDept() : infodto.getAssistpersonDept());
        info.setAssistpersonDeptname(null == infodto ? info.getAssistpersonDeptname() : infodto.getAssistpersonDeptname());
        info.setAssistpersonUsername(null == infodto ? info.getAssistpersonUsername() : infodto.getAssistpersonUsername());
        info.setAssistpersonUser(null == infodto ? info.getAssistpersonUser() : infodto.getAssistpersonUser());
        info.setLaborCost(null == infodto ? info.getLaborCost() : infodto.getLaborCost());
        info.setRushrepairtypeId(null == infodto ? info.getRushrepairtypeId() : infodto.getRushrepairtypeId());
        info.setAddress(null == infodto ? info.getAddress() : infodto.getAddress());

        if (!StringUtil.isEmpty(isnoprocess)) {
            info.setIsNoprocess(isnoprocess);
        }

        //修改对象表对象
        info_audit.setAuditTime(date);
        info_audit.setAuditUser(auditId);
        info_audit.setAuditDept(auditDeptId);
        info_audit.setAuditResult(auditResult);
        info_audit.setAuditNotes(auditNotes);
        if (!StringUtil.isEmpty(rushRepairType)) {
            info_audit.setType(rushRepairType);
        }
        if (!StringUtil.isEmpty(ischeck)) {
            info_audit.setIsCheck(ischeck);
        }
        //endregion

        //region 审核附件上传
        StringBuilder fileIdList = new StringBuilder("");
        // 判断当前是否有文件,则存入返回文件id
        if (null != dto.getAuditFiles() && dto.getAuditFiles().size() > 0) {
            fileIdList = iFileInfoService.saveFileList(dto.getAuditFiles(), rushRepairAuditFilePath, FileTableEnum.TF_YWPN_RUSHREPAIRAUDIT_W.getTableName());
        }
        info_audit.setUploadFileids(fileIdList.toString());
        //endregion


        //region 设置流程下一步状态
        //流程下一步审核状态
        String next_type = getRushrepairauditType(info_audit.getType(), auditResult, ischeck, isnoprocess);
        if (StringUtil.isEmpty(next_type)) {
            throw new BusinessException(Constant.RUSH_REPAIR_AUDIT_BEFORE_TYPE_ERROR_CODE, Constant.RUSH_REPAIR_AUDIT_BEFORE_TYPE_ERROR);
        }
        //更新主表状态
        info.setState(next_type);
        //endregion

        //region 设置审核通过/不通过 添加下一步审核数据
        if (!RushRepairSateEnum.RUSHREPAIR_SATE_PLACE_FILE.getCode().equals(next_type) && (null == rushRepairNextAuditDeptId || null == rushRepairNextAuditId)) {
            throw new BusinessException(Constant.RUSH_REPAIR_AUDIT_NEXT_AUDITID_DEPT_ERROR_CODE, Constant.RUSH_REPAIR_AUDIT_NEXT_AUDITID_DEPT_ERROR);
        }
        //归档不用添加审核记录
        if (AuditSateEnum.AUDIT_YES.getCode().equals(auditResult) && !RushRepairSateEnum.RUSHREPAIR_SATE_PLACE_FILE.getCode().equals(next_type)) {

            //region 添加材料
            //处理材料
            if (null != infodto && null != infodto.getMaterialList() && infodto.getMaterialList().size() > 0) {
                rushrepairAndMaterialService.saveRushrepairAndMaterial(audit_id, infodto.getMaterialList());
            }
            //endregion

            //region 添加图层设备id
            //图层设备id
            if (null != infodto && null != infodto.getRushrepairAndSidList() && infodto.getRushrepairAndSidList().size() > 0) {
                rushrepairAndSidService.saveRushrepairAndSid(id, infodto.getRushrepairAndSidList());
            }
            //endregion

            //审核通过 新增一条下一步状态未审核记录
            RushrepairauditDto new_rushrepairauditdto = new RushrepairauditDto();
            new_rushrepairauditdto.setRushrepairId(id);
            new_rushrepairauditdto.setType(next_type);
            new_rushrepairauditdto.setAuditResult("0");
            new_rushrepairauditdto.setAuditDept(rushRepairNextAuditDeptId);
            new_rushrepairauditdto.setAuditUser(rushRepairNextAuditId);
            new_rushrepairauditdto.setBeforeAuditid(info_audit.getId());
            IRushrepairauditService.savaBuildInfo(null, new_rushrepairauditdto);
        } else if (AuditSateEnum.AUDIT_NO.getCode().equals(auditResult)) {
            //如果上一步id等于本步骤id 且在第一步 进行驳回 不用增加审核记录
            if (!RushRepairSateEnum.RUSHREPAIR_SATE_SAFE_AUDIT.getCode().equals(rushRepairType) && !RushRepairSateEnum.RUSHREPAIR_SATE_TO_BE_SUBMIT.getCode().equals(next_type)) {
                //审核不通过-返回到上一步 新增一条上一步状态未审核记录
                RushrepairauditDto new_rushrepairauditdto = new RushrepairauditDto();
                new_rushrepairauditdto.setRushrepairId(id);
                new_rushrepairauditdto.setType(next_type);
                new_rushrepairauditdto.setAuditResult("0");
                new_rushrepairauditdto.setAuditDept(rushRepairNextAuditDeptId);//设置上一步的审核部门
                new_rushrepairauditdto.setAuditUser(rushRepairNextAuditId);//设置上一步的审核人
                new_rushrepairauditdto.setRushrepairId(id);
                new_rushrepairauditdto.setBeforeAuditid(info_audit.getId());
                IRushrepairauditService.savaBuildInfo(null, new_rushrepairauditdto);
            }
        }
        //endregion

        //region 执行更新
        IRushrepairService.updateById(info);
        IRushrepairauditService.updateById(info_audit);
        //endregion

        //region 如果是热线 则工单归档
        if (RushRepairSateEnum.RUSHREPAIR_SATE_PLACE_FILE.getCode().equals(next_type)) {
            if (null != info.getUploadType()) {
                if (info.getUploadType().equals("1")) {
                    gdglinfoClient.gdGdUpdateState(info.getId().toString(), 1);
                }
            }
        }
        //ednregion

        //region 发送mq 设置待办
        // 发送mq消息(审核成功/失败)给上报对象到消息提示列表
        // 通知审核对象mq消息
        iAuditMessageService.auditRushRepair(dto, info_audit.getSubmitUser());

        // 待办事项处理状态置为完成
        // 待办事项处理
        Usertask usertask = new Usertask();
        usertask.setTableName(MessageTopicEnum.REPORT_RUSH_REPAIR_AUDIT.getTableName());
        usertask.setHandleUser(auditId);
        usertask.setTableId(id);
        List<Usertask> result = userTaskClient.getUserTaskInfo(usertask).getResult();
        if (null != result && result.size() > 0) {
            result.forEach(task -> {
                task.setHandleTime(DateTimeUtil.convertLDTToDate(LocalDateTime.now()));
                task.setHandleUser(auditId);
                task.setIshandle(UserTaskStateEnumEnum.IS_HANDLER_COMPLETED.getCode());
                userTaskClient.updateById(task);
            });
        }
        //endregion
    }


    /**
     * 获取审核状态
     *
     * @param type        流程状态
     * @param auditResult 审核结果
     * @param ischeck     是否复核
     * @param isnoprocess 是否自行处理
     * @return
     */
    private String getRushrepairauditType(String type, String auditResult, String ischeck, String isnoprocess) {
        if (AuditSateEnum.AUDIT_YES.getCode().equals(auditResult)) {
            if (RushRepairSateEnum.RUSHREPAIR_SATE_SAFE_AUDIT.getCode().equals(type) && StringUtil.isEmpty(ischeck)) {
                throw new BusinessException(Constant.RUSH_REPAIR_AUDIT_ISCHECK_ERROR_CODE, Constant.RUSH_REPAIR_AUDIT_ISCHECK_ERROR);
            }
            switch (type) {
                case "1":
                    if (ischeck.equals(RushRepairIsCheckEnum.RUSHREPAIR_IS_CHECK.getCode())) {
                        return RushRepairSateEnum.RUSHREPAIR_SATE_SITE_SURVERY.getCode();
                    } else if (ischeck.equals(RushRepairIsCheckEnum.RUSHREPAIR_NO_CHECK.getCode())) {
                        return RushRepairSateEnum.RUSHREPAIR_SATE_DEPT_LEADER_AUDIT.getCode();
                    } else if (ischeck.equals(RushRepairIsCheckEnum.RUSHREPAIR_SATE_RECEIVING_ORDERS.getCode())) {
                        return RushRepairSateEnum.RUSHREPAIR_SATE_RECEIVING_ORDERS.getCode();
                    }
                case "-1":
                case "2":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_SAFE_AUDIT.getCode();
                case "3":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_AUDIT_DISPATCH.getCode();
                case "4":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_RECEIVING_ORDERS.getCode();
                case "5":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_DEPT_AUDIT_PLACEONFILE.getCode();
                case "6":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_PLACE_FILE.getCode();
                case "8":
                    if (ischeck.equals(RushRepairIsCheckEnum.RUSHREPAIR_SATE_SAFE_AUDIT.getCode())) {
                        return RushRepairSateEnum.RUSHREPAIR_SATE_SAFE_AUDIT.getCode();
                    } else if (ischeck.equals(RushRepairIsCheckEnum.RUSHREPAIR_SATE_RECEIVING_ORDERS.getCode())) {
                        return RushRepairSateEnum.RUSHREPAIR_SATE_RECEIVING_ORDERS.getCode();
                    }
            }
        } else if (AuditSateEnum.AUDIT_NO.getCode().equals(auditResult)) {
            if (RushRepairSateEnum.RUSHREPAIR_SATE_DEPT_AUDIT_PLACEONFILE.getCode().equals(type) && StringUtil.isEmpty(isnoprocess)) {
                throw new BusinessException(Constant.RUSH_REPAIR_AUDIT_ISNOPROCESS_ERROR_CODE, Constant.RUSH_REPAIR_AUDIT_ISNOPROCESS_ERROR);
            }
            switch (type) {
                case "1":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_TO_BE_SUBMIT.getCode();
                case "2":
                case "3":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_SAFE_AUDIT.getCode();
                case "4":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_DEPT_LEADER_AUDIT.getCode();
                case "5":
                    return RushRepairSateEnum.RUSHREPAIR_SATE_AUDIT_DISPATCH.getCode();
                case "6":
                    return isnoprocess.equals(RushRepairNoProcessEnum.RUSHREPAIR_NO_HANDLE.getCode()) ? RushRepairSateEnum.RUSHREPAIR_SATE_TO_BE_SUBMIT.getCode() : RushRepairSateEnum.RUSHREPAIR_SATE_RECEIVING_ORDERS.getCode();
            }
        }
        return null;
    }
}
