/*
 * Author: Zev
 * 生成ServiceImpl
 */
package com.tofly.gpsboot.service.paln.task.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import com.tofly.api.DepartmentClient;
import com.tofly.api.UserClent;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.common.system.*;
import com.tofly.gpsboot.service.comm.IAuditService;
import com.tofly.gpsboot.service.paln.HydrantrecordService;
import com.tofly.gpsboot.service.paln.LeakRecordService;
import com.tofly.gpsboot.service.paln.PlanMtService;
import com.tofly.gpsboot.service.paln.PlantypeService;
import com.tofly.gpsboot.service.valvewell.ValvewellService;
import com.tofly.pojo.comm.Audit;
import com.tofly.pojo.plan.dto.PlanDto;
import com.tofly.pojo.plan.vo.*;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.mapper.plan.PlanMapper;
import com.tofly.gpsboot.service.comm.IDisposeDateService;
import com.tofly.gpsboot.service.mq.IPlanMessageService;
import com.tofly.gpsboot.service.paln.task.*;
import com.tofly.gpsboot.service.process.PiperecordService;
import com.tofly.gpsboot.service.process.PointtempService;
import com.tofly.gpsboot.service.config.InspecttypeService;
import com.tofly.gpsboot.service.config.PeriodService;
import com.tofly.gpsboot.service.process.PressrecordService;
import com.tofly.gpsboot.service.process.RecordService;
import com.tofly.gpsboot.service.sign.PdasignService;
import com.tofly.gpsboot.util.DateTimeUtil;
import com.tofly.pojo.User;
import com.tofly.pojo.plan.*;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;


/**
 * 巡检计划详细信息表：TF_YWPN__PLAN_W
 *
 * @author admin
 * @date Thu Mar 18 00:00:00 CST 2021
 */
@Service("planService")
public class PlanServiceImpl extends ServiceImpl<PlanMapper, Plan> implements PlanService {

    @Autowired
    private PeriodService periodService;

    @Autowired
    private PlanInspectGroupService planInspectGroupService;

    @Autowired
    private PlanInspectUserService planInspectUserService;

    @Autowired
    private PointtempService pointtempService;

    @Autowired
    private PlanchildService planchildService;

    @Autowired
    private PdasignService pdasignService;

    @Autowired
    private IPlanMessageService iPlanMessageService;

    @Autowired
    private DepartmentClient departmentClient;
    @Autowired
    private IDisposeDateService iDisposeDateService;

    @Autowired
    private RecordService recordService;
    @Autowired
    private PressrecordService pressrecordService;

    @Autowired
    private PiperecordService piperecordService;

    @Autowired
    private HydrantrecordService hydrantrecordService;

    @Autowired
    private LeakRecordService leakRecordService;

    @Autowired
    private PlantypeService plantypeService;

    @Autowired
    private PlanMtService planMtService;
    @Autowired
    private ValvewellService valvewellService;
    @Autowired
    private IAuditService iAuditService;

    @Autowired
    private InspecttypeService inspecttypeService;


    @Override
    public Page<Plan> getPlanPage(Page page, PlanDto dto) {
        Plan plan = JSONObject.parseObject(JSON.toJSONString(dto), Plan.class);
        QueryWrapper<Plan> queryWrapper = new QueryWrapper<>();
        Date startDate = dto.getPlanBegindate();
        Date endDate = dto.getPlanEnddate();
        String inspectUsers = dto.getInspectUsers();
        String planState = dto.getPlanState();
        String isSelMtHis = dto.getIsSelMtHis();
        Long auditUser = dto.getAuditUser();
        String inspectType = dto.getInspectType();
        if (!StringUtils.isEmpty(inspectType)) {
            List<String> longList = Arrays.stream(inspectType.split(Constant.CSV)).collect(Collectors.toList());
            queryWrapper.in("INSPECT_TYPE", longList);
          /*  longList.forEach(info->{
                queryWrapper.like("INSPECT_TYPE",info);
            });*/

            plan.setInspectType(null);
        }

//        if (StringUtils.isEmpty(dto.getFlag())) {
//            // 当前部门下用户才有权限
//            Long departmentId = SecurityUtils.getUser().getDepartmentId();
//            Set<Long> userIdList = departmentClient.getDeptUserList(departmentId).getResult().stream().map(User::getId).collect(Collectors.toSet());
//            queryWrapper.in("CREATE_USER", userIdList);
//        }

        if (null != startDate && null != endDate) {
            queryWrapper.ge("PLAN_BEGINDATE", startDate).lt("PLAN_ENDDATE", endDate);
//                    .or().le("PLAN_BEGINDATE", startDate).ge("PLAN_ENDDATE", endDate);
            plan.setPlanBegindate(null);
            plan.setPlanEnddate(null);
        } else if (null != startDate) {
            queryWrapper.ge("PLAN_BEGINDATE", startDate);
            plan.setPlanBegindate(null);
        } else if (null != endDate) {
            queryWrapper.le("PLAN_ENDDATE", endDate);
            plan.setPlanEnddate(null);
        }
        if (!StringUtils.isEmpty(planState)) {
            List<String> longList = Arrays.stream(planState.split(Constant.CSV)).collect(Collectors.toList());
            queryWrapper.in("PLAN_STATE", longList);
            plan.setPlanState(null);
        }

        if ("1".equals(isSelMtHis)) {
            // 查询维护历史计划
            List<PlanMt> list = planMtService.list(new QueryWrapper<PlanMt>().lambda().eq(PlanMt::getIsHis, "1"));
            if (null != list && list.size() > 0) {
                Set<Long> collect = list.stream().map(PlanMt::getPlanId).collect(Collectors.toSet());
                queryWrapper.in("ID", collect);
            } else {
                queryWrapper.in("ID", 0);
            }
        }
        if (null != auditUser) {
            List<Audit> list = iAuditService.list(new QueryWrapper<Audit>().lambda()
                    .eq(Audit::getAuditType, AuditTypeEnum.AUDIT_TYPE_PLAN_MT.getCode())
                    .eq(Audit::getAuditUser, auditUser)
                    .eq(Audit::getAuditState, AuditSateEnum.AUDIT_UNCOMMITTED.getCode()));
            if (null != list) {
                Set<Long> longSet = list.stream().map(Audit::getAuditPlanId).collect(Collectors.toSet());
                queryWrapper.in("ID", longSet);
            } else {
                queryWrapper.in("ID", 0);
            }
        } else {
            if (!StringUtils.isEmpty(inspectUsers)) {
                List<Long> longList = Arrays.stream(inspectUsers.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                        .collect(Collectors.toList());
                List<PlanInspectUser> list = planInspectUserService.list(new QueryWrapper<PlanInspectUser>().lambda().in(PlanInspectUser::getUserId, longList));
                if (null != list && list.size() > 0) {
                    Set<Long> collect = list.stream().map(PlanInspectUser::getPlanId).collect(Collectors.toSet());
                    queryWrapper.in("ID", collect);
                } else {
                    queryWrapper.in("ID", 0);
                }
            }
            /*else {
                if (StringUtils.isEmpty(dto.getMouthId())) {
                    queryWrapper.in("ID", 0);
                }

            }*/
        }

        // 当前状态超时不显示
//        queryWrapper.in("PLAN_STATE", "11");
        queryWrapper.setEntity(plan);
        queryWrapper.orderByDesc("CREATE_TIME");
        return this.page(page, queryWrapper);
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<Plan> savaPlanInfo(PlanDto parDto) throws BusinessException {
        // 获取巡检周期类型
        Period period = periodService.getById(parDto.getPeriodId());
        Optional.ofNullable(period).orElseThrow(() -> new BusinessException(Constant.PLAN_PERIOD_NULL_CODE, Constant.PLAN_PERIOD_NULL));

        Plan plan = JSON.parseObject(JSON.toJSONString(parDto), Plan.class);
        if (IsDispatchingEnum.IS_DISPATCHING_YES.getCode().equals(parDto.getIsDispatching())) {
            plan.setPlanState(PlanStateEnum.PLAN_STATE_UNENFORCEABLE.getCode());
        }
        // 判断传入类型是否为自定义,若为自定义则赋值天数
        Integer intervalDay = period.getIntervalDay();
        if (null == intervalDay) {
            intervalDay = parDto.getIntervalDay();
        }

        List<Plantype> planTypeOverList = parDto.getPlanTypeOverList();
        if (null == planTypeOverList) {
            throw new BusinessException(Constant.TASK_IS_REPAIR_ORDER_ADD_NULL_CODE, Constant.TASK_IS_REPAIR_ORDER_ADD_NULL);
        }
        List<Pointtemp> pointTempList = parDto.getPointTempList();
        if (null == pointTempList) {
            throw new BusinessException(Constant.TASK_IS_PLAN_STATE_POINT_PLACE_CODE, Constant.TASK_IS_PLAN_STATE_POINT_PLACE);
        }
        // 计算巡检任务的集合
        List<Plan> planList = this.calculatePlanList(plan, intervalDay, planTypeOverList);
        planList.forEach(this::save);

        // 新增到 计划与巡察组关联表,计划与巡查人关联表 ; 这个分开
        Long inspectGroupId = parDto.getInspectGroupId();
        String inspectUsers = parDto.getInspectUsers();
        List<String> asList = Arrays.asList(inspectUsers.split(Constant.SEMICOLON));

        List<PlanInspectGroup> savePlanInspectGroup = new ArrayList<>();
        List<PlanInspectUser> savePlanInspectUser = new ArrayList<>();
        planList.forEach(info -> {
            Long planId = info.getId();
            PlanInspectGroup inspectGroup = new PlanInspectGroup();
            inspectGroup.setPlanId(planId);
            inspectGroup.setInspectGroupId(inspectGroupId);
            savePlanInspectGroup.add(inspectGroup);
            asList.forEach(userid -> {
                PlanInspectUser inspectUser = new PlanInspectUser();
                inspectUser.setPlanId(planId);
                inspectUser.setUserId(Long.valueOf(userid));
                savePlanInspectUser.add(inspectUser);
            });

        });
        planInspectGroupService.saveBatch(savePlanInspectGroup);
        planInspectUserService.saveBatch(savePlanInspectUser);

        // 存入对应的点到位信息 /新增对应计划类型完成率初始化
        // 按照类型分组
        Map<Long, List<Pointtemp>> tempMap = pointTempList.stream().collect(Collectors.groupingBy(Pointtemp::getInspectTypeId));
        Map<Long, Float> floatMap = planTypeOverList.stream().collect(Collectors.toMap(Plantype::getTypeId, Plantype::getCompletionRate, (key1, key2) -> key2));
        List<Pointtemp> savePointTempList = new ArrayList<>();
        List<Plantype> savePlanType = new ArrayList<>();
        planList.forEach(info -> {
            Long typeId = Long.valueOf(info.getInspectType());
            Long planId = info.getId();

            List<Pointtemp> pointtempList = tempMap.get(typeId);

            for (Pointtemp temp : pointtempList) {
                Pointtemp pointtemp = new Pointtemp();
                BeanUtils.copyProperties(temp, pointtemp);
                pointtemp.setPlanId(planId);
                savePointTempList.add(pointtemp);
            }

            Plantype plantype = new Plantype();
            plantype.setPlanId(planId);
            plantype.setTypeId(typeId);
            plantype.setCompletionRate(floatMap.get(typeId));
            plantype.setTotalCount(BigDecimal.valueOf(pointtempList.size()));
            double sum = pointtempList.stream().filter(i -> null != i.getPipeLength()).mapToDouble(i -> i.getPipeLength().doubleValue()).sum();
            plantype.setTotalLength(BigDecimal.valueOf(sum));
            savePlanType.add(plantype);

        });
        pointtempService.saveBatch(savePointTempList);
        plantypeService.saveBatch(savePlanType);
        // 处理普通任务派发任务
        if (IsDispatchingEnum.IS_DISPATCHING_YES.getCode().equals(parDto.getIsDispatching())) {

            // 判断计划类型 隐患 工地 计划
            String planType = parDto.getPlanType();
            if (PlanTypeEnum.PLAN_TYPE_TROUBLE.getCode().equals(planType)) {
                iPlanMessageService.planSendMessageTrouble(planList);
            } else if (PlanTypeEnum.PLAN_TYPE_BUILD.getCode().equals(planType)) {
                iPlanMessageService.planSendMessageBuild(planList);
            } else {
                iPlanMessageService.planSendMessage(planList);
            }
        }

        return planList;
    }

    @Override
    public Page<Plan> getPlanTimeIntervalPage(Page page, PlanDto dto) {
        Plan plan = JSONObject.parseObject(JSON.toJSONString(dto), Plan.class);
        QueryWrapper<Plan> queryWrapper = new QueryWrapper<>();
        Date startDate = dto.getPlanBegindate();
        Date endDate = dto.getPlanEnddate();
        String inspectUsers = dto.getInspectUsers();
        String planState = dto.getPlanState();
        String inspectType = dto.getInspectType();
//        if (StringUtils.isEmpty(dto.getFlag())) {
//            // 当前部门下用户才有权限
//            Long departmentId = SecurityUtils.getUser().getDepartmentId();
//            Set<Long> userIdList = departmentClient.getDeptUserList(departmentId).getResult().stream().map(User::getId).collect(Collectors.toSet());
//            queryWrapper.in("CREATE_USER", userIdList);
//        }
        if (null != startDate) {
            queryWrapper.le("PLAN_BEGINDATE", startDate);
            plan.setPlanBegindate(null);

        }
        if (null != endDate) {
            queryWrapper.ge("PLAN_ENDDATE", endDate);
            plan.setPlanEnddate(null);
        }
        if (!StringUtils.isEmpty(inspectType)) {
            List<String> longList = Arrays.stream(inspectType.split(Constant.CSV)).collect(Collectors.toList());
            queryWrapper.in("INSPECT_TYPE", longList);
           /* longList.forEach(info->{
                queryWrapper.like("INSPECT_TYPE",info);
            });*/

            plan.setInspectType(null);
        }
        if (!StringUtils.isEmpty(planState)) {
            List<String> longList = Arrays.stream(planState.split(Constant.CSV)).collect(Collectors.toList());
            queryWrapper.in("PLAN_STATE", longList);
            plan.setPlanState(null);
        }

        if (!StringUtils.isEmpty(inspectUsers)) {
            List<Long> longList = Arrays.stream(inspectUsers.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                    .collect(Collectors.toList());
            List<PlanInspectUser> list = planInspectUserService.list(new QueryWrapper<PlanInspectUser>().lambda().in(PlanInspectUser::getUserId, longList));
            if (null != list && list.size() > 0) {
                Set<Long> collect = list.stream().map(PlanInspectUser::getPlanId).collect(Collectors.toSet());
                queryWrapper.in("ID", collect);
            } else {
                queryWrapper.in("ID", 0);
            }
        } else {
            if (StringUtils.isEmpty(dto.getMouthId())) {
                queryWrapper.in("ID", 0);
            }

        }
        // 当前状态超时不显示
//        queryWrapper.in("PLAN_STATE", "11");
        queryWrapper.setEntity(plan);
        queryWrapper.orderByDesc("CREATE_TIME");
        return this.page(page, queryWrapper);
    }

    @Override
    public Plan getPlanInfo(Long id) throws BusinessException {
        // 基本参数赋值
        Plan planVo = this.getById(id);
//        // 巡检组赋值（在查询时已赋值）
//        List<PlanInspectGroup> groups = planInspectGroupService.list(new QueryWrapper<PlanInspectGroup>().lambda().eq(PlanInspectGroup::getPlanId, id));
//        if (null != groups && groups.size() > 0) {
//            PlanInspectGroup inspectGroup = groups.get(0);
//            Long inspectGroupId = inspectGroup.getInspectGroupId();
//            if (null != inspectGroupId) {
//                planVo.setInspectGroup(inspectGroupId.toString());
//            }
//        }
//        // 巡检人赋值（在查询时已赋值）
//        List<PlanInspectUser> list = planInspectUserService.list(new QueryWrapper<PlanInspectUser>().lambda().eq(PlanInspectUser::getPlanId, id));
//        StringBuffer str = new StringBuffer("");
//        list.forEach(info -> {
//            str.append(info.getUserIdName()).append(Constant.CSV);
//
//        });
//        if (!StringUtils.isEmpty(str)) {
//            planVo.setInspectUserList(str.deleteCharAt(str.length() - 1).toString());
//        }
        planVo.setPlanTypeOverList(plantypeService.list(new QueryWrapper<Plantype>().lambda().eq(Plantype::getPlanId, id)));

        return planVo;
    }

    @Override
    public List<PlanDetailVo> getPlanDetail(Long id, String flag) {
        List<PlanDetailVo> voList = new ArrayList<>();
        // 获取计划信息
        Plan plan = this.getById(id);
        // 获取子计划信息
        List<Planchild> planChildVo = this.getPlanChildVo(id, flag);
        if (null != planChildVo && planChildVo.size() > 0) {
            planChildVo.forEach(info -> {
                PlanDetailVo vo = new PlanDetailVo();
                Long childId = info.getId();
                // 子计划添加
                vo.setPlanChildVo(info);
                // 巡查记录
                List<RecordVo> planRecordVo = this.getPlanRecordVo(info.getId());
                vo.setRecordVoList(planRecordVo);
                // 管线记录
                List<Piperecord> pipeRecordList = piperecordService.list(new QueryWrapper<Piperecord>().lambda().eq(Piperecord::getPlanChildid, childId));
                if (null != pipeRecordList && pipeRecordList.size() > 0) {
                    vo.setPipeRecordVoList(piperecordService.viewConversion(pipeRecordList));
                }
                // 水压记录
                List<Pressrecord> pressRecordList = pressrecordService.list(new QueryWrapper<Pressrecord>().lambda().eq(Pressrecord::getPlanChildid, childId));
                if (null != pressRecordList && pressRecordList.size() > 0) {
                    vo.setPressRecordVoList(pressrecordService.viewConversion(pressRecordList));
                }
                // 消防栓记录
                List<Hydrantrecord> hydrantRecordList = hydrantrecordService.list(new QueryWrapper<Hydrantrecord>().lambda().eq(Hydrantrecord::getPlanChildid, childId));
                if (null != hydrantRecordList && hydrantRecordList.size() > 0) {
                    vo.setHydrantRecordList(hydrantRecordList);
                }
                // 探漏记录
                List<LeakRecord> leakRecordList = leakRecordService.list(new QueryWrapper<LeakRecord>().lambda().eq(LeakRecord::getPlanChildId, childId));
                if (null != leakRecordList && leakRecordList.size() > 0) {
                    vo.setLeakRecordList(leakRecordList);
                }
                // 维护记录
                List<PlanMt> planMtList = planMtService.list(new QueryWrapper<PlanMt>().lambda().eq(PlanMt::getPlanChildId, childId).eq(PlanMt::getFlag, "1"));
                if (null != planMtList && planMtList.size() > 0) {
                    vo.setPlanMtList(planMtList);
                }
                // 阀门井
                List<Valvewell> valvewellList = valvewellService.list(new QueryWrapper<Valvewell>().lambda().eq(Valvewell::getPlanChildid, childId));
                if (null != valvewellList && valvewellList.size() > 0) {
                    List<ValvewellVo> valvewellVos = new ArrayList<>();
                    for (Valvewell valvewell : valvewellList) {
                        valvewellVos.add(valvewellService.getValvewellInfo(valvewell.getId()));
                    }
                    vo.setValvewellVos(valvewellVos);
                }
                // 计划信息
                vo.setPlanVo(plan);
                voList.add(vo);
            });

        }


        return voList;
    }

    @Override
    public Boolean planUrgent(Long planId, String planState, String userIds) {
        boolean update = this.update(new UpdateWrapper<Plan>().lambda().eq(Plan::getId, planId)
                .set(Plan::getIsEmergency, "1"));
        if (update) {
            iPlanMessageService.planUrgent(planId, planState, userIds);
        } else {
            return false;
        }

        return true;
    }

    @Override
    public PlanYearReportVo getReportPlanYear() {
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        return this.baseMapper.getReportPlanYear(year);
    }

    private List<Planchild> getPlanChildVo(Long id, String flag) {
        Set<Long> userIdList = new HashSet<>();
        if (IsOverEnum.IS_OVER_NO.getCode().equals(flag)) {
            // 当前部门下的用户子计划
            // 当前部门下用户才有权限
            Long departmentId = SecurityUtils.getUser().getDepartmentId();
            Set<Long> collect = departmentClient.getDeptUserList(departmentId).getResult().stream().map(User::getId).collect(Collectors.toSet());
            userIdList.addAll(collect);
        } else {
            userIdList.add(SecurityUtils.getUserId());
        }
        List<Planchild> list = planchildService.list(new QueryWrapper<Planchild>().lambda()
                .eq(Planchild::getPlanId, id)
        );
        if (list.size() > 0) {
            return list;
        }
        return null;
    }

    /**
     * 过程上报记录
     *
     * @param planChildId
     * @return
     */
    private List<RecordVo> getPlanRecordVo(Long planChildId) {
        List<Record> list = recordService.list(new QueryWrapper<Record>().lambda().eq(Record::getPlanChildid, planChildId));
        if (null != list && list.size() > 0) {
            List<RecordVo> recordVoList = recordService.viewConversion(list);
            recordVoList.sort(Comparator.comparing(RecordVo::getCreateTime).reversed());
            return recordVoList;
        }
        return null;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePlanInfo(PlanDto parDto) throws BusinessException {
        if (!iDisposeDateService.isDisposePermissions(parDto.getCreateUser())) {
            throw new BusinessException(Constant.IS_DISPOSE_PERMISSIONS_NULL_CODE, Constant.IS_DISPOSE_PERMISSIONS_NULL);
        }
        // 对已下发未开始的临时任务，可对其巡检任务内容进行修改。计划性任务不可修改。
        String planState = parDto.getPlanState();
//        String planType = parDto.getPlanType();
//        if (PlanTypeEnum.PLAN_TYPE_MONTH.getCode().equals(planType)) {
//            throw new BusinessException(Constant.TASK_PLAN_MONTH_NOT_UPDATE_CODE, Constant.TASK_PLAN_MONTH_NOT_UPDATE);
//        }
        if (PlanStateEnum.PLAN_STATE_NOT_SUBMITTED.getCode().equals(planState) || PlanStateEnum.PLAN_STATE_NOT_AUDIT.getCode().equals(planState)) {
            // 清空信息
            Long id = parDto.getId();
            List<Long> idList = new ArrayList<>();
            idList.add(id);
            this.deletePlan(idList);
            this.removeById(id);
            // 走保存
            this.savaPlanInfo(parDto);
        } else {
            throw new BusinessException(Constant.TASK_PLAN_NOT_UPDATE_CODE, Constant.TASK_PLAN_NOT_UPDATE);
        }


    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deletePlanInfo(List<Long> idList) throws BusinessException {
        // 对已下发未开始的临时任务，可对其巡检任务内容进行删除。计划性任务不可删除。
        List<Plan> list = this.list(new QueryWrapper<Plan>().lambda().in(Plan::getId, idList));
        list.forEach(info -> {
            if (!iDisposeDateService.isDisposePermissions(info.getCreateUser())) {
                throw new BusinessException(Constant.IS_DISPOSE_PERMISSIONS_NULL_CODE, Constant.IS_DISPOSE_PERMISSIONS_NULL);

            }
            String planState = info.getPlanState();
//            String planType = info.getPlanType();
//            if (PlanTypeEnum.PLAN_TYPE_MONTH.getCode().equals(planType)) {
//                throw new BusinessException(Constant.TASK_PLAN_MONTH_NOT_DELETE_CODE, Constant.TASK_PLAN_MONTH_NOT_DELETE);
//            }
            if (PlanStateEnum.PLAN_STATE_NOT_SUBMITTED.getCode().equals(planState) || PlanStateEnum.PLAN_STATE_NOT_AUDIT.getCode().equals(planState)) {
                this.deletePlan(idList);
            } else {
                throw new BusinessException(Constant.TASK_PLAN_NOT_DELETE_CODE, Constant.TASK_PLAN_NOT_DELETE);
            }

        });
        this.removeByIds(idList);

    }

    private void deletePlan(List<Long> planIdList) {
        // 删除点到位信息
        pointtempService.remove(new QueryWrapper<Pointtemp>().lambda().in(Pointtemp::getPlanId, planIdList));
        // 删除用户关联计划的信息
        // 清楚中间表关联数据 计划与巡察组关联表,计划与巡查人关联表
        planInspectGroupService.remove(new QueryWrapper<PlanInspectGroup>().lambda().in(PlanInspectGroup::getPlanId, planIdList));
        planInspectUserService.remove(new QueryWrapper<PlanInspectUser>().lambda().in(PlanInspectUser::getPlanId, planIdList));
        // 清空计划类型
        plantypeService.remove(new QueryWrapper<Plantype>().lambda().in(Plantype::getPlanId, planIdList));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updatePlanState(Long planId, String planState, Long userId) {

        Plan plan = this.getById(planId);
        // 若修改当前计划状态 为 3 正在执行
        if (PlanStateEnum.PLAN_STATE_BEING_PERFORMED.getCode().equals(planState)) {
            // 巡检计划为 2 未开始 或 5 已开始未执行
            if (PlanStateEnum.PLAN_STATE_NOT_START.getCode().equals(plan.getPlanState()) || PlanStateEnum.PLAN_STATE_EXECUTION_NOT_STARTED.getCode().equals(plan.getPlanState())) {
                // 巡检计划巡检开始时间若大于当前时间则不允许启动
                LocalDateTime startTime = DateTimeUtil.fromLong2LocalDateTime(plan.getPlanBegindate().getTime());
                if (LocalDateTime.now().isBefore(startTime)) {
                    throw new BusinessException(Constant.TASK_IS_PLAN_STATE_BEING_TIME_CODE, Constant.TASK_IS_PLAN_STATE_BEING_TIME);
                }
                // 判断当前用户是否已经签到
                if (SignTypeEnum.SIGN_TYPE_YES.getCode().equals(pdasignService.getUserSignState(userId))) {
                    // 获取对应用户的计划列表
                    List<PlanInspectUser> planList = planInspectUserService.list(new QueryWrapper<PlanInspectUser>().lambda().eq(PlanInspectUser::getUserId, userId));
                    List<PlanInspectUser> filterList = planList.stream().filter(p -> PlanStateEnum.PLAN_STATE_BEING_PERFORMED.getCode().equals(this.getById(p.getPlanId()).getPlanState())).collect(Collectors.toList());
                    // 判断当前用户是否存在正在执行的计划
                    if (filterList.size() > 0) {
                        throw new BusinessException(Constant.TASK_IS_PLAN_STATE_BEING_PERFORMED_CODE, Constant.TASK_IS_PLAN_STATE_BEING_PERFORMED);
                    }
                    plan.setPlanState(planState);
                    this.updateById(plan);

                } else {
                    throw new BusinessException(Constant.TASK_IS_PLAN_STATE_BEING_PERFORMED_SIGN_CODE, Constant.TASK_IS_PLAN_STATE_BEING_PERFORMED_SIGN);

                }
            } else {
                throw new BusinessException(Constant.TASK_IS_PLAN_STATE_OVER_PERFORMED_CODE, Constant.TASK_IS_PLAN_STATE_OVER_PERFORMED + PlanStateEnum.getCodeForValue(plan.getPlanState()) + "不可启动");

            }


        } else if (PlanStateEnum.PLAN_STATE_EXECUTION_NOT_STARTED.getCode().equals(planState)) {
            plan.setPlanState(planState);
            this.updateById(plan);
        }
    }


    @Override
    public Plan stopPlan(Plan plan, Planstop planStop) {
        // 暂停任务1.获取暂停申请计划重启时间2.修改当前主计划状态为未开始,计划巡检时间
        Date applyStartTime = planStop.getApplyStarttime();
        Date applyEndTime = planStop.getApplyEndtime();
        plan.setPlanBegindate(applyStartTime);
        plan.setPlanEnddate(applyEndTime);
        plan.setPlanState(PlanStateEnum.PLAN_STATE_PROJECT_SUSPENDED.getCode());
        return plan;
    }

    @Override
    public Plan disusePlan(Plan plan, Plancancle plancancle) {
        // 作废任务1.修改当前主计划状态为计划已作废
        plan.setPlanState(PlanStateEnum.PLAN_STATE_PLAN_DISUSE.getCode());
        return plan;
    }

    /**
     * 巡检任务次数计算
     *
     * @param plan         当前巡检计划内容
     * @param intervalDay  计划周期间隔时间
     * @param planTypeList 计划类型
     * @return 巡检任务集合
     */
    private List<Plan> calculatePlanList(Plan plan, Integer intervalDay, List<Plantype> planTypeList) {
        List<Plan> planList = new ArrayList<>();
        planTypeList.forEach(type -> {
            Date planBeginDate = plan.getPlanBegindate();
            // 由于前端传入的是以59:59:59 结尾,后台处理  +1天取当前天数起始日
            Date planEndDate = DateTimeUtil.convertLDTToDate(LocalDateTime.of(DateTimeUtil.fromLong2LocalDateTime(plan.getPlanEnddate().getTime()).plusDays(1).toLocalDate(), LocalTime.MIN));
            // 由于前端已经做了类型与天数的交互,后台只分析对应时间范围天数包含几个周期任务
            Long aLong = DateTimeUtil.differDays(DateTimeUtil.fromLong2LocalDateTime(planBeginDate.getTime()), DateTimeUtil.fromLong2LocalDateTime(planEndDate.getTime()));
            int k = 0;
            BigDecimal divide = BigDecimal.valueOf(aLong).divide(BigDecimal.valueOf(intervalDay), BigDecimal.ROUND_HALF_UP);
            int i = divide.intValue();

            while (i > k) {
                Plan info = JSON.parseObject(JSON.toJSONString(plan), Plan.class);
                info.setInspectType(type.getTypeId() + "");
                info.setPlanBegindate(planBeginDate);
                //滚动开始时间
                planBeginDate = DateTimeUtil.convertLDTToDate(DateTimeUtil.fromLong2LocalDateTime(planBeginDate.getTime()).plusDays(intervalDay));
                //根据前端传入23:59:59 来进行存储 计算还是按照00:00:00计算
                Date planBeginDate_new = DateTimeUtil.convertLDTToDate(
                        LocalDateTime.of(DateTimeUtil.fromLong2LocalDateTime(planBeginDate.getTime()).plusDays(-1).toLocalDate(), LocalTime.MAX));
                info.setPlanEnddate(planBeginDate_new);
                // 若最大时间不超过生成最大时间,则默认为用户输入最大时间
                Long days = DateTimeUtil.differDays(DateTimeUtil.fromLong2LocalDateTime(plan.getPlanEnddate().getTime()), DateTimeUtil.fromLong2LocalDateTime(planEndDate.getTime()));
                if (0 >= days) {
                    info.setPlanEnddate(planBeginDate_new);
                }
                planList.add(info);
                k++;
            }


        });

        return planList;
    }


    private void savePlanType(List<Plan> planList, List<Plantype> planTypeOverList, List<Pointtemp> pointTempList) {
        planList.forEach(plan -> {
            // 获取计划id
            Long planId = plan.getId();
            // 判断是已存在当前计划 存在则删除
            List<Plantype> list = plantypeService.list(new QueryWrapper<Plantype>().lambda().eq(Plantype::getPlanId, planId));
            if (null != list && list.size() > 0) {
                Set<Long> collect = list.stream().map(Plantype::getId).collect(Collectors.toSet());
                plantypeService.removeByIds(collect);
            }
            // 按照类型分组
            Map<Long, List<Pointtemp>> collect = pointTempList.stream().filter(info -> null != info.getInspectTypeId()).collect(Collectors.groupingBy(Pointtemp::getInspectTypeId));
            Map<Long, Float> floatMap = planTypeOverList.stream().collect(Collectors.toMap(Plantype::getTypeId, Plantype::getCompletionRate, (key1, key2) -> key2));
            collect.forEach((k, v) -> {
                Plantype plantype = new Plantype();
                plantype.setPlanId(planId);
                plantype.setTypeId(k);
                plantype.setCompletionRate(floatMap.get(k));
                plantype.setTotalCount(BigDecimal.valueOf(v.size()));

                plantypeService.save(plantype);
            });
        });

    }


}
