package com.tofly.gpsboot.service.report.impl;

import com.tofly.api.UserTaskClient;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.common.system.MessageTopicEnum;
import com.tofly.gpsboot.common.system.ReportSateEnum;
import com.tofly.gpsboot.common.system.UserTaskStateEnumEnum;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.service.mq.IReportMessageService;
import com.tofly.gpsboot.service.report.IReportRushRepairServer;
import com.tofly.gpsboot.service.rushrepair.IRushrepairService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairauditService;
import com.tofly.gpsboot.util.DateTimeUtil;
import com.tofly.pojo.message.Usertask;
import com.tofly.pojo.plan.dto.ReportAuditDto;
import com.tofly.pojo.rushrepair.Rushrepairaudit;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;

/**
 * 抢修上报 实现类
 *
 * @author sunjian
 * @date 2023/3/22
 */
@Service("REPORT_TYPE_RUSH_REPAIR")
public class ReportRushRepairServerImpl implements IReportRushRepairServer {

    @Autowired
    private IRushrepairService IRushrepairService;

    @Autowired
    private IRushrepairauditService IRushrepairauditService;

    @Autowired
    private IReportMessageService iReportMessageService;
    @Autowired
    private UserTaskClient userTaskClient;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void report(ReportAuditDto dto) {
        Long id = dto.getId();
        Rushrepairaudit info = IRushrepairauditService.getById(id);

        String isSubmit = info.getIsSubmit();
        String auditResult = info.getAuditResult();
        // 上报条件 1.当前对象未上报
        // 2.当前对象已上报 审核状态为不通过
        if (ReportSateEnum.REPORT_UNCOMMITTED.getCode().equals(isSubmit)) {
            // 修改当前对象上报状态为 已上报,上报时间,修改审核状态为 未审批(在审核类里修改)
            info.setSubmitTime(DateTimeUtil.convertLDTToDate(LocalDateTime.now()));
            info.setSubmitUser(SecurityUtils.getUserId());
            info.setIsSubmit(ReportSateEnum.REPORT_SUBMITTED.getCode());

            //info.setAuditResult(AuditSateEnum.AUDIT_UNCOMMITTED.getCode());
            //info.setAuditUser(dto.getAuditId());
            IRushrepairauditService.updateById(info);

            iReportMessageService.reportTypeRushRepair(dto);

            //0自行处理 自动完成上报和待办
            if (info.getType().equals("0")) {
                // 待办事项处理状态置为完成
                Usertask usertask = new Usertask();
                usertask.setTableName(MessageTopicEnum.REPORT_RUSH_REPAIR_AUDIT.getTableName());
                usertask.setHandleUser(info.getAuditUser());
                usertask.setTableId(id);
                List<Usertask> result = userTaskClient.getUserTaskInfo(usertask).getResult();
                if (null != result && result.size() > 0) {
                    result.forEach(task -> {
                        task.setHandleTime(DateTimeUtil.convertLDTToDate(LocalDateTime.now()));
                        task.setHandleUser(info.getAuditUser());
                        task.setIshandle(UserTaskStateEnumEnum.IS_HANDLER_COMPLETED.getCode());
                        userTaskClient.updateById(task);
                    });
                }
            }
        } else {
            throw new BusinessException(Constant.REPORT_NOT_UNCOMMITTED_CODE, Constant.REPORT_NOT_UNCOMMITTED);
        }
    }
}
