package com.tofly.gpsboot.service.rushrepair.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.api.UserClent;
import com.tofly.gpsboot.mapper.rushrepair.RushrepairAndMaterialMapper;
import com.tofly.gpsboot.service.rushrepair.IRushrepairAndMaterialService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairmaterialService;
import com.tofly.pojo.User;
import com.tofly.pojo.rushrepair.RushrepairAndMaterial;
import com.tofly.pojo.rushrepair.dto.RushrepairAndMaterialDto;
import com.tofly.pojo.rushrepair.vo.RushrepairAndMaterialVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * @author sunjian
 * @date 2023/4/13
 */
@Service("rushrepairAndMaterialService")
public class RushrepairAndMaterialServiceImpl extends ServiceImpl<RushrepairAndMaterialMapper, RushrepairAndMaterial> implements IRushrepairAndMaterialService {

    @Autowired
    private IRushrepairmaterialService rushrepairmaterialService;
    @Autowired
    private UserClent userClent;

    /**
     * 根据抢修单对应的材料记录列表
     *
     * @param rushrepairauditid 抢修审核id
     * @return
     */
    @Override
    public List<RushrepairAndMaterialVo> getRushrepairAndMaterialByRushRepairId(Long rushrepairauditid) {
        List<RushrepairAndMaterial> rushrepairAndMaterialList = this.list(new QueryWrapper<RushrepairAndMaterial>().lambda().eq(RushrepairAndMaterial::getRushrepairauditId, rushrepairauditid));
        if (null != rushrepairAndMaterialList && rushrepairAndMaterialList.size() > 0) {
            List<RushrepairAndMaterialVo> rushrepairAndMaterialVoList = new ArrayList<>();
            rushrepairAndMaterialList.forEach(info -> {
                RushrepairAndMaterialVo baseInfoVo = JSON.parseObject(JSON.toJSONString(info), RushrepairAndMaterialVo.class);
                DecimalFormat priceFormat = new DecimalFormat("#.00");
                baseInfoVo.setMaterialPrice(priceFormat.format(info.getMaterialPrice()));
                baseInfoVo.setMaterialMoney(priceFormat.format(info.getMaterialMoney()));
                //添加相关人 名称
                if (null != info.getCreateUser() && 0 != info.getCreateUser()) {
                    User result = userClent.getById(info.getCreateUser()).getResult();
                    if (null != result) {
                        baseInfoVo.setCreateUserName(result.getRealName());
                    }
                }
                rushrepairAndMaterialVoList.add(baseInfoVo);
            });
            return rushrepairAndMaterialVoList;
        }
        return null;
    }

    /**
     * 保存关系表数据
     *
     * @param rushrepairauditid
     * @param rushrepairAndMaterialDtoList
     */
    @Override
    public void saveRushrepairAndMaterial(Long rushrepairauditid, List<RushrepairAndMaterialDto> rushrepairAndMaterialDtoList ) {
        if (!StringUtils.isEmpty(rushrepairauditid) && 0 != rushrepairauditid && rushrepairAndMaterialDtoList.size() > 0) {
            rushrepairAndMaterialDtoList.forEach(rushrepairmaterialDto -> {
                RushrepairAndMaterial rushrepairAndMaterial = JSON.parseObject(JSON.toJSONString(rushrepairmaterialDto), RushrepairAndMaterial.class);
                rushrepairAndMaterial.setRushrepairauditId(rushrepairauditid);
                this.save(rushrepairAndMaterial);
            });
        }
    }
}
