/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.gpsboot.service.rushrepair.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.api.DepartmentClient;
import com.tofly.api.UserClent;
import com.tofly.common.core.constant.ResponeStatus;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.common.system.*;
import com.tofly.gpsboot.mapper.rushrepair.RushrepairMapper;
import com.tofly.gpsboot.service.comm.IFileInfoService;
import com.tofly.gpsboot.service.rushrepair.*;
import com.tofly.gpsboot.util.DateTimeUtil;
import com.tofly.pojo.User;
import com.tofly.pojo.rushrepair.Rushrepair;
import com.tofly.pojo.rushrepair.RushrepairAndSid;
import com.tofly.pojo.rushrepair.Rushrepairaudit;
import com.tofly.pojo.rushrepair.Rushrepairtype;
import com.tofly.pojo.rushrepair.dto.RushrepairDto;
import com.tofly.pojo.rushrepair.dto.RushrepairauditDto;
import com.tofly.pojo.rushrepair.vo.RushrepairVo;
import com.tofly.pojo.rushrepair.vo.RushrepairauditVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 抢修上报信息表
 *
 * @author sunjian
 * @date Tue Mar 21 00:00:00 CST 2023
 */
@Service("rushrepairService")
public class RushrepairServiceImpl extends ServiceImpl<RushrepairMapper, Rushrepair> implements IRushrepairService {

    /**
     * 获取用户服务
     */
    @Autowired
    private UserClent userClent;

    @Autowired
    private DepartmentClient departmentClient;

    /**
     * 获取文件服务
     */
    @Autowired
    private IFileInfoService iFileInfoService;

    @Autowired
    private IRushrepairauditService IRushrepairauditService;

    @Autowired
    private IRushrepairtypeService iRushrepairtypeService;

    @Value("${xrty.ftp.rushRepair}")
    private String rushRepairFilePath;

    @Autowired
    private IRushrepairAndMaterialService rushrepairAndMaterialService;

    @Autowired
    private IRushrepairAndSidService rushrepairAndSidService;

    @Override
    public List<RushrepairVo> viewConversion(List<Rushrepair> infoList) {
        if (CollectionUtils.isEmpty(infoList)) {
            return Collections.emptyList();
        }

        List<RushrepairVo> infoVoList = new ArrayList<>();
        infoList.forEach(info -> {
            RushrepairVo baseInfo = JSONObject.parseObject(JSON.toJSONString(info), RushrepairVo.class);

            baseInfo.setUploadType(RushRepairReportTypeEnum.getCodeForValue(info.getUploadType()));
            baseInfo.setStateName(RushRepairSateEnum.getCodeForValue(info.getState()));

            //添加相关人 名称
            Long createUser = info.getCreateUser();
            if (null != info.getIsNoprocess()) {
                baseInfo.setIsNoprocessName(RushRepairNoProcessEnum.getCodeForValue(info.getIsNoprocess()));
            }
            if (null != createUser && 0 != createUser) {
                User result = userClent.getById(createUser).getResult();
                if (null != result) {
                    baseInfo.setCreateUserName(result.getRealName());
                }
            }
            //抢修类型
            if (null != info.getRushrepairtypeId()) {
                String[] split = info.getRushrepairtypeId().split(",");
                if (split.length > 0) {
                    StringBuilder rushrepairtypeName = new StringBuilder();
                    for (String typeid : split) {
                        Rushrepairtype rushrepairtype = iRushrepairtypeService.getById(typeid);
                        rushrepairtypeName.append(rushrepairtype.getName());
                        rushrepairtypeName.append(",");
                    }
                    baseInfo.setRushrepairtypeName(rushrepairtypeName.substring(0, rushrepairtypeName.length() - 1));
                }
            }

            List<RushrepairAndSid> rushrepairAndSidList = rushrepairAndSidService.list(Wrappers.
                    <RushrepairAndSid>lambdaQuery().eq(RushrepairAndSid::getRushrepairId, baseInfo.getId()));
            baseInfo.setRushrepairAndSidList(rushrepairAndSidList);

            infoVoList.add(baseInfo);
        });
        return infoVoList;
    }

    public List<RushrepairVo> viewConversion(List<Rushrepair> infoList, Boolean isAuditVoShow) {
        if (null == infoList || infoList.size() == 0) {
            return null;
        }
        List<RushrepairVo> rushrepairVos = viewConversion(infoList);
        //是否显示审核VO
        if (isAuditVoShow) {
            rushrepairVos.forEach(baseInfo -> {
                //设置文件对象
                if (!StringUtil.isEmpty(baseInfo.getUploadFileids())) {
                    baseInfo.setFilePathList(iFileInfoService.getFilePathList(Arrays.stream(baseInfo.getUploadFileids().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                            .collect(Collectors.toList())));
                    baseInfo.setFileList(iFileInfoService.getFileList(Arrays.stream(baseInfo.getUploadFileids().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                            .collect(Collectors.toList())));
                }
                //获取审核信息
                List<RushrepairauditVo> rushrepairauditVoList = IRushrepairauditService.getAuditVoByRushRepairId(baseInfo.getId());
                if (null != rushrepairauditVoList && rushrepairauditVoList.size() > 0) {
                    baseInfo.setAuditvo(rushrepairauditVoList);
                }
            });
        }
        return rushrepairVos;
    }

    /**
     * 自定义分页查询
     *
     * @param page
     * @param rushrepairDto
     * @return
     */
    @Override
    public Page<RushrepairVo> getRushRepairPage(Page page, RushrepairDto rushrepairDto) {
        // 审批状态 , 分割查询
        String auditResults = rushrepairDto.getAuditResult();
        Long auditUser = rushrepairDto.getAuditUser();
        Rushrepair rushrepair = JSON.parseObject(JSON.toJSONString(rushrepairDto), Rushrepair.class);
        QueryWrapper<Rushrepair> wrapper = new QueryWrapper();
        QueryWrapper<Rushrepairaudit> wrapper_audit = new QueryWrapper();

        //查询条件存在查询结果
        //只能查询当前登陆人员对应的审核状态
        if (!StringUtils.isEmpty(auditResults) && !StringUtils.isEmpty(auditUser)) {
            wrapper_audit.in("AUDIT_USER", auditUser);
            //分割查询审批状态
            List<String> longList = Arrays.stream(auditResults.split(Constant.CSV)).collect(Collectors.toList());
            wrapper_audit.in("AUDIT_RESULT", longList);

            List<Rushrepairaudit> list = IRushrepairauditService.list(wrapper_audit);
            if (null != list && list.size() > 0) {
                Set<Long> rushrepairIds = list.stream().map(Rushrepairaudit::getRushrepairId).collect(Collectors.toSet());
                if (CollUtil.isNotEmpty(rushrepairIds)) {
                    wrapper.lambda().in(Rushrepair::getId, rushrepairIds);
                }
            }
        }
        wrapper.setEntity(rushrepair);

        //模糊查询
        String address = rushrepairDto.getAddress();
        if (!StringUtils.isEmpty(address)) {
            wrapper.like("ADDRESS", address);
            rushrepair.setAddress(null);
        }
        String alarmpersonName = rushrepairDto.getAlarmpersonName();
        if (!StringUtils.isEmpty(alarmpersonName)) {
            wrapper.lambda().like(Rushrepair::getAlarmpersonName, alarmpersonName);
            rushrepair.setAlarmpersonName(null);
        }
        if (!StringUtils.isEmpty(rushrepairDto.getStartCreateTime()) && !StringUtils.isEmpty(rushrepairDto.getEndCreateTime())) {
            wrapper.lambda().ge(Rushrepair::getCreateTime, rushrepairDto.getStartCreateTime())
                    .le(Rushrepair::getCreateTime, rushrepairDto.getEndCreateTime());
        }
        if (org.apache.commons.lang3.StringUtils.isNotBlank(rushrepairDto.getSid())) {
            List<String> sidList = Arrays.asList(rushrepairDto.getSid().split(","));
            List<RushrepairAndSid> rushrepairAndSidList = rushrepairAndSidService.list(new QueryWrapper<RushrepairAndSid>().lambda().in(RushrepairAndSid::getSId, sidList));
            if (CollUtil.isNotEmpty(rushrepairAndSidList)) {
                wrapper.in("ID", rushrepairAndSidList.stream().map(RushrepairAndSid::getRushrepairId).distinct().collect(Collectors.toList()));
            } else {
                wrapper.in("ID", 0L);
            }
        }
        wrapper.orderByDesc("CREATE_TIME");

        Page infoPageList = this.page(page, wrapper);
        infoPageList.setRecords(this.viewConversion(infoPageList.getRecords()));
        return infoPageList;
    }

    /**
     * 通过id进行查询
     *
     * @param id
     * @return
     */
    @Override
    public RushrepairVo getRushRepairById(Long id) {
        if (null != id || 0 != id) {
            Rushrepair rushrepair = this.getById(id);
            if (null != rushrepair) {
                List<Rushrepair> rushrepairList = new ArrayList<>();
                rushrepairList.add(rushrepair);
                List<RushrepairVo> rushrepairVos = this.viewConversion(rushrepairList, true);
                if (null != rushrepairVos && rushrepairVos.size() > 0) {
                    return rushrepairVos.get(0);
                }
            }
        }
        return null;
    }

    /**
     * 保存记录
     *
     * @param files
     * @param auditFiles
     * @param rushrepairDto
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Rushrepair savaBuildInfo(List<MultipartFile> files, List<MultipartFile> auditFiles, RushrepairDto rushrepairDto) {
        StringBuilder fileIdList = new StringBuilder("");
        // 判断当前是否有文件,则存入返回文件id
        if (null != files && files.size() > 0) {
            fileIdList = iFileInfoService.saveFileList(files, rushRepairFilePath, FileTableEnum.TF_YWPN_RUSHREPAIR_W.getTableName());
        }
        // 根据文件id反射填充对象
        Rushrepair info = JSONObject.parseObject(JSON.toJSONString(rushrepairDto), Rushrepair.class);
        info.setUploadFileids(fileIdList.toString());

        // 0正常处理，进入1安全运维部管理员审核；1无需处理，进入8环投建工
        info.setState(RushRepairNoProcessEnum.RUSHREPAIR_NO_HANDLE.getCode().equals(info.getIsNoprocess()) ? "6" : "8");

        // 主表对象入库
        this.save(info);

        //新增抢修单 默认审核不传附件
        Date date = DateTimeUtil.convertLDTToDate(LocalDateTime.now());
        RushrepairauditDto rushrepairauditDto = new RushrepairauditDto();
        rushrepairauditDto.setRushrepairId(info.getId());
        rushrepairauditDto.setAuditDept(rushrepairDto.getRushRepairNextAuditDeptId());
        rushrepairauditDto.setAuditUser(rushrepairDto.getRushRepairNextAuditId());
        rushrepairauditDto.setAuditTime(date);
        rushrepairauditDto.setAuditResult(AuditSateEnum.AUDIT_UNCOMMITTED.getCode());

        // 0正常处理 进入1  1无需处理 新增一条审核记录0 新增一条审核记录6
        if (RushRepairNoProcessEnum.RUSHREPAIR_NO_HANDLE.getCode().equals(info.getIsNoprocess())) {
            // 新增一条审核记录0
            RushrepairauditDto new_rushrepairauditDto = new RushrepairauditDto();
            new_rushrepairauditDto.setRushrepairId(info.getId());
            new_rushrepairauditDto.setAuditNotes(rushrepairDto.getAuditNotes());
            new_rushrepairauditDto.setAuditDept(rushrepairDto.getRushRepairNextAuditDeptId());
            new_rushrepairauditDto.setAuditUser(rushrepairDto.getRushRepairNextAuditId());
            new_rushrepairauditDto.setAuditResult(AuditSateEnum.AUDIT_YES.getCode());
            new_rushrepairauditDto.setType("0");
            //处理时间 第一次审核 根据审核时间排序 所以减一秒
            Calendar rightNow = Calendar.getInstance();
            rightNow.setTime(date);
            rightNow.add(Calendar.SECOND, -1);// 减一秒
            new_rushrepairauditDto.setAuditTime(rightNow.getTime());
            // 审核表对象入库 设置上步id
            rushrepairauditDto.setBeforeAuditid(IRushrepairauditService.savaBuildInfo(auditFiles, new_rushrepairauditDto));
            //设置状态 6
            rushrepairauditDto.setType("6");
            //自行处理如果有材料需要添加 ,添加设备图层id
            if (null != rushrepairDto.getMaterialList() && rushrepairDto.getMaterialList().size() > 0) {
                rushrepairAndMaterialService.saveRushrepairAndMaterial(rushrepairauditDto.getBeforeAuditid(), rushrepairDto.getMaterialList());
            }
            if (null != rushrepairDto.getRushrepairAndSidList() && rushrepairDto.getRushrepairAndSidList().size() > 0) {
                rushrepairAndSidService.saveRushrepairAndSid(info.getId(), rushrepairDto.getRushrepairAndSidList());
            }
        } else {
            rushrepairauditDto.setType("8");
        }

        // 审核表对象入库
        IRushrepairauditService.savaBuildInfo(null, rushrepairauditDto);
        return info;
    }

    /**
     * 通过id批量删除抢修上报信息表（自动删除审核表信息）
     *
     * @param ids
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResultRespone<Object> removeByIdList(String ids) {
        ResultRespone<Object> rest = new ResultRespone<>();
        if (!StringUtil.isEmpty(ids)) {
            //删除审核表信息
            Arrays.asList(ids.split(Constant.CSV)).forEach(id -> {
                List<RushrepairauditVo> auditVoByRushRepairId = IRushrepairauditService.getAuditVoByRushRepairId(Long.valueOf(id));
                String auditids = auditVoByRushRepairId.stream().map(p -> Long.toString(p.getId())).collect(Collectors.joining(Constant.CSV));
                if (!IRushrepairauditService.removeByIds(Arrays.asList(auditids.split(Constant.CSV)))) {
                    log.error("通过ID删除审核信息对象失败:auditids = " + auditids);
                    rest.setMessage(Constant.DELETE_FAIL);
                    rest.setCode(Constant.DELETE_FAIL_CODE);
                }
            });
            if (!this.removeByIds(Arrays.asList(ids.split(Constant.CSV)))) {
                log.error("通过ID删除基本信息对象失败:ids = " + ids);
                rest.setMessage(Constant.DELETE_FAIL);
                rest.setCode(Constant.DELETE_FAIL_CODE);
            }
        } else {
            log.error("通过ID删除基本信息对象失败异常:ids不能为空！");
            rest.setCode(ResponeStatus.FAIL.getCode());
            rest.setMessage(Constant.DELETE_FAIL);
        }
        return rest;
    }

    @Override
    public List<Map> getSidSelect(String typeid, Map map) {
        return baseMapper.getSidSelect(typeid, map);
    }



}
