/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.gpsboot.service.rushrepair.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.api.DepartmentClient;
import com.tofly.api.UserClent;
import com.tofly.common.core.util.StringUtil;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.common.system.*;
import com.tofly.gpsboot.mapper.rushrepair.RushrepairauditMapper;
import com.tofly.gpsboot.service.comm.IFileInfoService;
import com.tofly.gpsboot.service.report.IReportRushRepairServer;
import com.tofly.gpsboot.service.rushrepair.IRushrepairAndMaterialService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairService;
import com.tofly.gpsboot.service.rushrepair.IRushrepairauditService;
import com.tofly.pojo.Department;
import com.tofly.pojo.User;
import com.tofly.pojo.plan.dto.ReportAuditDto;
import com.tofly.pojo.rushrepair.Rushrepair;
import com.tofly.pojo.rushrepair.Rushrepairaudit;
import com.tofly.pojo.rushrepair.dto.RushrepairauditDto;
import com.tofly.pojo.rushrepair.vo.RushrepairAndMaterialVo;
import com.tofly.pojo.rushrepair.vo.RushrepairauditVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 抢修信息审核表
 *
 * @author sunjian
 * @date Tue Mar 21 00:00:00 CST 2023
 */
@Service("rushrepairauditService")
public class RushrepairauditServiceImpl extends ServiceImpl<RushrepairauditMapper, Rushrepairaudit> implements IRushrepairauditService {

    @Autowired
    private UserClent userClent;

    @Autowired
    private DepartmentClient departmentClient;
    /**
     * 获取文件服务
     */
    @Autowired
    private IFileInfoService iFileInfoService;

    @Value("${xrty.ftp.rushRepairAudit}")
    private String rushRepairAuditFilePath;

    @Autowired
    private IReportRushRepairServer iReportRushRepairServer;

    @Autowired
    private IRushrepairService IRushrepairService;

    @Autowired
    private IRushrepairAndMaterialService rushrepairAndMaterialService;

    @Override
    public List<RushrepairauditVo> viewConversion(List<Rushrepairaudit> infoList) {
        if (CollectionUtils.isEmpty(infoList)) {
            return Collections.emptyList();
        }

        List<RushrepairauditVo> infoVoList = new ArrayList<>();
        infoList.forEach(info -> {
            Rushrepair rushrepair = IRushrepairService.getById(info.getRushrepairId());
            RushrepairauditVo baseInfo = JSONObject.parseObject(JSON.toJSONString(info), RushrepairauditVo.class);
            //添加主表信息
            baseInfo.setAlarmpersonName(rushrepair.getAlarmpersonName());
            baseInfo.setAlarmpersonPhone(rushrepair.getAlarmpersonPhone());
            baseInfo.setUserCode(rushrepair.getUserCode());
            baseInfo.setReportLgtd(rushrepair.getReportLgtd());
            baseInfo.setReportLttd(rushrepair.getReportLttd());
            baseInfo.setHandleLgtd(rushrepair.getHandleLgtd());
            baseInfo.setHandleLttd(rushrepair.getHandleLttd());
            baseInfo.setAddress(rushrepair.getAddress());
            baseInfo.setPhone(rushrepair.getPhone());
            baseInfo.setDetailed(rushrepair.getDetailed());

            //添加审核表信息
            baseInfo.setTypeName(RushRepairSateEnum.getCodeForValue(info.getType()));
            baseInfo.setIsSubmit(IsSubmitEnum.getCodeForValue(info.getIsSubmit()));

            //材料
            List<RushrepairAndMaterialVo> rushrepairAndMaterialVoList = rushrepairAndMaterialService.getRushrepairAndMaterialByRushRepairId(baseInfo.getId());
            if (null != rushrepairAndMaterialVoList && rushrepairAndMaterialVoList.size() > 0) {
                baseInfo.setMaterialvo(rushrepairAndMaterialVoList);
            }
            //添加相关人 名称
            Long createUser = rushrepair.getCreateUser();//创建人和时间展示为主表的创建人和创建时间
            Long auditDept = info.getAuditDept();
            Long auditUser = info.getAuditUser();
            if (null != rushrepair.getIsNoprocess()) {
                baseInfo.setIsNoprocessName(rushrepair.getIsNoprocess());
                baseInfo.setIsNoprocessName(RushRepairNoProcessEnum.getCodeForValue(rushrepair.getIsNoprocess()));
            }
            if (null != info.getIsCheck()) {
                baseInfo.setIsCheck(info.getIsCheck());
                baseInfo.setIsCheckName(RushRepairIsCheckEnum.getCodeForValue(info.getIsCheck()));
            }

            if (null != createUser && 0 != createUser) {
                User result = userClent.getById(createUser).getResult();
                if (null != result) {
                    baseInfo.setCreateUserName(result.getRealName());
                    baseInfo.setCreateTime(rushrepair.getCreateTime());//创建人和时间展示为主表的创建人和创建时间
                }
            }
            if (null != auditUser && 0 != auditUser) {
                User result = userClent.getById(auditUser).getResult();
                if (null != result) {
                    baseInfo.setAuditUserName(result.getRealName());
                }
            }
            if (null != auditDept && 0 != auditDept) {
                Department result = departmentClient.getById(auditDept).getResult();
                if (null != result) {
                    baseInfo.setAuditDeptName(result.getName());
                }
            }
            //设置文件对象
            if (!StringUtil.isEmpty(info.getUploadFileids())) {
                baseInfo.setFilePathList(iFileInfoService.getFilePathList(Arrays.stream(info.getUploadFileids().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                        .collect(Collectors.toList())));
                baseInfo.setFileList(iFileInfoService.getFileList(Arrays.stream(info.getUploadFileids().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                        .collect(Collectors.toList())));
            }
            infoVoList.add(baseInfo);
        });
        return infoVoList;
    }

    /**
     * 自定义分页查询
     *
     * @param page
     * @param rushrepairauditDto
     * @return
     */
    @Override
    public Page<RushrepairauditVo> getRushRepairAuditPage(Page page, RushrepairauditDto rushrepairauditDto) {
        Rushrepairaudit rushrepairaudit = JSON.parseObject(JSON.toJSONString(rushrepairauditDto), Rushrepairaudit.class);
        QueryWrapper<Rushrepairaudit> wrapper = new QueryWrapper();
        wrapper.setEntity(rushrepairaudit);
        wrapper.orderByDesc("CREATE_TIME");
        Page infoPageList = this.page(page, wrapper);
        infoPageList.setRecords(this.viewConversion(infoPageList.getRecords()));
        return infoPageList;
    }

    /**
     * 添加记录
     *
     * @param fileList
     * @param rushrepairauditDto
     */
    @Transactional(rollbackFor = Exception.class)
    public Long savaBuildInfo(List<MultipartFile> fileList, RushrepairauditDto rushrepairauditDto) {
        StringBuilder fileIdList = new StringBuilder("");
        // 判断当前是否有文件,则存入返回文件id
        if (null != fileList && fileList.size() > 0) {
            fileIdList = iFileInfoService.saveFileList(fileList, rushRepairAuditFilePath, FileTableEnum.TF_YWPN_RUSHREPAIRAUDIT_W.getTableName());
        }

        // 根据文件id反射填充对象
        Rushrepairaudit info = JSONObject.parseObject(JSON.toJSONString(rushrepairauditDto), Rushrepairaudit.class);
        info.setUploadFileids(fileIdList.toString());
        // 主表对象入库
        this.save(info);

        // 自动上报
        ReportAuditDto dto = new ReportAuditDto();
        dto.setId(info.getId());
        dto.setAuditNotes(info.getAuditNotes() == null ? "有新的抢修审核！" : info.getAuditNotes());
        dto.setAuditId(info.getAuditUser());
        iReportRushRepairServer.report(dto);

        return info.getId();
    }

    /**
     * 根据抢修单对应的审核记录列表
     *
     * @param rushrepairid 主表id
     * @return
     */
    @Override
    public List<RushrepairauditVo> getAuditVoByRushRepairId(Long rushrepairid) {
        List<RushrepairauditVo> rushrepairauditVos = new ArrayList<>();
        List<Rushrepairaudit> rushrepairauditList = this.list(new QueryWrapper<Rushrepairaudit>().lambda().eq(Rushrepairaudit::getRushrepairId, rushrepairid).orderByAsc(Rushrepairaudit::getCreateTime));
        if (null != rushrepairauditList && rushrepairauditList.size() > 0) {
            return this.viewConversion(rushrepairauditList);
        }
        return null;
    }

//    /**
//     * 查询审核人审核列表
//     *
//     * @param dto
//     * @return
//     */
//    private List<RushrepairauditVo> getAuditVoByAuditUser(RushrepairauditDto dto) {
//        List<RushrepairauditVo> rushrepairauditVos = new ArrayList<>();
//        Long audituser = dto.getAuditUser();
//
//        LambdaQueryWrapper<Rushrepairaudit> queryWrapper = new QueryWrapper<Rushrepairaudit>().lambda().eq(Rushrepairaudit::getAuditUser, ).eq(Rushrepairaudit::getAuditResult, AuditSateEnum.AUDIT_UNCOMMITTED.getCode());
//        List<Rushrepairaudit> rushrepairauditList = this.list();
//        if (null != rushrepairauditList && rushrepairauditList.size() > 0) {
//            rushrepairauditList.forEach(info -> {
//                RushrepairauditVo rushrepairauditVo = JSON.parseObject(JSON.toJSONString(info), RushrepairauditVo.class);
//                rushrepairauditVos.add(rushrepairauditVo);
//            });
//        }
//        if (rushrepairauditVos.size() > 0) {
//            return rushrepairauditVos;
//        }
//        return null;
//    }
}
