/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.gpsboot.service.valvewell.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.api.DepartmentClient;
import com.tofly.api.UserClent;
import com.tofly.gpsboot.common.Constant;
import com.tofly.gpsboot.common.system.FileTableEnum;
import com.tofly.gpsboot.exception.BusinessException;
import com.tofly.gpsboot.mapper.plan.ValvewellMapper;
import com.tofly.gpsboot.service.comm.IAuditService;
import com.tofly.gpsboot.service.comm.IDisposeDateService;
import com.tofly.gpsboot.service.comm.IFileInfoService;
import com.tofly.gpsboot.service.valvewell.ValvewelldetailService;
import com.tofly.gpsboot.service.valvewell.ValvewellService;
import com.tofly.pojo.comm.File;
import com.tofly.pojo.maintain.Repairmaterial;
import com.tofly.pojo.plan.Valvewell;
import com.tofly.pojo.plan.Valvewelldetail;
import com.tofly.pojo.plan.dto.ValvewellDto;
import com.tofly.pojo.plan.dto.ValvewelldetailDto;
import com.tofly.pojo.plan.vo.ValvewellVo;
import com.tofly.pojo.statement.vo.MaterialConditionsVo;
import com.tofly.pojo.valewell.dto.valewellStaDto;
import com.tofly.pojo.valewell.vo.valewellStaVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 阀门井信息表
 *
 * @author admin
 * @date Wed Mar 29 00:00:00 GMT+08:00 2023
 */
@Service("valvewellService")
public class ValvewellServiceImpl extends ServiceImpl<ValvewellMapper, Valvewell> implements ValvewellService {
    @Autowired
    private ValvewelldetailService valvewelldetailService;

   @Autowired
   private ValvewellService valvewellService;

    @Autowired
    private IFileInfoService iFileInfoService;
    @Value("${xrty.ftp.pressRecord}")
    private String pressRecordFilePath;

    @Override
    public List<ValvewellVo> viewConversion(List<Valvewell> infoList) {
        List<ValvewellVo> infoVoList = new ArrayList<>();
        infoList.forEach(info -> {
            ValvewellVo valvewellVo=JSON.parseObject(JSON.toJSONString(info),ValvewellVo.class);
            List<Valvewelldetail> valvewelldetaillist = valvewelldetailService.list(new QueryWrapper<Valvewelldetail>().lambda().eq(Valvewelldetail::getVwId, info.getId()));
            valvewellVo.setValvewelldetailList(valvewelldetaillist);
            if(!StringUtils.isEmpty(info.getFileIds())){
                valvewellVo.setFilesPathList(iFileInfoService.getFilePathList(Arrays.stream(info.getFileIds().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                        .collect(Collectors.toList())));
            }
            infoVoList.add(valvewellVo);
        });
        return infoVoList;
    }

    @Override
    public Page<ValvewellVo> getValvewellPage(Page page, ValvewellDto parDto) {
        Valvewell valvewell = JSON.parseObject(JSON.toJSONString(parDto), Valvewell.class);
        QueryWrapper<Valvewell> wrapper = new QueryWrapper();
//        long id = parDto.getId();
//        if (!StringUtils.isEmpty(id)) {
//            wrapper.like("id", id);
//            valvewell.setId(null);
//        }
        wrapper.setEntity(valvewell);
        Page infoPageList = this.page(page, wrapper);

        infoPageList.setRecords(this.viewConversion(infoPageList.getRecords()));

        return infoPageList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ValvewellDto savaValvewellInfo(List<MultipartFile> fileList, ValvewellDto parDto) throws BusinessException {
        StringBuilder fileIdList = new StringBuilder("");
        // 判断当前是否有文件,则存入返回文件id
        if (null != fileList && fileList.size() > 0) {
            fileIdList = iFileInfoService.saveFileList(fileList, pressRecordFilePath, FileTableEnum.TF_YWPN_VALVEWELL_W.getTableName());
        }
        if (null != parDto.getValvewelldetailVoList() && parDto.getValvewelldetailVoList().size() > 0) {
            // JSON转换
            Valvewell valvewell = JSON.parseObject(JSON.toJSONString(parDto), Valvewell.class);
            valvewell.setFileIds(fileIdList.toString());
            this.save(valvewell);

            Long id = valvewell.getId();
            List<Valvewelldetail> detailids = JSONArray.parseArray(JSON.toJSONString(parDto.getValvewelldetailVoList()), Valvewelldetail.class);
            detailids.forEach(info -> {
                info.setVwId(id);
                valvewelldetailService.save(info);
            });
            parDto.setId(valvewell.getId());
            parDto.setCreateUser(valvewell.getCreateUser());
            return parDto;
        } else {
            throw new BusinessException(Constant.PLAN_YEAR_DETAIL_NULL_CODE, Constant.PLAN_YEAR_DETAIL_NULL);
        }
    }


    @Override
    public ValvewellVo getValvewellInfo(Long id) throws BusinessException {
        Valvewell info = this.getById(id);
        Optional.ofNullable(info).orElseThrow(() -> new BusinessException(Constant.PLAN_YEAR_NULL_CODE, Constant.PLAN_YEAR_NULL));
        ValvewellVo valvewellVo = JSON.parseObject(JSON.toJSONString(info), ValvewellVo.class);
        List<Valvewelldetail> list = valvewelldetailService.list(new QueryWrapper<Valvewelldetail>().lambda().eq(Valvewelldetail::getVwId, valvewellVo.getId()));
        if(!StringUtils.isEmpty(info.getFileIds())){
            valvewellVo.setFilesPathList(iFileInfoService.getFilePathList(Arrays.stream(info.getFileIds().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                    .collect(Collectors.toList())));
        }
        valvewellVo.setValvewelldetailList(list);


        return valvewellVo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateValvewellInfo(List<MultipartFile> fileList, ValvewellDto parDto) throws BusinessException {
        Valvewell updatevalvewell = JSON.parseObject(JSON.toJSONString(parDto), Valvewell.class);
        List<Long> fileIdLongList = new ArrayList<>();
        if (null != fileList && fileList.size() > 0) {
            StringBuilder fileIdList = iFileInfoService.saveFileList(fileList, pressRecordFilePath, FileTableEnum.TF_YWPN_VALVEWELL_W.getTableName());
            fileIdLongList = Arrays.stream(fileIdList.toString().split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                    .collect(Collectors.toList());
        }
        Valvewell valvewell = this.getById(parDto.getId());
        String lostFileIds = valvewell.getFileIds();
        String fileIds = parDto.getFileIds();
        List<Long> removeFileId = new ArrayList<>();
        List<Long> fileIdLongs = new ArrayList<>();

        if (!StringUtils.isEmpty(fileIds)) {
            fileIdLongs = Arrays.stream(fileIds.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                    .collect(Collectors.toList());
        }
        // 判断当前文件是否有更新
        if (!StringUtils.isEmpty(lostFileIds) && lostFileIds.equals(fileIds)) {
            List<Long> lostFileIdLongs = Arrays.stream(lostFileIds.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                    .collect(Collectors.toList());

            List<Long> finalFileIdLongs = fileIdLongs;
            lostFileIdLongs.forEach(lost -> {
                if (!finalFileIdLongs.contains(lost)) {
                    removeFileId.add(lost);
                }
            });
            // 删除对应文件数据
            if (removeFileId.size() > 0) {
                // 删除远程文件
                iFileInfoService.removeFileList(removeFileId);
            }
            if (lostFileIdLongs.size() > 0) {
                // 删除远程文件
                iFileInfoService.removeFileList(lostFileIdLongs);
            }

        }
        // 更新数据信息
        fileIdLongList.addAll(fileIdLongs);
        if (fileIdLongList.size() > 0) {
            StringBuilder stringBuilder = new StringBuilder("");
            fileIdLongList.forEach(info -> {
                stringBuilder.append(info).append(Constant.CSV);
            });
            updatevalvewell.setFileIds(stringBuilder.deleteCharAt(stringBuilder.length() - 1).toString());
            this.updateById(updatevalvewell);
        } else {
            this.updateById(updatevalvewell);
        }
        List<ValvewelldetailDto> valvewelldetailDtoList = parDto.getValvewelldetailVoList();

        valvewelldetailService.removeByIds(valvewelldetailService.list(new QueryWrapper<Valvewelldetail>().lambda().eq(Valvewelldetail::getVwId, valvewell.getId())).stream().map(Valvewelldetail::getId).collect(Collectors.toList()));
        if (null != valvewelldetailDtoList && valvewelldetailDtoList.size() > 0) {
            List<Valvewelldetail> valvewelldetailList = JSONArray.parseArray(JSON.toJSONString(valvewelldetailDtoList), Valvewelldetail.class);
            valvewelldetailList.forEach(info -> {
                info.setVwId(parDto.getId());
                valvewelldetailService.save(info);
            });
        }

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteValvewellInfo(List<Long> idList) throws BusinessException {
        List<Long> removeFileIdList = new ArrayList<>();
        List<Valvewell> valvewellList = this.list(new QueryWrapper<Valvewell>().lambda().in(Valvewell::getId, idList));
        valvewellList.forEach(info -> {
            String fileIds = info.getFileIds();
            if (!StringUtils.isEmpty(fileIds)) {
                List<Long> fileIdLongs = Arrays.stream(fileIds.split(Constant.CSV)).map(s -> Long.parseLong(s.trim()))
                        .collect(Collectors.toList());
                removeFileIdList.addAll(fileIdLongs);
            }
            List<Valvewelldetail> list = valvewelldetailService.list(new QueryWrapper<Valvewelldetail>().lambda().in(Valvewelldetail::getVwId, idList));
            this.removeByIds(idList);
            if (Optional.ofNullable(list).isPresent()) {
                List<Long> ids = list.stream().map(Valvewelldetail::getVwId).collect(Collectors.toList());
                valvewelldetailService.removeByIds(ids);
            }
        });

    }

    @Override
    public List<valewellStaVo> getValvewellSta(valewellStaDto dto) {
        List<valewellStaVo> voList = new ArrayList<>();
        List<String> sidList=Arrays.asList(dto.getSid().split(Constant.CSV));
        List<Valvewell> valvewellList=valvewellService.list(new QueryWrapper<Valvewell>().lambda().
                in(null!=sidList,Valvewell::getSid,sidList));
        if(null!=valvewellList&&valvewellList.size()>0){
            Map<String,List<Valvewell>> collect=valvewellList.stream().collect(Collectors.groupingBy(Valvewell::getSid));
            collect.forEach((k,v)->{
                valewellStaVo vo=new valewellStaVo();
                vo.setSid(k);
                long sum=v.size();
                vo.setStaNum(sum);
                voList.add(vo);
            });
        }
        return voList;
    }

}
