/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtymis.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtymis.custom.service.TfYwpnManholeService;
import com.tofly.xrtymis.entity.TfYwpnManhole;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;

/**
 * 检查井信息采集数据表：TF_YWPN_MANHOLE_W
 *
 * @author fcl
 * @date Fri Apr 16 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfywpnmanhole")
@Api(tags="检查井信息采集数据表：TF_YWPN_MANHOLE_W接口")
public class TfYwpnManholeController {

    private final TfYwpnManholeService tfYwpnManholeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfYwpnManhole 检查井信息采集数据表：TF_YWPN_MANHOLE_W
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTfYwpnManholePage(Page page, @ModelAttribute TfYwpnManhole tfYwpnManhole) {
      return ResultRespone.success(tfYwpnManholeService.page(page, Wrappers.query(tfYwpnManhole)));
    }

    /**
     * 通过id查询检查井信息采集数据表：TF_YWPN_MANHOLE_W
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(tfYwpnManholeService.getById(id));
    }

    /**
     * 新增检查井信息采集数据表：TF_YWPN_MANHOLE_W
     * @param tfYwpnManhole 检查井信息采集数据表：TF_YWPN_MANHOLE_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增检查井信息采集数据表：TF_YWPN_MANHOLE_W" )
    @ApiOperation(value = "新增检查井信息采集数据表：TF_YWPN_MANHOLE_W")
    @PostMapping("/add")
    public ResultRespone save(TfYwpnManhole tfYwpnManhole) {
      return tfYwpnManholeService.saveCheckWell(tfYwpnManhole);
    }

    /**
     * 修改检查井信息采集数据表：TF_YWPN_MANHOLE_W
     * @param tfYwpnManhole 检查井信息采集数据表：TF_YWPN_MANHOLE_W
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改检查井信息采集数据表：TF_YWPN_MANHOLE_W" )
    @ApiOperation(value = "修改检查井信息采集数据表：TF_YWPN_MANHOLE_W")
    @PostMapping("/update")
    public ResultRespone updateById(List<MultipartFile> fileList, TfYwpnManhole tfYwpnManhole) {
      return tfYwpnManholeService.updateManholeById(fileList,tfYwpnManhole);
    }

    /**
     * 通过id删除检查井信息采集数据表：TF_YWPN_MANHOLE_W
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除检查井信息采集数据表：TF_YWPN_MANHOLE_W" )
    @ApiOperation(value = "通过ID删除检查井信息采集数据表：TF_YWPN_MANHOLE_W")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return tfYwpnManholeService.removeManholeById(id);
    }

//    /**
//     * 通过id批量删除检查井信息采集数据表：TF_YWPN_MANHOLE_W
//     * @param ids id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id批量删除检查井信息采集数据表：TF_YWPN_MANHOLE_W" )
//    @ApiOperation(value = "通过ID批量删除检查井信息采集数据表：TF_YWPN_MANHOLE_W")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
//    })
//    @DeleteMapping("/deleteByIds" )
//    public ResultRespone removeById(String ids) {
//        return ResultRespone.success(tfYwpnManholeService.removeByIds(Arrays.asList(ids.split(","))));
//    }

}
