/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtymis.custom.controller.osm;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtymis.custom.service.osm.CustomersService;
import com.tofly.xrtymis.entity.osm.Customers;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;

/**
 * 营销档案数据
 *
 * @author sunjian
 * @date Tue Jul 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/customers")
@Api(tags = "营销档案数据接口")
public class CustomersController {

    private final CustomersService customersService;

    /**
     * 分页查询
     *
     * @param page      分页对象
     * @param customers 营销档案数据
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getCustomersPage(Page page, @ModelAttribute Customers customers) {
        return ResultRespone.success(customersService.page(page, Wrappers.query(customers)));
    }


    /**
     * 通过id查询营销档案数据
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(customersService.getById(id));
    }

    /**
     * 新增营销档案数据
     *
     * @param customers 营销档案数据
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增营销档案数据")
    @ApiOperation(value = "新增营销档案数据")
    @PostMapping
    public ResultRespone save(@RequestBody Customers customers) {
        return ResultRespone.success(customersService.save(customers));
    }

    /**
     * 修改营销档案数据
     *
     * @param customers 营销档案数据
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改营销档案数据")
    @ApiOperation(value = "修改营销档案数据")
    @PutMapping
    public ResultRespone updateById(@RequestBody Customers customers) {
        return ResultRespone.success(customersService.updateById(customers));
    }

    /**
     * 通过id删除营销档案数据
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除营销档案数据")
    @ApiOperation(value = "通过ID删除营销档案数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(customersService.removeById(id));
    }

    /**
     * 通过id批量删除营销档案数据
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除营销档案数据")
    @ApiOperation(value = "通过ID批量删除营销档案数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(customersService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
