/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtymis.custom.controller.osm;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtymis.custom.service.osm.PaymentsService;
import com.tofly.xrtymis.entity.osm.Payments;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 营销缴费数据
 *
 * @author sunjian
 * @date Tue Jul 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/payments")
@Api(tags="营销缴费数据接口")
public class PaymentsController {

    private final PaymentsService paymentsService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param payments 营销缴费数据
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPaymentsPage(Page page, @ModelAttribute Payments payments) {
      return ResultRespone.success(paymentsService.page(page, Wrappers.query(payments)));
    }


    /**
     * 通过id查询营销缴费数据
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(paymentsService.getById(id));
    }

    /**
     * 新增营销缴费数据
     * @param payments 营销缴费数据
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增营销缴费数据" )
    @ApiOperation(value = "新增营销缴费数据")
    @PostMapping
    public ResultRespone save(@RequestBody Payments payments) {
      return ResultRespone.success(paymentsService.save(payments));
    }

    /**
     * 修改营销缴费数据
     * @param payments 营销缴费数据
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改营销缴费数据" )
    @ApiOperation(value = "修改营销缴费数据")
    @PutMapping
    public ResultRespone updateById(@RequestBody Payments payments) {
      return ResultRespone.success(paymentsService.updateById(payments));
    }

    /**
     * 通过id删除营销缴费数据
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除营销缴费数据" )
    @ApiOperation(value = "通过ID删除营销缴费数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(paymentsService.removeById(id));
    }

    /**
     * 通过id批量删除营销缴费数据
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除营销缴费数据" )
    @ApiOperation(value = "通过ID批量删除营销缴费数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(paymentsService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
