/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.xrtymis.custom.controller.osm;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.xrtymis.custom.service.osm.RecordsService;
import com.tofly.xrtymis.entity.osm.Records;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 营销抄表数据
 *
 * @author sunjian
 * @date Tue Jul 18 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/records")
@Api(tags="营销抄表数据接口")
public class RecordsController {

    private final RecordsService recordsService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param records 营销抄表数据
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getRecordsPage(Page page, @ModelAttribute Records records) {
      return ResultRespone.success(recordsService.page(page, Wrappers.query(records)));
    }


    /**
     * 通过id查询营销抄表数据
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(recordsService.getById(id));
    }

    /**
     * 新增营销抄表数据
     * @param records 营销抄表数据
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增营销抄表数据" )
    @ApiOperation(value = "新增营销抄表数据")
    @PostMapping
    public ResultRespone save(@RequestBody Records records) {
      return ResultRespone.success(recordsService.save(records));
    }

    /**
     * 修改营销抄表数据
     * @param records 营销抄表数据
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改营销抄表数据" )
    @ApiOperation(value = "修改营销抄表数据")
    @PutMapping
    public ResultRespone updateById(@RequestBody Records records) {
      return ResultRespone.success(recordsService.updateById(records));
    }

    /**
     * 通过id删除营销抄表数据
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除营销抄表数据" )
    @ApiOperation(value = "通过ID删除营销抄表数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(recordsService.removeById(id));
    }

    /**
     * 通过id批量删除营销抄表数据
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除营销抄表数据" )
    @ApiOperation(value = "通过ID批量删除营销抄表数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(recordsService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
