/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.xrtymis.custom.service.osm.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.xrtymis.custom.service.osm.CustomersService;
import com.tofly.xrtymis.custom.utils.DateTimeUtil;
import com.tofly.xrtymis.entity.osm.*;
import com.tofly.xrtymis.mapper.osm.CustomersMapper;
import com.tofly.xrtymis.util.ApiClient;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 营销档案数据
 *
 * @author sunjian
 * @date Tue Jul 18 00:00:00 CST 2023
 */
@Service("customersService")
@Slf4j
public class CustomersServiceImpl extends ServiceImpl<CustomersMapper, Customers> implements CustomersService {
    @Autowired
    private LangChaoUserInfo langChaoUserInfo;

    @Autowired
    private LangChaoParam langChaoParam;

    /**
     * 同步营销系统数据(每天晚上同步新增的数据)
     *
     * @return
     */
    @XxlJob("osmSynchronousUserInfoHandler")
    public ReturnT<String> osmSynchronousUserInfo() {
        long l = System.currentTimeMillis();
        LocalDate localDate = LocalDate.now();
        Date date = DateTimeUtil.localDateToDate(localDate);
        log.info("开始同步浪潮营销系统数据,日期{}", date);
        //组装请求参数
        Map<String, Object> headmap = BeanUtil.beanToMap(langChaoUserInfo);
        langChaoParam.setStart("1");
        langChaoParam.setLimit("100");
        Map<String, Object> map = BeanUtil.beanToMap(langChaoParam);
        String params = transformationParams(map);
        //开始请求
        Object o = ApiClient.getReset(langChaoUserInfo.getUrl(), headmap, params);
        LangChaoUserInfoResult langChaoUserInfoResult = JSONObject.parseObject(JSONObject.toJSONString(o), LangChaoUserInfoResult.class);
        //状态为0 则是成功
        if (langChaoUserInfoResult.getCode().equals("0")) {
            LangChaoUserInfoResultModel result = langChaoUserInfoResult.getResult();
            langChaoUserInfoResult.getResult().getData();
            //保存数据
            if (saveBatchCustomer(langChaoUserInfoResult.getResult().getData())) {
                //从第二页开始同步 上面同步的第一页
                int i = 2;
                while (i * 100 < result.getTotal()) {
                    langChaoParam.setStart(String.valueOf(i));
                    map.clear();
                    map = BeanUtil.beanToMap(langChaoParam);
                    params = transformationParams(map);
                    o = ApiClient.getReset(langChaoUserInfo.getUrl(), headmap, params);
                    langChaoUserInfoResult = JSONObject.parseObject(JSONObject.toJSONString(o), LangChaoUserInfoResult.class);
                    //状态为0 则是成功
                    if (langChaoUserInfoResult.getCode().equals("0")) {
                        List<LangChaoUserInfoResultModelData> data = langChaoUserInfoResult.getResult().getData();
                        //保存数据 只同步大于当前日期减去一天的数据
                        if (!saveBatchCustomer(data)) {
                            log.info("同步条数：{}", i * 100);
                            return ReturnT.SUCCESS;
                        }
                    }
                    i++;
                    log.info("同步条数：{}", i * 100);
                }
            }

        } else {
            log.info("同步结束失败浪潮营销系统数据！");
        }
        long l1 = System.currentTimeMillis();
        log.info("同步结束浪潮营销系统数据,用时{}mm", l1 - l);
        return ReturnT.SUCCESS;
    }

    /**
     * 批量保存
     *
     * @param data
     */
    public boolean saveBatchCustomer(List<LangChaoUserInfoResultModelData> data) {
        List<Customers> listCustomer = new ArrayList<>();
        LocalDate localDate = LocalDate.now().minusDays(1);//减一天
        Date nowdate = DateTimeUtil.localDateToDate(localDate);

        for (LangChaoUserInfoResultModelData d : data) {
            Date creatime = DateUtil.parseDateTime(d.getKaihutime());
            if (nowdate.getTime() > creatime.getTime()) {
                if ((long) listCustomer.size() != 0) {
                    this.saveBatch(listCustomer);
                }
                return false;
            }
            Customers customers = new Customers();
            customers.setOrganizationCode(d.getAreaid());
            customers.setOrganizationName(d.getAreaname());
            customers.setBookId(0L);
            customers.setBookName(d.getSection());
            customers.setCustomerCode(d.getUserid());
            customers.setCustomerName(d.getUsername());
            customers.setAddress(d.getAddress());
            customers.setLongitudeLatitude(d.getPosition());
            customers.setStampNo(d.getWatertabid());
            customers.setTypeName(d.getMetertype());
            customers.setTelphone(d.getTel());
            customers.setTelphone1(d.getTel1());
            customers.setNaturename1(d.getWatertype1());
            customers.setNaturename2(d.getWatertype2());
            customers.setNaturename3(d.getWatertype3());
            customers.setNaturename4(d.getWatertype4());
            customers.setCaliber(Long.valueOf("null".equals(d.getCaliber()) ? "0" : d.getCaliber()));
            customers.setEnabledTime(creatime);
            customers.setStatus(d.getFguserstartid());
            listCustomer.add(customers);
        }
        saveBatchWithTransaction(listCustomer);
        return true;
    }

    /**
     * get请求参数转换
     *
     * @param map map集合参数
     * @return 字符参数
     */
    public String transformationParams(Map<String, Object> map) {
        log.info("正在组装数据----------------------------------");
        StringBuffer stringBuffer = new StringBuffer();
        log.info("正在组装数据------------------token----------------");
        if (ObjectUtils.isEmpty(map)) {
            return stringBuffer.toString();
        }
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            if (entry.getValue() != null && entry.getValue() != "") {
                stringBuffer
                        .append(entry.getKey())
                        .append("=")
                        .append(entry.getValue()).append("&");
            }
        }
        log.info("正在组装数据------------------success------ data：{}----------", stringBuffer.toString());
        return stringBuffer.toString();
    }

    /**
     * 开独立事务批量保存
     *
     * @param entityList
     */
    @Transactional(propagation = Propagation.REQUIRES_NEW)
    public void saveBatchWithTransaction(List<Customers> entityList) {
        this.saveBatch(entityList);
    }
}
