package com.tofly.scada.service.impl;

import cn.hutool.core.util.URLUtil;
import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.sun.jndi.toolkit.url.UrlUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.scada.config.PlatConfig;
import com.tofly.scada.entity.DeviceArchiveManage;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.entity.ScadaMonitor;
import com.tofly.scada.entity.vo.third.*;
import com.tofly.scada.entity.vo.third.DeviceVo.DeviceDetailVo;
import com.tofly.scada.service.DeviceArchiveManageService;
import com.tofly.scada.service.ScadaMonitorService;
import com.tofly.scada.service.ScadaService;
import com.tofly.scada.service.ThirdDataPullService;
import com.tofly.scada.service.thirdDevice.Impl.FlowResolve;
import com.tofly.scada.service.thirdDevice.ThirdCommonDataResolve;
import com.tofly.scada.util.HttpClientUtil;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.security.web.util.UrlUtils;
import org.springframework.stereotype.Service;
import springfox.documentation.spring.web.json.Json;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ThirdDataPullServiceImpl implements ThirdDataPullService {
    @Autowired
    private PlatConfig platConfig;
    @Autowired
    private FlowResolve flowResolve;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private DeviceArchiveManageService deviceArchiveManageService;
    private static final String  THIRDTOKENWLW="THIRD:DATA:TOKEN";
    private static String accessToken;
    @Autowired
    private ScadaService scadaService;

    @Autowired
    private ScadaMonitorService scadaMonitorService;

    @Override
    public void pullDevice() {
         accessToken = loginGetToken();
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization","Bearer "+accessToken);
        headers.put("Content-Type","application/json");
        try {
                String requestJson = "{\"pageSize\": 999}";
                final String s = HttpClientUtil.doPostJson(platConfig.getIp() + platConfig.getDeviceListUrl(), headers,requestJson);

                 ThirdDataResponse<ThirdPage<ThirdDevice>> thirdDataResponse = JSON.parseObject(s,new TypeReference <ThirdDataResponse<ThirdPage<ThirdDevice>>>(){});


                final ThirdPage<ThirdDevice> result = thirdDataResponse.getResult();
                final List<ThirdDevice> data = result.getData();
                List<DeviceArchiveManage> deviceArchiveManageList = new ArrayList<>();
                //获取当前系统所有设备

            final List<DeviceArchiveManage> list = deviceArchiveManageService.list();
            //获取当前所有设备id
            final Set<String> collect
                    = list.stream().map(DeviceArchiveManage::getCode).collect(Collectors.toSet());


            for (ThirdDevice device : data) {
                    //DeviceArchiveManage
                    //查询设备
                     DeviceArchiveManage deviceArchiveManage = deviceArchiveManageService.
                            getOne(Wrappers.<DeviceArchiveManage>lambdaQuery()
                                    .eq(DeviceArchiveManage::getCode, device.getId()));
                    if(deviceArchiveManage!=null){

                    }else{


                        deviceArchiveManage = new DeviceArchiveManage();
                        deviceArchiveManage.setCode(device.getId());
                        deviceArchiveManage.setDeviceType(ThirdDeviceTypeEnum.getDeviceTypeByProductId(device.getProductId()));
                        deviceArchiveManage.setName(device.getName());

                    }
                    //查询设备经纬度等信息
                    final List<DeviceDetailVo> deviceDetailVos = pullDeviceInfo(device.getId());
                    fillDeviceByRemoteInfo(deviceArchiveManage,deviceDetailVos);

                    //更新在线状态
                    if("online".equals(device.getState().getValue())){
                        deviceArchiveManage.setStatus("1");

                    }
                     deviceArchiveManage.setName(device.getName());
                        final String deviceTypeByProductId =
                                ThirdDeviceTypeEnum.getDeviceTypeByProductId(device.getProductId());
                        if(deviceTypeByProductId==null){

                            log.error("未找到设备类型"+device.getProductId());
                            continue;
                        }else{
                            deviceArchiveManage.setDeviceType(deviceTypeByProductId);
                        }


                    deviceArchiveManageList.add(deviceArchiveManage);
                    //从set中移除
                     collect.remove(device.getId());
                }
                //遍历collect 如果存在数据  则表示数据缺失 标记为  0故障
                    for (String s1 : collect) {
                        DeviceArchiveManage notGetDevice = deviceArchiveManageService.
                                getOne(Wrappers.<DeviceArchiveManage>lambdaQuery()
                                        .eq(DeviceArchiveManage::getCode, s1));
                        notGetDevice.setStatus("0");
                        deviceArchiveManageList.add(notGetDevice);

                    }

                deviceArchiveManageService.saveOrUpdateBatch(deviceArchiveManageList);
                data.forEach(d->{
                    try {
                        pullDeviceData(d,20);
                    }catch (Exception e){
                        log.error(e.getMessage(),e);
                    }

                });


        } catch (Exception e) {
            log.error(e.getMessage(),e);
        }


    }

    /**
     * 通过从第三方获取的设备信息  填充本地设备信息
     * @param deviceArchiveManage
     * @param deviceDetailVos  第三方提供的设备信息
     */
    private void fillDeviceByRemoteInfo(DeviceArchiveManage deviceArchiveManage, List<DeviceDetailVo> deviceDetailVos) {

        for (DeviceDetailVo deviceDetailVo : deviceDetailVos) {
            if("geoPoint".equals(deviceDetailVo.getKey())){
                final String[] split = deviceDetailVo.getValue().split(",");
                try {
                    deviceArchiveManage.setLongitude(new BigDecimal(split[0]));
                    deviceArchiveManage.setLatitude(new BigDecimal(split[1]));
                }catch (Exception e){
                    log.error(JSON.toJSONString(deviceDetailVo),e);
                }


            }

            if("districtTown".equals(deviceDetailVo.getKey())){
                deviceArchiveManage.setAddress(deviceDetailVo.getValue());
            }
        }

    }

    @Override
    public void pullDeviceData(ThirdDevice device, Integer dataNum) {

        final String accessToken = loginGetToken();

        StringBuffer requestUrl = new StringBuffer(platConfig.getIp());
        requestUrl.append(platConfig.getLogUrl());

         String fillUrl
                = requestUrl.toString().replace("{deviceId}", device.getId());
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization","Bearer "+accessToken);
        headers.put("Content-Type","application/json");
         //拼接其它的条件参数
        Map<String,Object> paraMap = new HashMap<>();
        paraMap.put("pageSize",dataNum);
        paraMap.put("pageIndex",0);

        paraMap.put("sorts[0].name","timestamp");
        paraMap.put("sorts[0].order","desc");

         String s=null;
        try {
            s = HttpClientUtil.doGet(fillUrl, headers, paraMap, null);
        } catch (Exception e) {
            e.printStackTrace();
        }

        final String productId = device.getProductId();

            final String serviceNameByProductId = ThirdDeviceTypeEnum.getServiceNameByProductId(productId);
            if(StringUtils.isBlank(serviceNameByProductId)){
                log.error("未找到对应的service:"+productId);
                return;
            }

            ThirdCommonDataResolve thirdCommonDataResolve = (ThirdCommonDataResolve) SpringUtil.getBean(serviceNameByProductId);
            thirdCommonDataResolve.dataResolve(s);



    }

    @Override
    public void pullWaterData() {
            //1 。拉取水厂数据
        Map<String, String> headers = new HashMap<>();
        headers.put("Appkey","oievemzt7bowa9v9sf2iczmg");
        headers.put("AppSecret","9rbbnwcs3idyyzfpvs2xv9aqr2g28hd7");

        try {
            final String resultJson  =
                    HttpClientUtil.doGet("http://172.16.0.16:30123/api/sa/api/agent/getWaterWorkRealTime/1.0", headers, null, null);
            final JSONObject jsonObject = JSON.parseObject(resultJson);
           JSONObject result = jsonObject.getJSONObject("result");
            final Map<String, JSONArray> data = result.getObject("data", Map.class);
            // key 为水厂  waterworkd2 这种
            data.keySet().forEach(key->{

                final JSONArray jsonElements = data.get(key);
                for (Object jsonElement : jsonElements) {
                    final Map<String,JSONArray> jsonElement1 = (Map) jsonElement;
                    //key为设备名称  2sc_1gsbf 这种
                    for (String eleKey : jsonElement1.keySet()) {
                        final JSONArray o = jsonElement1.get(eleKey);
                        List<Scada> batchList = new ArrayList<>();
                        List<ScadaMonitor> monitors = new ArrayList<>();
                        for (Object o1 : o) {
                            final JSONObject o11 = (JSONObject) o1;
                            final ThirdWaterDetailVo thirdWaterDetailVo
                                    = o11.toJavaObject(ThirdWaterDetailVo.class);
                           // System.out.println(thirdWaterDetailVo);
                            final String deviceId = thirdWaterDetailVo.getDeviceId();
                            final String itemId = thirdWaterDetailVo.getItemId();
                            StringBuffer combineCode = new StringBuffer(deviceId).append(".").append(itemId);

                            String redisKey = combineCode.toString()+thirdWaterDetailVo.getTm();
                            //判断该数据是否拉取过
                            if(redisTemplate.opsForHash().hasKey(key,redisKey)){
                                continue;
                            }else{
                                redisTemplate.opsForHash().put(key,redisKey,"1");
                            }
                            Scada scada = new Scada();
                            scada.setScadaTime(thirdWaterDetailVo.getDate());
                            scada.setValue(thirdWaterDetailVo.getItemValue());
                            scada.setCode(combineCode.toString());
                            batchList.add(scada);
                             ScadaMonitor one = scadaMonitorService.getOne(Wrappers.<ScadaMonitor>
                                    lambdaQuery().eq(ScadaMonitor::getCode, combineCode.toString()));
                         if(one==null){
                             one = new ScadaMonitor();
                             one.setCode(combineCode.toString());
                         }
                            one.setScadaTime(thirdWaterDetailVo.getDate());
                            one.setValue(thirdWaterDetailVo.getItemValue());
                            monitors.add(one);

                        }

                        scadaService.saveBatch(batchList);
                        scadaMonitorService.saveOrUpdateBatch(monitors);
                    }
                }
            });

           // final ThirdWaterResult thirdWaterResult = JSON.parseObject(result, ThirdWaterResult.class);
           /* ThirdWaterVo jsonObject1 = JSON.parseObject(data,ThirdWaterVo.class);
            System.out.println(data);*/
            System.out.println("data");
        } catch (Exception e) {
            e.printStackTrace();
        }


        //2 拆分水厂数据
    }

    /**
     * 登录获取accessToken
     * @return
     */
    private synchronized String loginGetToken(){

        if(redisTemplate.opsForValue().get(THIRDTOKENWLW)!=null){
            String access_token =  redisTemplate.opsForValue().get(THIRDTOKENWLW);
            return  access_token;
        }else{
            log.info("未从redis获取到token，重新登录");
        }

        Map map = new HashMap<String ,Object>();
        map.put("client_id",platConfig.getClientId());
        map.put("client_secret",platConfig.getClientSecret());
        map.put("grant_type","client_credentials");

        try {
            final String s = HttpClientUtil.doGet(platConfig.getIp()+platConfig.getLoginUrl(), null, map, null);

            log.info("登录地址{}通过参数{}返回的数据{}",platConfig.getIp()+platConfig.getLoginUrl(),map,s);



            final JSONObject loginObject = JSON.parseObject(s);
            String access_token = loginObject.getString("access_token");
            final Integer expires_in = loginObject.getInteger("expires_in");
            redisTemplate.opsForValue().set(THIRDTOKENWLW,access_token,
                    expires_in/60 , TimeUnit.MINUTES);

            return access_token;
        } catch (Exception e) {
            log.error("登录地址{}通过参数{} 数据{}",platConfig.getIp()+platConfig.getLoginUrl(),map,e);
            e.printStackTrace();
        }
      throw new ToflyDeniedException("未获取到token");
    }

    protected List<DeviceDetailVo> pullDeviceInfo(String deviceId){
        accessToken = loginGetToken();
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization","Bearer "+accessToken);
        headers.put("Content-Type","application/json");

        StringBuffer requestUrl = new StringBuffer(platConfig.getIp());
        requestUrl.append(platConfig.getDeviceInfo());

        String fillUrl
                = requestUrl.toString().replace("{deviceId}", deviceId);

        try {
            final String s = HttpClientUtil.doGet(fillUrl, headers, null, null);

            final ThirdDataResponse<List<DeviceDetailVo>> deviceDetailVoThirdDataResponse
                    = JSON.parseObject(s, new TypeReference<ThirdDataResponse<List<DeviceDetailVo>>>() {
            });

              if(deviceDetailVoThirdDataResponse!=null){
                  final List<DeviceDetailVo> result

                          = deviceDetailVoThirdDataResponse.getResult();


                    if(result!=null){
                        return  result;
                    }
              }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return Collections.emptyList();
    }



    @XxlJob("pullDeviceAndData")
    public ReturnT pullDeviceAndData(){
        this.pullDevice();
        return   ReturnT.SUCCESS;
    }


    @XxlJob("pullWaterData")
    public ReturnT pullWaterLogData(){
        this.pullWaterData();
        return   ReturnT.SUCCESS;
    }
}
