package com.tofly.scada.service.thirdDevice.Impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.entity.ScadaMonitor;
import com.tofly.scada.entity.vo.third.DeviceVo.ThirdFlowDataVo;
import com.tofly.scada.entity.vo.third.DeviceVo.ThirdPressureDataVo;
import com.tofly.scada.entity.vo.third.ThirdContentVo;
import com.tofly.scada.entity.vo.third.ThirdDataResponse;
import com.tofly.scada.entity.vo.third.ThirdDeviceMessage;
import com.tofly.scada.entity.vo.third.ThirdPage;
import com.tofly.scada.service.ScadaMonitorService;
import com.tofly.scada.service.ScadaService;
import com.tofly.scada.service.thirdDevice.ThirdCommonDataResolve;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import springfox.documentation.spring.web.json.Json;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 流量
 */
@Service("flowResolve")
@Slf4j
public class FlowResolve extends ThirdCommonDataResolveImpl<ThirdFlowDataVo> {
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private ScadaService scadaService;


    @Override
    public void dataResolve(String message) {
         ThirdDataResponse<ThirdPage<ThirdDeviceMessage>> thirdPageThirdDataResponse=null;
        try {
             thirdPageThirdDataResponse
                    = messageToEntity(message);
        }catch (Exception e){
            log.error(message,e);
        }

        final ThirdPage<ThirdDeviceMessage> result
                = thirdPageThirdDataResponse.getResult();
        final List<ThirdDeviceMessage> data = result.getData();
        List<Scada> scadas = new ArrayList<>();
        int index = 0;
        for (ThirdDeviceMessage datum : data) {
            final ThirdFlowDataVo thirdFlowDataVo = dataParse(datum.getContent());
            if(thirdFlowDataVo!=null){

                 //如果数据已存在
                if(! dataFlagPut(datum.getMessageId())){
                    continue;
                }
                if(thirdFlowDataVo.getAwf()==null ||thirdFlowDataVo.getWf()==null){
                    continue;
                }
                final DateTime date = DateUtil.date(Long.valueOf(thirdFlowDataVo.getTm()));
                Scada scada = new Scada();
                scada.setCode(datum.getDeviceId()+".awf");
                scada.setValue(String.valueOf(thirdFlowDataVo.getAwf()));
                scada.setScadaTime(date);
                scada.setUpdateTime(new Date());
                scadas.add(scada);
                Scada scada2 = new Scada();
                scada2.setCode(datum.getDeviceId()+".wf");
                scada2.setValue(String.valueOf(thirdFlowDataVo.getWf()));
                scada2.setScadaTime(date);
                scadas.add(scada2);
                //存储最新值  因为接口为时间倒序，所以第一条为该设备的最新一条数据
                if(index==0){
                    updateScadaMointor(scada,datum.getMessageId());
                    updateScadaMointor(scada2,datum.getMessageId());
                    index =2;
                }

            }
        }
        scadaService.saveBatch(scadas);

    }

    @Override
    public ThirdFlowDataVo dataParse(String content) {
        log.info("content:"+content);
        try {
            final ThirdContentVo<ThirdFlowDataVo> thirdFlowDataVoThirdContentVo
                    = JSON.parseObject(content, new TypeReference<ThirdContentVo<ThirdFlowDataVo>>() {
            });
            final ThirdFlowDataVo properties = thirdFlowDataVoThirdContentVo.getProperties();
            if(properties ==null ||(properties.getType()!=null && "F2".equals(properties.getType()))){
                //丢掉心跳数据
                return  null;
            }
            return properties;
        }catch (Exception e ){
            log.error(content,e);
        }
        return  null;

    }



}
