package com.tofly.spire.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.scada.common.BaseController;
import com.tofly.spire.entity.SpireGate;
import com.tofly.spire.entity.dto.SpireGateDto;
import com.tofly.spire.service.SpireGateService;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

/**
 * 小表（户表）阀门控制(SpireGate)表控制层
 *
 * @author HaiQiu
 * @since 2022-11-11 13:40:45
 */
@Api(tags = "小表（户表）阀门控制")
@RestController
@RequestMapping("spireGate")
public class SpireGateController extends BaseController {
    /**
     * 服务对象
     */
    @Resource
    private SpireGateService spireGateService;


    /**
     * 小表（户表）阀门控制-通过此接口控制表具阀门的开关(需要表端支持阀门控制)
     * @param sn 表号
     * @param flag true=开阀 false=关阀
     * @return 调用结果
     */
    @GetMapping("/valveControl")
    @ApiOperation(value = "阀门控制")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sn",required = true,value = "表号"),
            @ApiImplicitParam(name = "flag",required = true,value = "true=开阀 false=关阀")
    })
    public ResultRespone valveControl(String sn, boolean flag){
        return spireGateService.valveControl(sn,flag);
    }

    /**
     * 分页查询数据
     *
     * @param page      分页对象
     * @param spireGate 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "分页查询数据")
    @GetMapping("/page")
    public ResultRespone pageList(Page page, SpireGate spireGate) {
        return decide(spireGateService.pageList(page, spireGate));
    }

    /**
     * 查询所有数据
     *
     * @param spireGate 查询实体
     * @return 所有数据
     */
    @ApiOperation(value = "查询所有数据")
    @GetMapping("/list")
    public ResultRespone allList(SpireGate spireGate) {
        return decide(spireGateService.allList(spireGate));
    }

    /**
     * 通过主键查询单条数据
     *
     * @param id 主键
     * @return 单条数据
     */
    @ApiOperation(value = "通过主键查询单条数据")
    @ApiImplicitParam(name = "id", value = "数据ID")
    @GetMapping("{id}")
    public ResultRespone get(@PathVariable Serializable id) {
        return decide(spireGateService.get(id));
    }

    /**
     * 新增数据
     *
     * @param spireGateDto 实体
     * @return 新增结果
     */
    @ApiOperation(value = "新增数据")
    @PostMapping
    public ResultRespone insert(@Validated @RequestBody SpireGateDto spireGateDto) {
        spireGateService.insert(spireGateDto);
        return decide("新增成功");
    }

    /**
     * 修改数据
     *
     * @param spireGateDto 实体
     * @return 修改结果
     */
    @ApiOperation(value = "修改数据")
    @PutMapping
    public ResultRespone edit(@Validated @RequestBody SpireGateDto spireGateDto) {
        spireGateService.edit(spireGateDto);
        return decide("修改成功");
    }

    /**
     * 删除数据
     *
     * @param idList 主键结合
     * @return 删除结果
     */
    @ApiOperation(value = "删除数据")
    @ApiImplicitParam(name = "idList", value = "数据ID数组集合")
    @DeleteMapping
    public ResultRespone delete(@RequestParam("idList") List<Long> idList) {
        spireGateService.delete(idList);
        return decide("删除成功");
    }

    /**
     * 数据导入Excel模板下载
     */
    @ApiOperation(value = "模版下载")
    @GetMapping("/template")
    public void template() {
        spireGateService.template();
    }

    /**
     * Excel数据导入
     *
     * @param file 导入的Excel数据文件
     * @return 导入结果
     */
    @ApiOperation(value = "数据导入")
    @ApiImplicitParam(name = "file", value = "Excel文件", required = true)
    @PostMapping("/import")
    public ResultRespone importData(@RequestParam(name = "file") MultipartFile file) {
        spireGateService.importData(file);
        return decide("导入成功");
    }


    /**
     * 数据导出
     *
     * @param page      分页
     * @param spireGate 查询模型
     * @param isAll     是否导出全部数据，true是，false否
     */
    @ApiOperation(value = "数据导出")
    @ApiImplicitParams(value = {
            @ApiImplicitParam(name = "isAll", value = "是否导出全部数据，true是，false不是"),
    })
    @GetMapping("/export")
    public void export(Page page, SpireGate spireGate, Boolean isAll) {
        spireGateService.export(page, spireGate, isAll);
    }
}

