package com.tofly.spire.service.impl;


import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.scada.entity.Allocation;
import com.tofly.scada.entity.Scada;
import com.tofly.scada.service.ScadaService;
import com.tofly.spire.common.Constants;
import com.tofly.spire.entity.IndexAllocation;
import com.tofly.spire.entity.TfDeviceArchiveManage;
import com.tofly.spire.entity.SpireBigGrid;
import com.tofly.spire.entity.dto.SpireBigGridDto;
import com.tofly.spire.mapper.SpireBigGridMapper;
import com.tofly.spire.service.DeviceArchiveManageService;
import com.tofly.spire.service.IndexAllocationService;
import com.tofly.spire.service.SpireBigGridService;
import com.tofly.spire.utils.ResultScada;
import com.tofly.spire.utils.SpireUtils;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.*;

/**
 * 大表（管网）(SpireBigGrid)表服务实现类
 *
 * @author HaiQiu
 * @since 2022-11-11 09:16:07
 */
@Slf4j
@Service("spireBigGridService")
public class SpireBigGridServiceImpl extends ServiceImpl<SpireBigGridMapper, SpireBigGrid> implements SpireBigGridService {

    @Autowired
    private SpireUtils spireUtils;

    @Autowired
    private DeviceArchiveManageService deviceArchiveManageService;

    @Autowired
    private IndexAllocationService allocationService;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private ScadaService scadaService;


    /**
     * 定时获取-此接口用户获取主网表具数据信息以及日志信息。
     *
     * @param params
     * @return
     */
    @XxlJob("getBulkMeterData")
//    @Scheduled(cron = "0 0/1 * * * ?")
    public  ReturnT getBulkMeterData(String params) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.DAY_OF_MONTH, -1);
        String format = DateUtil.format(calendar.getTime(), "yyyy-MM-dd");
        ResultScada bulkMeterData = spireUtils.getBulkMeterData(null, format);
        List<SpireBigGrid> data = JSONObject.parseArray(bulkMeterData.getData().toString(), SpireBigGrid.class);
        log.info("获取到接口数据：{}条",data.size());
        //临时存储新数据集合
        List<SpireBigGrid> spireBigGrids = new ArrayList<>();
        for (SpireBigGrid datum : data) {
            if (count(Wrappers.lambdaQuery(datum)) == 0) {
                spireBigGrids.add(datum);
            }
        }
        //批量新增
        if (!CollectionUtils.isEmpty(spireBigGrids)) {
            log.info("正在同步大表管网数据进行批量添加，数据条数：{}",spireBigGrids.size());
            saveBatch(spireBigGrids);
            //redisTemplate.opsForValue().set(Constants.SPIRE_MT_BIG_KEY,JSONObject.toJSONString(spireBigGrids));
            log.info("正在添加指标设备和历史数据进行批量添加，数据条数：{}",spireBigGrids.size());
            saveDataScada(spireBigGrids);

        }
        return ReturnT.SUCCESS;
    }


    /***
     * 定时处理缓存中的数据集合
     * @param spireBigGrids 数据集合
     */
    @Async
    public void saveDataScada(List<SpireBigGrid> spireBigGrids) {
        for (SpireBigGrid spireBigGrid : spireBigGrids) {
            //新增设备
            int count = deviceArchiveManageService.count(Wrappers.<TfDeviceArchiveManage>lambdaQuery()
                    .eq(TfDeviceArchiveManage::getCode, spireBigGrid.getSn()));
            if (count == 0) {
                log.info("正在添加设备添加");
                TfDeviceArchiveManage manage = new TfDeviceArchiveManage();
                manage.setName(spireBigGrid.getSn() + spireBigGrid.getMeterName());
                manage.setStatus("1");
                manage.setCode(spireBigGrid.getSn());
                deviceArchiveManageService.save(manage);
                //新增指标
                log.info("正在添加新增指标");
                saveAllocations(spireBigGrid, manage);
            } else {
                //新增指标
                List<TfDeviceArchiveManage> list = deviceArchiveManageService.list(Wrappers.<TfDeviceArchiveManage>lambdaQuery()
                        .eq(TfDeviceArchiveManage::getCode, spireBigGrid.getSn()));
                if (!CollectionUtils.isEmpty(list)) {
                    log.info("正在添加新增指标");
                    saveAllocations(spireBigGrid, list.get(0));
                }
            }
            //新增历史表数据
            log.info("正在添加新增历史表数据");
            saveScadas(spireBigGrid);
        }
    }


    /**
     * 自动新增指标
     *
     * @param spireBigGrid
     * @param manage
     */
    private void saveAllocations(SpireBigGrid spireBigGrid, TfDeviceArchiveManage manage) {
        //新增指标累计量(m³)
        if (allocationService.count(Wrappers.<IndexAllocation>lambdaQuery()
                .eq(IndexAllocation::getVariableCode, spireBigGrid.getSn() + spireBigGrid.getMeterStatus() + "." + Constants.accFlow)) == 0) {
            IndexAllocation allocation = new IndexAllocation();
            allocation.setDeviceId(manage.getId());
            allocation.setCreateTime(new Date());
            allocation.setDisplayName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "#累计量#" + Constants.accFlow);
            allocation.setVariableName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.accFlow);
            allocation.setVariableCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.accFlow);
            allocationService.save(allocation);
        }
        //新增指标反向累积量 (m³)
        if (allocationService.count(Wrappers.<IndexAllocation>lambdaQuery()
                .eq(IndexAllocation::getVariableCode, spireBigGrid.getSn() + spireBigGrid.getMeterStatus() + "." + Constants.decAccFlow)) == 0) {
            IndexAllocation allocation = new IndexAllocation();
            allocation.setDeviceId(manage.getId());
            allocation.setCreateTime(new Date());
            allocation.setDisplayName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "#反向累积量#" + Constants.decAccFlow);
            allocation.setVariableName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.decAccFlow);
            allocation.setVariableCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.decAccFlow);
            allocationService.save(allocation);
        }
        //新增指标瞬时量 (m³/h)
        if (allocationService.count(Wrappers.<IndexAllocation>lambdaQuery()
                .eq(IndexAllocation::getVariableCode, spireBigGrid.getSn() + spireBigGrid.getMeterStatus() + "." + Constants.flow)) == 0) {
            IndexAllocation allocation = new IndexAllocation();
            allocation.setDeviceId(manage.getId());
            allocation.setCreateTime(new Date());
            allocation.setDisplayName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "#瞬时量#" + Constants.flow);
            allocation.setVariableName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.flow);
            allocation.setVariableCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.flow);
            allocationService.save(allocation);
        }
        //新增指标进水温度 (℃)
        if (allocationService.count(Wrappers.<IndexAllocation>lambdaQuery()
                .eq(IndexAllocation::getVariableCode, spireBigGrid.getSn() + spireBigGrid.getMeterStatus() + "." + Constants.inTemp)) == 0) {
            IndexAllocation allocation = new IndexAllocation();
            allocation.setDeviceId(manage.getId());
            allocation.setCreateTime(new Date());
            allocation.setDisplayName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "#进水温度#" + Constants.inTemp);
            allocation.setVariableName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.inTemp);
            allocation.setVariableCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.inTemp);
            allocationService.save(allocation);
        }
        //新增压力值 (kPa
        if (allocationService.count(Wrappers.<IndexAllocation>lambdaQuery()
                .eq(IndexAllocation::getVariableCode, spireBigGrid.getSn() + spireBigGrid.getMeterStatus() + "." + Constants.pressure)) == 0) {
            IndexAllocation allocation = new IndexAllocation();
            allocation.setDeviceId(manage.getId());
            allocation.setCreateTime(new Date());
            allocation.setDisplayName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "#压力值#" + Constants.pressure);
            allocation.setVariableName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.pressure);
            allocation.setVariableCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.pressure);
            allocationService.save(allocation);
        }
        //新增指标出水温度 (℃)
        if (allocationService.count(Wrappers.<IndexAllocation>lambdaQuery()
                .eq(IndexAllocation::getVariableCode, spireBigGrid.getSn() + spireBigGrid.getMeterStatus() + "." + Constants.temp)) == 0) {
            IndexAllocation allocation = new IndexAllocation();
            allocation.setDeviceId(manage.getId());
            allocation.setCreateTime(new Date());
            allocation.setDisplayName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "#出水温度#" + Constants.temp);
            allocation.setVariableName(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.temp);
            allocation.setVariableCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.temp);
            allocationService.save(allocation);
        }
        log.info("正在添加新增指标-------------------->完成");
    }


    /**
     * 新增scada历史表数据
     *
     * @param spireBigGrid 原始表数据
     */
    private void saveScadas(SpireBigGrid spireBigGrid) {
        List<Scada> scadas = new ArrayList<>();
        //新增指标累计量(m³)
        if (StringUtils.hasText(spireBigGrid.getAccFlow())) {
            Scada scada = new Scada();
            scada.setCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.accFlow);
            scada.setScadaTime(spireBigGrid.getReadTime());
            scada.setUpdateTime(new Date());
            scada.setValue(spireBigGrid.getAccFlow());
            scadas.add(scada);
        }
        //新增指标反向累积量 (m³)
        if (StringUtils.hasText(spireBigGrid.getDecAccFlow())) {
            Scada scada = new Scada();
            scada.setCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.decAccFlow);
            scada.setScadaTime(spireBigGrid.getReadTime());
            scada.setUpdateTime(new Date());
            scada.setValue(spireBigGrid.getDecAccFlow());
            scadas.add(scada);
        }
        //新增指标瞬时量 (m³/h)
        if (StringUtils.hasText(spireBigGrid.getFlow())) {
            Scada scada = new Scada();
            scada.setCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.flow);
            scada.setScadaTime(spireBigGrid.getReadTime());
            scada.setUpdateTime(new Date());
            scada.setValue(spireBigGrid.getFlow());
            scadas.add(scada);
        }
        //新增指标进水温度 (℃)
        if (StringUtils.hasText(spireBigGrid.getInTemp())) {
            Scada scada = new Scada();
            scada.setCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.inTemp);
            scada.setScadaTime(spireBigGrid.getReadTime());
            scada.setUpdateTime(new Date());
            scada.setValue(spireBigGrid.getInTemp());
            scadas.add(scada);
        }
        //新增压力值 (kPa
        if (StringUtils.hasText(spireBigGrid.getPressure())) {
            Scada scada = new Scada();
            scada.setCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.pressure);
            scada.setScadaTime(spireBigGrid.getReadTime());
            scada.setUpdateTime(new Date());
            scada.setValue(spireBigGrid.getPressure());
            scadas.add(scada);
        }
        //新增指标出水温度 (℃)
        if (StringUtils.hasText(spireBigGrid.getTemp())) {
            Scada scada = new Scada();
            scada.setCode(spireBigGrid.getSn() + spireBigGrid.getMeterName() + spireBigGrid.getMeterStatus() + "." + Constants.temp);
            scada.setScadaTime(spireBigGrid.getReadTime());
            scada.setUpdateTime(new Date());
            scada.setValue(spireBigGrid.getTemp());
            scadas.add(scada);
        }
        if (!CollectionUtils.isEmpty(scadas)) {
            log.info("新增scada历史数据：{}", scadas);
            scadaService.saveBatch(scadas);
//            for (Scada scada : scadas) {
//                scadaService.save(scada);
//            }
        }
    }

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public SpireBigGrid get(Serializable id) {
        return getById(id);
    }

    /**
     * 新增
     *
     * @param spireBigGridDto 实体
     */
    @Override
    public void insert(SpireBigGridDto spireBigGridDto) {
        SpireBigGrid spireBigGrid = convert(spireBigGridDto);
        if (!save(spireBigGrid)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param spireBigGridDto 实体
     */
    @Override
    public void edit(SpireBigGridDto spireBigGridDto) {
        SpireBigGrid spireBigGrid = convert(spireBigGridDto);
        if (!updateById(spireBigGrid)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param spireBigGridDto dto实体
     */
    private SpireBigGrid convert(SpireBigGridDto spireBigGridDto) {
        SpireBigGrid spireBigGrid = new SpireBigGrid();
        BeanUtils.copyProperties(spireBigGridDto, spireBigGrid);
        return spireBigGrid;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        SpireBigGrid spireBigGrid = new SpireBigGrid();
        ExcelUtils.createTemplate("大表（管网）", "SpireBigGrid大表（管网）",
                "SpireBigGrid大表（管网）", SpireBigGrid.class, Collections.singletonList(spireBigGrid));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<SpireBigGrid> datas = ExcelUtils.getExcelToList("大表（管网）", file, SpireBigGrid.class);
        if (!CollectionUtils.isEmpty(datas)) {
            saveBatch(datas);
        }
    }

    /**
     * 分页查询
     *
     * @param page         分页
     * @param spireBigGrid 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<SpireBigGrid> pageList(Page page, SpireBigGrid spireBigGrid) {
        return page(page, new QueryWrapper<>(spireBigGrid)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page         分页
     * @param spireBigGrid 查询模型
     * @param isAll        是否导出全部数据，true是，false否
     */
    @Override
    public void export(Page page, SpireBigGrid spireBigGrid, Boolean isAll) {
        Page<SpireBigGrid> pageData = new Page<>();
        if (isAll == null || !isAll) {
            pageData = page(page, new QueryWrapper<>(spireBigGrid)).addOrder(page.getOrders());
        } else {
            pageData.setRecords(list(new QueryWrapper<>(spireBigGrid)));
        }
        //导出数据
        ExcelUtils.getListToExcel(SpireBigGrid.class, pageData.getRecords(), "大表（管网）");
    }

    /**
     * 查询所有数据
     *
     * @param spireBigGrid 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<SpireBigGrid> allList(SpireBigGrid spireBigGrid) {
        return list(new QueryWrapper<>(spireBigGrid));
    }
}

