package com.tofly.spire.service.impl;


import com.alibaba.fastjson.JSONObject;
import com.haiqiu.tools.excel.utils.ExcelUtils;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.spire.common.Constants;
import com.tofly.spire.utils.DateConvertUtil;
import com.tofly.spire.utils.ResultScada;
import com.tofly.spire.utils.SpireUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.spire.mapper.SpireGateMapper;
import com.tofly.spire.entity.SpireGate;
import com.tofly.spire.entity.dto.SpireGateDto;
import com.tofly.spire.service.SpireGateService;
import org.springframework.stereotype.Service;

import org.springframework.web.multipart.MultipartFile;

import java.util.Collections;
import java.util.List;
import java.io.Serializable;

/**
 * 小表（户表）阀门控制(SpireGate)表服务实现类
 *
 * @author HaiQiu
 * @since 2022-11-11 13:40:45
 */
@Service("spireGateService")
public class SpireGateServiceImpl extends ServiceImpl<SpireGateMapper, SpireGate> implements SpireGateService {


    @Autowired
    private SpireUtils spireUtils;


    /**
     * 小表（户表）阀门控制-通过此接口控制表具阀门的开关(需要表端支持阀门控制)
     * @param sn 表号
     * @param flag true=开阀 false=关阀
     * @return 调用结果
     */
    @Override
    public ResultRespone valveControl(String sn, boolean flag){
        ResultScada resultScada = spireUtils.valveControl(sn,flag);
        SpireGate spireGate = new SpireGate();
        spireGate.setSn(sn);
        spireGate.setFlag(flag ? "开阀" : "关阀");
        spireGate.setResult(JSONObject.toJSONString(resultScada));
        spireGate.setStatus(JSONObject.toJSONString(resultScada.getData()));
        spireGate.setUploadTime(new DateConvertUtil().getDateTime(Long.parseLong(resultScada.getTimestamp()),0));
        save(spireGate);
        if (Constants.HTTP_OK.equals(resultScada.getHttpCode())){
           return ResultRespone.success(resultScada,resultScada.getData().toString());
        }
        return ResultRespone.failed(resultScada,resultScada.getData().toString());
    }

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    @Override
    public SpireGate get(Serializable id) {
        return getById(id);
    }

    /**
     * 新增
     *
     * @param spireGateDto 实体
     */
    @Override
    public void insert(SpireGateDto spireGateDto) {
        SpireGate spireGate = convert(spireGateDto);
        if (!save(spireGate)) {
            throw new RuntimeException("新增失败");
        }
    }

    /**
     * 修改
     *
     * @param spireGateDto 实体
     */
    @Override
    public void edit(SpireGateDto spireGateDto) {
        SpireGate spireGate = convert(spireGateDto);
        if (!updateById(spireGate)) {
            throw new RuntimeException("修改失败");
        }
    }

    /**
     * 实体转换
     *
     * @param spireGateDto dto实体
     */
    private SpireGate convert(SpireGateDto spireGateDto) {
        SpireGate spireGate = new SpireGate();
        BeanUtils.copyProperties(spireGateDto, spireGate);
        return spireGate;
    }

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    @Override
    public void delete(List<Long> idList) {
        if (!removeByIds(idList)) {
            throw new RuntimeException("删除失败");
        }
    }

    /**
     * 生成模板下载
     */
    @Override
    public void template() {
        SpireGate spireGate = new SpireGate();
        ExcelUtils.createTemplate("小表（户表）阀门控制", "SpireGate小表（户表）阀门控制",
                "SpireGate小表（户表）阀门控制", SpireGate.class, Collections.singletonList(spireGate));
    }

    /**
     * 数据导入
     *
     * @param file 文件
     */
    @Override
    public void importData(MultipartFile file) {
        List<SpireGate> datas = ExcelUtils.getExcelToList("小表（户表）阀门控制", file, SpireGate.class);
        if (!CollectionUtils.isEmpty(datas)) {
            saveBatch(datas);
        }
    }

    /**
     * 分页查询
     *
     * @param page      分页
     * @param spireGate 查询wrapper
     * @return 数据集合
     */
    @Override
    public Page<SpireGate> pageList(Page page, SpireGate spireGate) {
        return page(page, new QueryWrapper<>(spireGate)).addOrder(page.getOrders());
    }

    /**
     * 数据导出
     *
     * @param page      分页
     * @param spireGate 查询模型
     * @param isAll     是否导出全部数据，true是，false否
     */
    @Override
    public void export(Page page, SpireGate spireGate, Boolean isAll) {
        Page<SpireGate> pageData = new Page<>();
        if (isAll == null || !isAll) {
            pageData = page(page, new QueryWrapper<>(spireGate)).addOrder(page.getOrders());
        } else {
            pageData.setRecords(list(new QueryWrapper<>(spireGate)));
        }
        //导出数据
        ExcelUtils.getListToExcel(SpireGate.class, pageData.getRecords(), "小表（户表）阀门控制");
    }

    /**
     * 查询所有数据
     *
     * @param spireGate 查询条件
     * @return 查询所有数据
     */
    @Override
    public List<SpireGate> allList(SpireGate spireGate) {
        return list(new QueryWrapper<>(spireGate));
    }
}

