package com.tofly.water.entity;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.tofly.common.constant.SqlCondition;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.annotate.JsonSerialize;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;
import com.haiqiu.tools.excel.annotation.Excel;

/**
 * 水泵基础信息(BasePumpDetail)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:02
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@TableName(value = "TF_BASE_PUMP_DETAIL", resultMap = "BasePumpDetailMap")
@ApiModel(value = "BasePumpDetail对象", description = "水泵基础信息")
public class BasePumpDetail extends Model<BasePumpDetail> {

    private static final long serialVersionUID = -18593666998094151L;

    /**
     * 主键ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "主键ID")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;
    /**
     * TF_BASE_PUMP表ID（所属泵站ID）
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "TF_BASE_PUMP表ID（所属泵站ID）")
    @ApiModelProperty(value = "TF_BASE_PUMP表ID（所属泵站ID）")
    private Long pumpId;
    @TableField(exist = false)
    @ApiModelProperty(value = "泵站名称")
    private String pumpIdName;
    /**
     * 水泵机组ID
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "水泵机组ID")
    @ApiModelProperty(value = "水泵机组ID")
    private Long pumpGroupId;
    @TableField(exist = false)
    @ApiModelProperty(value = "泵组名称")
    private String pumpGroupIdName;
    /**
     * 水泵名称
     */
    @Excel(name = "水泵名称")
    @ApiModelProperty(value = "水泵名称")
    @TableField(condition = SqlCondition.ORACLE_LIKE)
    private String name;
    /**
     * 型号
     */
    @Excel(name = "型号")
    @ApiModelProperty(value = "型号")
    private String model;
    /**
     * 额定流量（m³/h）
     */
    @Excel(name = "额定流量（m³/h）")
    @ApiModelProperty(value = "额定流量（m³/h）")
    private String ratedFlow;
    /**
     * 额定扬程（m）
     */
    @Excel(name = "额定扬程（m）")
    @ApiModelProperty(value = "额定扬程（m）")
    private String ratedHead;
    /**
     * 额定转速（r/min）
     */
    @Excel(name = "额定转速（r/min）")
    @ApiModelProperty(value = "额定转速（r/min）")
    private String ratedSpeed;
    /**
     * 额定电压（V）
     */
    @Excel(name = "额定电压（V）")
    @ApiModelProperty(value = "额定电压（V）")
    private String ratedVoltage;
    /**
     * 额定电流（A）
     */
    @Excel(name = "额定电流（A）")
    @ApiModelProperty(value = "额定电流（A）")
    private String ratedCurrent;
    /**
     * 额定功率（KW）
     */
    @Excel(name = "额定功率（KW）")
    @ApiModelProperty(value = "额定功率（KW）")
    private String ratedPower;
    /**
     * 额定频率（Hz）
     */
    @Excel(name = "额定频率（Hz）")
    @ApiModelProperty(value = "额定频率（Hz）")
    private String ratedFrequency;
    /**
     * 排序
     */
    @Excel(name = "排序")
    @ApiModelProperty(value = "排序")
    private Long sort;
    /**
     * 删除，0未删除，1已删除
     */
    @Excel(name = "删除，0未删除，1已删除")
    @TableLogic
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;
    /**
     * 创建人
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "创建人")
    @ApiModelProperty(value = "创建人")
    private Long createdUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "创建人", hidden = true)
    private String createdUserName;
    /**
     * 创建时间
     */
    @Excel(name = "创建时间")
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;
    /**
     * 更新人
     */
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    @Excel(name = "更新人")
    @ApiModelProperty(value = "更新人")
    private Long updatedUser;
    @TableField(exist = false)
    @ApiModelProperty(value = "更新人", hidden = true)
    private String updatedUserName;
    /**
     * 更新时间
     */
    @Excel(name = "更新时间")
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updatedTime;
}

