package com.tofly.water.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 水泵基础信息(BasePumpDetail)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:02
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "BasePumpDetail对象", description = "水泵基础信息")
public class BasePumpDetailDto implements Serializable {

    private static final long serialVersionUID = -54146558173028027L;

    /**
     * 主键ID
     */
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * TF_BASE_PUMP表ID（所属泵站ID）
     */
    @Digits(integer = 20, fraction = 0, message = "TF_BASE_PUMP表ID（所属泵站ID）长度不能超过20")
    @ApiModelProperty(value = "TF_BASE_PUMP表ID（所属泵站ID）")
    private Long pumpId;

    /**
     * 水泵机组ID
     */
    @Digits(integer = 20, fraction = 0, message = "水泵机组ID长度不能超过20")
    @ApiModelProperty(value = "水泵机组ID")
    private Long pumpGroupId;

    /**
     * 水泵名称
     */
    @Size(max = 30, message = "水泵名称长度不能超过30")
    @ApiModelProperty(value = "水泵名称")
    private String name;

    /**
     * 型号
     */
    @Size(max = 85, message = "型号长度不能超过85")
    @ApiModelProperty(value = "型号")
    private String model;

    /**
     * 额定流量（m³/h）
     */
    @Size(max = 85, message = "额定流量（m³/h）长度不能超过85")
    @ApiModelProperty(value = "额定流量（m³/h）")
    private String ratedFlow;

    /**
     * 额定扬程（m）
     */
    @Size(max = 85, message = "额定扬程（m）长度不能超过85")
    @ApiModelProperty(value = "额定扬程（m）")
    private String ratedHead;

    /**
     * 额定转速（r/min）
     */
    @Size(max = 85, message = "额定转速（r/min）长度不能超过85")
    @ApiModelProperty(value = "额定转速（r/min）")
    private String ratedSpeed;

    /**
     * 额定电压（V）
     */
    @Size(max = 85, message = "额定电压（V）长度不能超过85")
    @ApiModelProperty(value = "额定电压（V）")
    private String ratedVoltage;

    /**
     * 额定电流（A）
     */
    @Size(max = 85, message = "额定电流（A）长度不能超过85")
    @ApiModelProperty(value = "额定电流（A）")
    private String ratedCurrent;

    /**
     * 额定功率（KW）
     */
    @Size(max = 85, message = "额定功率（KW）长度不能超过85")
    @ApiModelProperty(value = "额定功率（KW）")
    private String ratedPower;

    /**
     * 额定频率（Hz）
     */
    @Size(max = 85, message = "额定频率（Hz）长度不能超过85")
    @ApiModelProperty(value = "额定频率（Hz）")
    private String ratedFrequency;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 删除，0未删除，1已删除
     */
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;

    /**
     * 创建人
     */
    @Digits(integer = 20, fraction = 0, message = "创建人长度不能超过20")
    @ApiModelProperty(value = "创建人")
    private Long createdUser;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;

    /**
     * 更新人
     */
    @Digits(integer = 20, fraction = 0, message = "更新人长度不能超过20")
    @ApiModelProperty(value = "更新人")
    private Long updatedUser;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updatedTime;

}

