package com.tofly.water.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.tofly.water.entity.BasePumpDetail;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.math.BigDecimal;
import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 水泵机组管理(BasePumpGroup)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:03
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "BasePumpGroup对象", description = "水泵机组管理")
public class BasePumpGroupDto implements Serializable {

    private static final long serialVersionUID = 543184044181502143L;

    /**
     * 主键ID
     */
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * TF_BASE_PUMP表ID（所属泵站ID）
     */
    @Digits(integer = 20, fraction = 0, message = "TF_BASE_PUMP表ID（所属泵站ID）长度不能超过20")
    @NotNull(message = "TF_BASE_PUMP表ID（所属泵站ID）不能为空")
    @ApiModelProperty(value = "TF_BASE_PUMP表ID（所属泵站ID）")
    private Long pumpId;

    /**
     * 机组名称
     */
    @Size(max = 30, message = "机组名称长度不能超过30")
    @NotBlank(message = "机组名称不能为空")
    @ApiModelProperty(value = "机组名称")
    private String name;

    /**
     * 机组编号
     */
    @Size(max = 85, message = "机组编号长度不能超过85")
    @NotBlank(message = "机组编号不能为空")
    @ApiModelProperty(value = "机组编号")
    private String code;

    /**
     * 分组
     */
    @Size(max = 85, message = "分组长度不能超过85")
    @ApiModelProperty(value = "分组")
    private String grouping;

    /**
     * 供应商简称
     */
    @Size(max = 30, message = "供应商简称长度不能超过30")
    @NotBlank(message = "供应商简称不能为空")
    @ApiModelProperty(value = "供应商简称")
    private String supplierSimple;

    /**
     * 供应商全称
     */
    @Size(max = 300, message = "供应商全称长度不能超过300")
    @ApiModelProperty(value = "供应商全称")
    private String supplierDetail;

    /**
     * 购买金额
     */
    @Digits(integer = 24, fraction = 2, message = "购买金额长度不能超过24")
    @NotNull(message = "购买金额不能为空")
    @ApiModelProperty(value = "购买金额")
    private BigDecimal purchaseAmount;

    /**
     * 购买日期
     */
    @ApiModelProperty(value = "购买日期")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date purchaseDate;

    /**
     * 投运日期
     */
    @ApiModelProperty(value = "投运日期")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date investmentDate;

    /**
     * 备注
     */
    @Size(max = 300, message = "备注长度不能超过300")
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 进水管径
     */
    @Size(max = 85, message = "进水管径长度不能超过85")
    @ApiModelProperty(value = "进水管径")
    private String inletPipeDiameter;

    /**
     * 出水管径
     */
    @Size(max = 85, message = "出水管径长度不能超过85")
    @ApiModelProperty(value = "出水管径")
    private String outletPipeDiameter;

    /**
     * 附件（照片）
     */
    @Size(max = 85, message = "附件（照片）长度不能超过85")
    @ApiModelProperty(value = "附件（照片）")
    private String annex;

    /**
     * 生产厂商简称
     */
    @Size(max = 30, message = "生产厂商简称长度不能超过30")
    @NotBlank(message = "生产厂商简称不能为空")
    @ApiModelProperty(value = "生产厂商简称")
    private String productionSimple;

    /**
     * 生产厂商全称
     */
    @Size(max = 300, message = "生产厂商全称长度不能超过300")
    @ApiModelProperty(value = "生产厂商全称")
    private String productionDetail;

    /**
     * 水泵数量
     */
    @Size(max = 85, message = "水泵数量长度不能超过85")
    @ApiModelProperty(value = "水泵数量")
    private String quantity;

    /**
     * 水泵数据集合json扩展
     */
    @Size(max = 1333, message = "水泵数据集合json扩展长度不能超过1333")
    @ApiModelProperty(value = "水泵数据集合json扩展")
    private String basePumpList;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 删除，0未删除，1已删除
     */
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;

    /**
     * 创建人
     */
    @Digits(integer = 20, fraction = 0, message = "创建人长度不能超过20")
    @ApiModelProperty(value = "创建人")
    private Long createdUser;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;

    /**
     * 更新人
     */
    @Digits(integer = 20, fraction = 0, message = "更新人长度不能超过20")
    @ApiModelProperty(value = "更新人")
    private Long updatedUser;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updatedTime;

    /**
     * 水泵基本信息集合
     */
    @ApiModelProperty(value = "水泵基本信息集合")
    private List<BasePumpDetail> pumpDetails;
}

