package com.tofly.water.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import com.tofly.water.entity.PlanPump;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 调度方案管理(LaborPlan)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:01
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "LaborPlan对象", description = "调度方案管理")
public class LaborPlanDto implements Serializable {

    private static final long serialVersionUID = 173112153340218740L;

    /**
     * 主键ID
     */
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 方案名称
     */
    @Size(max = 85, message = "方案名称长度不能超过85")
    @NotBlank(message = "方案名称不能为空")
    @ApiModelProperty(value = "方案名称")
    private String name;

    /**
     * TF_BASE_PUMP表ID（所属泵站ID）
     */
    @Digits(integer = 20, fraction = 0, message = "TF_BASE_PUMP表ID（所属泵站ID）长度不能超过20")
    @NotNull(message = "TF_BASE_PUMP表ID（所属泵站ID）不能为空")
    @ApiModelProperty(value = "TF_BASE_PUMP表ID（所属泵站ID）")
    private Long pumpId;

    /**
     * TF_BASE_PUMP_GROUP表（水泵机组ID）
     */
    @Digits(integer = 20, fraction = 0, message = "TF_BASE_PUMP_GROUP表（水泵机组ID）长度不能超过20")
    @NotNull(message = "TF_BASE_PUMP_GROUP表（水泵机组ID）不能为空")
    @ApiModelProperty(value = "TF_BASE_PUMP_GROUP表（水泵机组ID）")
    private Long pumpGroupId;

    /**
     * 方案编号
     */
    @Size(max = 85, message = "方案编号长度不能超过85")
    @ApiModelProperty(value = "方案编号")
    private String code;

    /**
     * 运行时长
     */
    @Size(max = 85, message = "运行时长长度不能超过85")
    @NotBlank(message = "运行时长不能为空")
    @ApiModelProperty(value = "运行时长")
    private String runTime;

    /**
     * 预计供水量
     */
    @Size(max = 85, message = "预计供水量长度不能超过85")
    @ApiModelProperty(value = "预计供水量")
    private String estimateWaterCapacity;

    /**
     * 应用场景
     */
    @Size(max = 85, message = "应用场景长度不能超过85")
    @ApiModelProperty(value = "应用场景")
    private String scene;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 删除，0未删除，1已删除
     */
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;

    /**
     * 创建人
     */
    @Digits(integer = 20, fraction = 0, message = "创建人长度不能超过20")
    @ApiModelProperty(value = "创建人")
    private Long createdUser;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;

    /**
     * 更新人
     */
    @Digits(integer = 20, fraction = 0, message = "更新人长度不能超过20")
    @ApiModelProperty(value = "更新人")
    private Long updatedUser;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updatedTime;

    @TableField(exist = false)
    @ApiModelProperty(value = "水泵信息集合")
    private List<PlanPump> planPumps;
}

