package com.tofly.water.entity.dto;


import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.baomidou.mybatisplus.annotation.*;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiModel;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.codehaus.jackson.map.ser.std.ToStringSerializer;
import org.springframework.format.annotation.DateTimeFormat;
import lombok.Data;

import java.util.*;

import org.springframework.web.multipart.MultipartFile;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;

import java.io.Serializable;

import lombok.EqualsAndHashCode;

import javax.validation.constraints.*;

/**
 * 安防监控（门禁管理）(SafeMonitor)表实体类
 *
 * @author HaiQiu
 * @since 2022-11-22 13:57:02
 */
@Data
@Accessors(chain = true)
@NoArgsConstructor
@EqualsAndHashCode(callSuper = false)
@ApiModel(value = "SafeMonitor对象", description = "安防监控（门禁管理）")
public class SafeMonitorDto implements Serializable {

    private static final long serialVersionUID = 432053420262919012L;

    /**
     * 主键ID
     */
    @Digits(integer = 20, fraction = 0, message = "主键ID长度不能超过20")
    @ApiModelProperty(value = "主键ID")
    @TableId(type = IdType.ASSIGN_ID)
    private Long id;

    /**
     * 门禁名称
     */
    @Size(max = 30, message = "门禁名称长度不能超过30")
    @NotBlank(message = "门禁名称不能为空")
    @ApiModelProperty(value = "门禁名称")
    private String name;

    /**
     * TF_BASE_PUMP表ID（所属泵站ID）
     */
    @Digits(integer = 20, fraction = 0, message = "TF_BASE_PUMP表ID（所属泵站ID）长度不能超过20")
    @NotNull(message = "TF_BASE_PUMP表ID（所属泵站ID）不能为空")
    @ApiModelProperty(value = "TF_BASE_PUMP表ID（所属泵站ID）")
    private Long pumpId;

    /**
     * 门禁编号
     */
    @Size(max = 85, message = "门禁编号长度不能超过85")
    @ApiModelProperty(value = "门禁编号")
    private String code;

    /**
     * 安装位置
     */
    @Size(max = 300, message = "安装位置长度不能超过300")
    @ApiModelProperty(value = "安装位置")
    private String installationPosition;

    /**
     * 管理部门
     */
    @Size(max = 85, message = "管理部门长度不能超过85")
    @ApiModelProperty(value = "管理部门")
    private String managementDept;

    /**
     * 管理负责人
     */
    @Size(max = 85, message = "管理负责人长度不能超过85")
    @ApiModelProperty(value = "管理负责人")
    private String managementLeader;

    /**
     * 管理联系电话
     */
    @Size(max = 85, message = "管理联系电话长度不能超过85")
    @ApiModelProperty(value = "管理联系电话")
    private String managementLeaderPhone;

    /**
     * 供应商
     */
    @Size(max = 85, message = "供应商长度不能超过85")
    @ApiModelProperty(value = "供应商")
    private String supplierSimple;

    /**
     * 供应商联系电话
     */
    @Size(max = 85, message = "供应商联系电话长度不能超过85")
    @ApiModelProperty(value = "供应商联系电话")
    private String supplierPhone;

    /**
     * 安装负责人
     */
    @Size(max = 85, message = "安装负责人长度不能超过85")
    @ApiModelProperty(value = "安装负责人")
    private String installationLeader;

    /**
     * 安装时间
     */
    @ApiModelProperty(value = "安装时间")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date installationDate;

    /**
     * 备注
     */
    @Size(max = 300, message = "备注长度不能超过300")
    @ApiModelProperty(value = "备注")
    private String remark;

    /**
     * 附件
     */
    @Size(max = 1333, message = "附件长度不能超过1333")
    @ApiModelProperty(value = "附件")
    private String annex;

    /**
     * 排序
     */
    @Digits(integer = 15, fraction = 0, message = "排序长度不能超过15")
    @ApiModelProperty(value = "排序")
    private Long sort;

    /**
     * 删除，0未删除，1已删除
     */
    @ApiModelProperty(value = "删除，0未删除，1已删除")
    private Boolean deleted;

    /**
     * 创建人
     */
    @Digits(integer = 20, fraction = 0, message = "创建人长度不能超过20")
    @ApiModelProperty(value = "创建人")
    private Long createdUser;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createdTime;

    /**
     * 更新人
     */
    @Digits(integer = 20, fraction = 0, message = "更新人长度不能超过20")
    @ApiModelProperty(value = "更新人")
    private Long updatedUser;

    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updatedTime;

}

