package com.tofly.water.service;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.IService;
import com.tofly.water.entity.SafeMonitor;
import com.tofly.water.entity.dto.SafeMonitorDto;

import org.springframework.web.multipart.MultipartFile;

import java.io.Serializable;
import java.util.List;

/**
 * 安防监控（门禁管理）(SafeMonitor)表服务接口
 *
 * @author HaiQiu
 * @since 2022-11-22 11:39:36
 */
public interface SafeMonitorService extends IService<SafeMonitor> {

    /**
     * 根据ID查询
     *
     * @param id id
     * @return 数据
     */
    SafeMonitor get(Serializable id);

    /**
     * 新增
     *
     * @param safeMonitorDto 实体
     * @param files          上传文件附件
     */
    void insert(SafeMonitorDto safeMonitorDto, List<MultipartFile> files);

    /**
     * 修改
     *
     * @param safeMonitorDto 实体
     * @param files          上传文件附件
     */
    void edit(SafeMonitorDto safeMonitorDto, List<MultipartFile> files);

    /**
     * 批量删除
     *
     * @param idList 数据ID集合
     */
    void delete(List<Long> idList);

    /**
     * 生成模板下载
     */
    void template();

    /**
     * 数据导入
     *
     * @param file 文件
     */
    void importData(MultipartFile file);

    /**
     * 分页查询
     *
     * @param page        分页
     * @param safeMonitor 查询wrapper
     * @return 数据集合
     */
    Page<SafeMonitor> pageList(Page page, SafeMonitor safeMonitor);

    /**
     * 数据导出
     *
     * @param page        分页
     * @param safeMonitor 查询模型
     * @param isAll       是否导出全部数据，true是，false否
     */
    void export(Page page, SafeMonitor safeMonitor, Boolean isAll);

    /**
     * 查询所有数据
     *
     * @param safeMonitor 查询条件
     * @return 查询所有数据
     */
    List<SafeMonitor> allList(SafeMonitor safeMonitor);
}

