package com.tofly.toflygetway.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.tofly.toflygetway.config.ThirdSysTokenConfig;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.*;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
@Slf4j
public class ThirdTokenSwitchFilter implements GlobalFilter, Ordered {
    private final  static  String  third_token_key="third:token:key";
    @Autowired
    ThirdSysTokenConfig thirdSysTokenConfig;
    @Autowired
    private RedisTemplate redisTemplate;
    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        final List<String> sysFrom = exchange.getRequest().getHeaders().get("sysFrom");
        if(sysFrom !=null &&sysFrom.size()>0){
            //有第三方请求的标记
            final List<String> authorization
                    = exchange.getRequest().getHeaders().get("Authorization");
            final String localToken ;
            if(redisTemplate.opsForHash().hasKey(third_token_key,authorization.get(0))){
                localToken= (String) redisTemplate.opsForHash().get(third_token_key,authorization.get(0));
            }else{
                localToken = exchangeToken(authorization.get(0));
                redisTemplate.opsForHash().put(third_token_key,authorization.get(0),localToken);
            }
            // 1. 清洗请求头中from 参数
            request.mutate()
                    .headers(httpHeaders -> httpHeaders.set("Authorization","Bearer "+localToken))
                    .build();

        }



        return chain.filter(exchange.mutate()
                .request(request.mutate()
                        .build()).build());
    }

    @Override
    public int getOrder() {


         return -80;
    }

   String exchangeToken(String token){
        log.info("开始token交换");
        HttpHeaders headers = new HttpHeaders();
        headers.set("Authorization", token);
        headers.setContentType(MediaType.APPLICATION_JSON);

        MultiValueMap<String, String> params = new LinkedMultiValueMap<String, String>();
//  也支持中文
        params.add("username", "token");
        params.add("password", token.replace("Bearer ","").replace("bearer ","").trim());
        HttpEntity<MultiValueMap<String, String>> requestEntity =
                new HttpEntity<MultiValueMap<String, String>>(params, headers);
        StringBuffer urlBuffer = new StringBuffer(thirdSysTokenConfig.getUrl());
        urlBuffer.append("?username=").append("token");
        urlBuffer.append("&password=").append(token.replace("Bearer ","").trim());
        RestTemplate restTemplate = new RestTemplate();
        ResponseEntity<String> responseEntity = restTemplate.exchange(urlBuffer.toString(), HttpMethod.POST, requestEntity, String.class);
        String responseBody = responseEntity.getBody();
        final JSONObject jsonObject = JSON.parseObject(responseBody);
        final JSONObject result = jsonObject.getJSONObject("result");
        String accessToken = (String) result.get("accessToken");
       log.info("结束token交换");
        return  accessToken;
    }
}
