/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.quality.controller;


import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.pojo.statics.dto.StaticsQueryDto;
import com.tofly.quality.entity.CollectMs;
import com.tofly.quality.entity.dto.CollectMsQuery;
import com.tofly.quality.service.CollectMsService;
import lombok.AllArgsConstructor;
import org.apache.ibatis.annotations.Param;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;


import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 眉山水质检测采集主表
 *
 * @author admin
 * @date Thu Mar 23 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/collectms")
@Api(tags = "眉山水质检测采集主表接口")
public class CollectMsController {

    private final CollectMsService collectMsService;

    /**
     * 分页查询
     *
     * @param page  分页对象
     * @param query 眉山水质检测采集主表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getCollectMsPage(Page page, @ModelAttribute CollectMsQuery query) {
        return ResultRespone.success(collectMsService.getPageData(page, query));
    }


    /**
     * 通过id查询眉山水质检测采集主表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(collectMsService.getById(id));
    }

    /**
     * 新增眉山水质检测采集主表
     *
     * @param collectMs 眉山水质检测采集主表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增眉山水质检测采集主表")
    @ApiOperation(value = "新增眉山水质检测采集主表")
    @PostMapping
    public ResultRespone save(@Validated CollectMs collectMs, @RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        return ResultRespone.success(collectMsService.saveCollectResult(collectMs, fileList, true));
    }

    /**
     * 修改眉山水质检测采集主表
     *
     * @param collectMs 眉山水质检测采集主表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改眉山水质检测采集主表")
    @ApiOperation(value = "修改眉山水质检测采集主表")
    @PutMapping
    public ResultRespone updateById(@Validated CollectMs collectMs, @RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        if (collectMs.getId() == 0) {
            return ResultRespone.failed("参数id不能为空");
        }
        return ResultRespone.success(collectMsService.saveCollectResult(collectMs, fileList, false));
    }

    /**
     * 通过id删除眉山水质检测采集主表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除眉山水质检测采集主表")
    @ApiOperation(value = "通过ID删除眉山水质检测采集主表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(collectMsService.deleteByIds(id + ""));
    }

    /**
     * 通过id批量删除眉山水质检测采集主表
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除眉山水质检测采集主表")
    @ApiOperation(value = "通过ID批量删除眉山水质检测采集主表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(collectMsService.deleteByIds(ids));
    }


}
