/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.quality.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.quality.entity.SolidwasteType;
import com.tofly.quality.entity.dto.SolidWasteTypeQuery;
import com.tofly.quality.service.SolidwasteTypeService;
import lombok.AllArgsConstructor;

import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.validation.Valid;


/**
 * 固废类型表
 *
 * @author admin
 * @date Thu Mar 30 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/solidwastetype")
@Api(tags = "固废类型表接口")
public class SolidwasteTypeController {

    private final SolidwasteTypeService solidwasteTypeService;

    /**
     * 分页查询
     *
     * @param page  分页对象
     * @param query 固废类型表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getSolidwasteTypePage(Page page, @ModelAttribute SolidWasteTypeQuery query) {
        return ResultRespone.success(solidwasteTypeService.getPageData(page, query));
    }


    /**
     * 通过id查询固废类型表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(solidwasteTypeService.getById(id));
    }

    /**
     * 新增固废类型表
     *
     * @param solidwasteType 固废类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增固废类型表")
    @ApiOperation(value = "新增固废类型表")
    @PostMapping
    public ResultRespone save(@RequestBody @Valid SolidwasteType solidwasteType) {
        Boolean flag = solidwasteTypeService.selectByName(solidwasteType.getTnameS(), 0L);
        if (flag) {
            return ResultRespone.success(solidwasteTypeService.save(solidwasteType));
        }
        return ResultRespone.failed("已存在的小类，无法重复创建！");
    }

    /**
     * 修改固废类型表
     *
     * @param solidwasteType 固废类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改固废类型表")
    @ApiOperation(value = "修改固废类型表")
    @PutMapping
    public ResultRespone updateById(@RequestBody @Valid SolidwasteType solidwasteType) {
        if (solidwasteType.getId() == null || solidwasteType.getId() == 0) {
            return ResultRespone.failed("参数id不能为空");
        }
        Boolean flag = solidwasteTypeService.selectByName(solidwasteType.getTnameS(), 0L);
        if (flag) {
            return ResultRespone.success(solidwasteTypeService.updateById(solidwasteType));
        }
        return ResultRespone.failed("已存在的小类，无法重复创建！");
    }

    /**
     * 通过id删除固废类型表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除固废类型表")
    @ApiOperation(value = "通过ID删除固废类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(solidwasteTypeService.deleteByIds(id+""));
    }

    /**
     * 通过id批量删除固废类型表
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除固废类型表")
    @ApiOperation(value = "通过ID批量删除固废类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(solidwasteTypeService.deleteByIds(ids));
    }
}
