/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.quality.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.quality.entity.WasteMgt;
import com.tofly.quality.entity.dto.WasteMgQuery;
import com.tofly.quality.service.WasteMgtService;
import lombok.AllArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 固废物处理台账
 *
 * @author Fall_Mu
 * @date Mon Apr 03 00:00:00 CST 2023
 */
@RestController
@AllArgsConstructor
@RequestMapping("/wastemgt")
@Api(tags = "固废物处理台账接口")
public class WasteMgtController {

    private final WasteMgtService wasteMgtService;

    /**
     * 分页查询
     *
     * @param page  分页对象
     * @param query 固废物处理台账
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getWasteMgtPage(Page page, @ModelAttribute WasteMgQuery query) {
        return ResultRespone.success(wasteMgtService.getPageData(page, query));
    }

    /**
     * 通过id查询固废物处理台账
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(wasteMgtService.getById(id));
    }

    /**
     * 新增固废物处理台账
     *
     * @param wasteMgt 固废物处理台账
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增固废物处理台账")
    @ApiOperation(value = "新增固废物处理台账")
    @PostMapping
    public ResultRespone save(@Validated WasteMgt wasteMgt, @RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        return ResultRespone.success(wasteMgtService.saveWasteMgtResult(wasteMgt, fileList, true));
    }

    /**
     * 修改固废物处理台账
     *
     * @param wasteMgt 固废物处理台账
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改固废物处理台账")
    @ApiOperation(value = "修改固废物处理台账")
    @PutMapping
    public ResultRespone updateById(@Validated WasteMgt wasteMgt, @RequestParam(value = "fileList", required = false) List<MultipartFile> fileList) {
        if (wasteMgt.getId() == 0) {
            return ResultRespone.failed("参数id不能为空");
        }
        return ResultRespone.success(wasteMgtService.saveWasteMgtResult(wasteMgt, fileList, false));
    }

    /**
     * 通过id删除固废物处理台账
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除固废物处理台账")
    @ApiOperation(value = "通过ID删除固废物处理台账")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(wasteMgtService.DeleteByIds(id + ""));
    }

    /**
     * 通过id批量删除固废物处理台账
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除固废物处理台账")
    @ApiOperation(value = "通过ID批量删除固废物处理台账")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(wasteMgtService.DeleteByIds(ids));
    }
}
