/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.quality.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.tofly.quality.entity.comm.File;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.format.annotation.DateTimeFormat;

import java.util.List;
import java.util.stream.Collectors;


/**
 * 固废物处理台账
 *
 * @author Fall_Mu
 * @date Mon Apr 03 00:00:00 CST 2023
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "TF_SOLID_WASTE_MGT_S", resultMap = "wasteMgtMap")
@ApiModel
public class WasteMgt extends Model<WasteMgt> {
    private static final long serialVersionUID = 1L;

    /**
     * 主键id
     */
    @TableId(value = "ID", type = IdType.ASSIGN_ID)
    @ApiModelProperty(value = "主键id")
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;
    /**
     * 产生单位
     */
    @ApiModelProperty(value = "产生单位")
    private String proUnit;
    /**
     * 类别id 关联tf_solidwaste_type_s表id
     */
    @ApiModelProperty(value = "类别id 关联tf_solidwaste_type_s表id")
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long typeId;
    /**
     * 固废物名称
     */
    @ApiModelProperty(value = "固废物名称")
    private String wasteName;
    /**
     * 处置公司
     */
    @ApiModelProperty(value = "处置公司")
    private String disposalCom;
    /**
     * 处置数量(吨)
     */
    @ApiModelProperty(value = "处置数量(吨)")
    private Double disposalNum;
    /**
     * 处置时间
     */
    @ApiModelProperty(value = "处置时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date disposalTime;
    /**
     * 物联单号，多个单号用','分割
     */
    @ApiModelProperty(value = "物联单号，多个单号用','分割")
    private String serialNumber;
    /**
     * 创建人
     */
    @ApiModelProperty(value = "创建人")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 更新人员
     */
    @ApiModelProperty(value = "更新人员")
    @TableField(value = "UPDATE_USER", fill = FieldFill.INSERT_UPDATE)
    private Long updateUser;
    @ApiModelProperty(hidden = true)
    private String updateUserName;
    /**
     * 更新时间
     */
    @ApiModelProperty(value = "更新时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "UPDATE_TIME", fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;
    /**
     * 作废标志 0-正常，1-作废
     */
    @ApiModelProperty(value = "作废标志 0-正常，1-作废")
    @TableLogic(value = "0",delval = "1")
    private String isDelete;

    /**
     * 附件IDS，多个附件id用英文等号分割“,”,关联公共文件表TF_YWPN_FILE _W主键
     */
    @ApiModelProperty(value = "附件IDS，多个附件id用英文等号分割“,”,关联公共文件表TF_YWPN_FILE _W主键")
    private String uploadFileids;

    /**
     * 备注描述
     */
    @ApiModelProperty(value = "备注描述")
    private String remark;


    @ApiModelProperty(hidden = true, value = "文件路径列表")
    @TableField(exist = false)
    private List<File> fileUpLoadList;

    /***
     *固废物类型列表
     */
    @ApiModelProperty(hidden = true, value = "固废物类型列表")
    @TableField(exist = false)
    private List<SolidwasteType> solidWasteTypeDetails;


}
