/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.quality.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.pojo.statics.dto.StaticsQueryDto;
import com.tofly.quality.entity.AssayResMxW;
import com.tofly.quality.entity.CollectMs;
import com.tofly.quality.entity.CollectW;
import com.tofly.quality.entity.dto.CollectMsQuery;
import com.tofly.quality.entity.dto.CollectWQuery;
import com.tofly.quality.service.AssayResMxWService;
import com.tofly.quality.service.CollectMsService;
import com.tofly.quality.service.comm.FileService;
import com.tofly.quality.unit.UploadFiles;
import org.apache.commons.lang.time.DateFormatUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.tofly.quality.mapper.CollectMsMapper;
import org.springframework.web.multipart.MultipartFile;

import java.security.Security;
import java.sql.Wrapper;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 眉山水质检测采集主表
 *
 * @author admin
 * @date Thu Mar 23 00:00:00 CST 2023
 */
@Service("collectMsService")
public class CollectMsServiceImpl extends ServiceImpl<CollectMsMapper, CollectMs> implements CollectMsService {

    @Autowired
    CollectMsMapper collectMsMapper;

    @Value("${quality.ftp.build_result}")
    private String buildFilePath;

    @Autowired
    private FileService fileService;

    @Autowired
    private AssayResMxWService assayResMxWService;


    @Override
    public boolean saveCollectResult(CollectMs CollectMs, List<MultipartFile> fileList, boolean isAdd) {
        boolean flag = false;
        // 判断当前是否有文件,则存入返回文件id
        String tableName = "TF_WATERQUALITY_COLLECT_MS_W";
        if (CollectMs.getAssayResultDetails() == null || CollectMs.getAssayResultDetails().stream().count() == 0) {
            throw new RuntimeException("化验项明细数据不能为空");

        }
        if (isAdd) {
            if (CollUtil.isNotEmpty(fileList)) {
                String fileIds = UploadFiles.saveFiles(fileService, fileList, CollectMs.getUploadFileids(), CollectMs.getUploadFileids(),
                        buildFilePath, tableName);
                if (!StringUtil.isEmpty(fileIds)) {
                    CollectMs.setUploadFileids(fileIds);
                }
            }

            Date now = new Date(); // 创建一个Date对象，获取当前时间
            String strDateFormat = "yyyyMMdd";
            String start_no = DateFormatUtils.format(now, strDateFormat);
            //获取当天最大的采样编号
            String Sample_No = this.baseMapper.getResultMaxNo(start_no);
            int firstNum = 1;
            if (Sample_No.length() == 12) {
                firstNum = Integer.parseInt(Sample_No.substring(7)) + 1;
            }
            String end_no = String.format("%04d", firstNum); //补全数字，不足用0补位

            CollectMs.setCollectNo(start_no + end_no);
            flag = this.save(CollectMs);
        } else {
            String fileIds = "";
            if (StringUtil.isNotEmpty(CollectMs.getUploadFileids())) {
                fileIds = CollectMs.getUploadFileids();
                if (fileIds.equals("[]")) {
                    fileIds = "";
                }
            }
            if (StringUtil.isEmpty(CollectMs.getRemark()) || CollectMs.getRemark().equals("null")) {
                CollectMs.setRemark("");
            }
            CollectMs old_data = this.getById(CollectMs.getId());
            String fileIdsList = UploadFiles.saveFiles(fileService, fileList, old_data.getUploadFileids(),
                    fileIds, buildFilePath, tableName);

            CollectMs.setUploadFileids(fileIdsList);
            assayResMxWService.remove(Wrappers.<AssayResMxW>lambdaQuery()
                    .eq(AssayResMxW::getAssayResultId, CollectMs.getId()));
            flag = this.updateById(CollectMs);
        }
        for (AssayResMxW detail : CollectMs.getAssayResultDetails()
        ) {
            detail.setAssayResultId(CollectMs.getId());
            detail.setCollectNo(CollectMs.getCollectNo());
            assayResMxWService.save(detail);
        }
        return flag;
    }

    @Override
    public Page<CollectMs> getPageData(Page page, CollectMsQuery query) {
        if(query.getCollectStartTime()!=null&&query.getCollectEndTime()!=null){
            if(query.getCollectStartTime().getTime()>query.getCollectEndTime().getTime()){
                throw new RuntimeException("采样开始时间不能大于结束时间");
            }
        }
        if(query.getInspectionStartTime()!=null&&query.getInspectionEndTime()!=null){
            if(query.getInspectionStartTime().getTime()>query.getInspectionEndTime().getTime()){
                throw  new RuntimeException("送检开始时间不能大于送检结束时间");
            }
        }
        return this.baseMapper.getPageData(page, query);
    }

    @Override
    public boolean deleteByIds(String ids) {
        List<String> list = Arrays.asList(ids.split(","));
     //   Integer  count_check =0;
//        count_check=this.baseMapper.selectCount(Wrappers.<CollectMs>lambdaQuery()
//                .in(CollectMs::getId,list).ne(CollectMs::getCreateUser, SecurityUtils.getUserId()));
//        if (count_check > 0) {
//            throw new RuntimeException("只能删除自己创建的数据");
//        }
        //删除明细数据
        assayResMxWService.remove(Wrappers.<AssayResMxW>lambdaQuery()
                .in(AssayResMxW::getAssayResultId,list));
        return super.removeByIds(list);
    }

    @Override
    public List<CollectMs> getResultList(Long collectMsId) {
        if (collectMsId == null) {
            collectMsId = 0L;
        }
        return this.baseMapper.getResultList(collectMsId);
    }

    @Override
    public List<Map<String, Integer>> staticsDataOverMonth(StaticsQueryDto staticsQueryDto) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper. ge("collect_Time", staticsQueryDto.getBeginDate());
        queryWrapper. le("collect_Time", staticsQueryDto.getEndDate());
        queryWrapper.groupBy( "DATE_FORMAT(collect_Time, '%Y%m')");
        return baseMapper.staticsDataOverMonth(staticsQueryDto,queryWrapper);
    }

}
