/*
 * Author: wangjian
 * 生成ServiceImpl
 */
package com.tofly.quality.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.quality.entity.WasteMgt;
import com.tofly.quality.entity.dto.WasteMgQuery;
import com.tofly.quality.service.WasteMgtService;
import com.tofly.quality.service.comm.FileService;
import com.tofly.quality.unit.UploadFiles;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.tofly.quality.mapper.WasteMgtMapper;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;


/**
 * 固废物处理台账
 *
 * @author Fall_Mu
 * @date Mon Apr 03 00:00:00 CST 2023
 */
@Service("wasteMgtService")
public class WasteMgtServiceImpl extends ServiceImpl<WasteMgtMapper, WasteMgt> implements WasteMgtService {

    @Value("${quality.ftp.build_result}")
    private String buildFilePath;

    @Autowired
    private FileService fileService;

    @Override
    public boolean saveWasteMgtResult(WasteMgt WasteMgt, List<MultipartFile> fileList, boolean isAdd) {
        boolean flag = false;
        //判断当前是否有文件，有则返回id
        String tabName = "TF_SOLID_WASTE_MGT_S";
        if (isAdd) {
            if (CollUtil.isNotEmpty(fileList)) {
                String fileIds = UploadFiles.saveFiles(fileService, fileList, WasteMgt.getUploadFileids(),
                        WasteMgt.getUploadFileids(), buildFilePath, tabName);
                if (!StringUtil.isEmpty(fileIds)) {
                    WasteMgt.setUploadFileids(fileIds);
                }
            }
            flag = this.save(WasteMgt);
        } else {
            String fileIds = "";
            if (StringUtil.isNotEmpty(WasteMgt.getUploadFileids())) {
                fileIds = WasteMgt.getUploadFileids();
                if (fileIds.equals("[]")) {
                    fileIds = "";
                }
            }
            if (StringUtil.isEmpty(WasteMgt.getRemark()) || WasteMgt.getRemark().equals("null")) {
                WasteMgt.setRemark("");
            }
            WasteMgt old_data = this.getById(WasteMgt.getId());
            String fileIdsList = UploadFiles.saveFiles(fileService, fileList, old_data.getUploadFileids(),
                    fileIds, buildFilePath, tabName);
            WasteMgt.setUploadFileids(fileIdsList);
            flag = this.updateById(WasteMgt);

        }
        return flag;
    }

    @Override
    public Page<WasteMgt> getPageData(Page page, WasteMgQuery query) {
        if (query.getDisposalStartTime() != null && query.getDisposalEndTime() != null) {
            if (query.getDisposalStartTime().getTime() > query.getDisposalEndTime().getTime()) {
                throw new RuntimeException("处置开始时间不能大于处置结束时间");
            }
        }
        return this.baseMapper.getPageData(page, query);
    }

    @Override
    public boolean DeleteByIds(String ids) {
        List<String> list = Arrays.asList(ids.split(","));
        return super.removeByIds(list);
    }
}
