/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.watermeter.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.watermeter.entity.Bwztcode;
import com.tofly.watermeter.service.BwztcodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;

/**
 * 水表状态编码表
 *
 * @author admin
 * @date Wed Mar 10 00:00:00 CST 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/bwztcode")
@Api(tags="水表状态编码表接口")
public class BwztcodeController {

    private final  BwztcodeService bwztcodeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param bwztcode 水表状态编码表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getBwztcodePage(Page page, @ModelAttribute Bwztcode bwztcode) {
      return ResultRespone.success(bwztcodeService.page(page, Wrappers.query(bwztcode)));
    }


    /**
     * 通过id查询水表状态编码表
     * @param bm id
     * @return ResultRespone
     */
    @GetMapping("/{bm}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bm", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("bm" ) Long bm) {
      return ResultRespone.success(bwztcodeService.getById(bm));
    }

    /**
     * 新增水表状态编码表
     * @param bwztcode 水表状态编码表
     * @return ResultRespone
     */
    //="新增水表状态编码表" )
    @ApiOperation(value = "新增水表状态编码表")
    @PostMapping
    public ResultRespone save(@RequestBody Bwztcode bwztcode) {
      return ResultRespone.success(bwztcodeService.save(bwztcode));
    }

    /**
     * 修改水表状态编码表
     * @param bwztcode 水表状态编码表
     * @return ResultRespone
     */
    //="修改水表状态编码表" )
    @ApiOperation(value = "修改水表状态编码表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Bwztcode bwztcode) {
      return ResultRespone.success(bwztcodeService.updateById(bwztcode));
    }

    /**
     * 通过id删除水表状态编码表
     * @param bm id
     * @return ResultRespone
     */
    //="通过id删除水表状态编码表" )
    @ApiOperation(value = "通过ID删除水表状态编码表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "bm", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{bm}" )
    public ResultRespone removeById(@PathVariable Long bm) {
      return ResultRespone.success(bwztcodeService.removeById(bm));
    }

    /**
     * 通过id批量删除水表状态编码表
     * @param ids id
     * @return ResultRespone
     */
    //="通过id批量删除水表状态编码表" )
    @ApiOperation(value = "通过ID批量删除水表状态编码表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(bwztcodeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
