package com.tofly.watermeter.custom.service.meterBusiness.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUnit;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.api.UserClent;
import com.tofly.common.core.util.DateUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ChangeTableBatchDto;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ChangeTableCondition;
import com.tofly.watermeter.custom.entity.meterBusiness.dto.ChangeTableDto;
import com.tofly.watermeter.custom.mapper.meterBusiness.ChangeTableMapper;
import com.tofly.watermeter.custom.service.meterBusiness.ChangeTableService;
import com.tofly.watermeter.custom.service.meterBusiness.export.ExportCommonCodeService;
import com.tofly.watermeter.entity.Bwhbdatail;
import com.tofly.watermeter.entity.Bwrksbdatail;
import com.tofly.watermeter.entity.File;
import com.tofly.watermeter.entity.Qfkhjbxx;
import com.tofly.watermeter.mapper.BwhbdatailMapper;
import com.tofly.watermeter.service.BwhbdatailService;
import com.tofly.watermeter.service.BwrksbdatailService;
import com.tofly.watermeter.service.FileService;
import com.tofly.watermeter.service.QfkhjbxxService;
import com.tofly.watermeter.util.CallAPIUtil;
import com.tofly.watermeter.util.service.FileUpLoadService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
@Slf4j
public class ChangeTableServiceImpl implements ChangeTableService {

    private final QfkhjbxxService qfkhjbxxService;
    private final BwrksbdatailService bwrksbdatailService;
    private final BwhbdatailService bwhbdatailService;
    private final ChangeTableMapper changeTableMapper;
    private final ExportCommonCodeService exportCommonCodeService;
    private final FileUpLoadService fileUpLoadService;
    private final UserClent userClent;
    private final FileService fileService;
    private final BwhbdatailMapper bwhbdatailMapper;

    /**
     * 水表换表
     *
     * @param dto dto
     * @return boolean
     * @throws Exception
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeTable(ChangeTableDto dto, List<MultipartFile> signature, List<MultipartFile> images) throws Exception {

        if (StringUtils.isEmpty(dto.getYhbh())) {
            throw new ToflyDeniedException("用户编号为空");
        }

        Qfkhjbxx qfkhjbxx = qfkhjbxxService.getOne(Wrappers.<Qfkhjbxx>lambdaQuery()
                .eq(Qfkhjbxx::getYhbh, dto.getYhbh()));

        if (qfkhjbxx != null) {

            String signatureId = null;
            String imageId = null;
            if (CollectionUtil.isNotEmpty(signature)) {
                signatureId = fileUpLoadService.saveFileList(signature, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
            }
            if ((CollectionUtil.isNotEmpty(images))) {
                imageId = fileUpLoadService.saveFileList(images, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
            }

            String date = DateUtil.getDateFormat(new Date(), "yyyyMMdd-");

            dto.setAction("QUERY");

            Map map = CallAPIUtil.callAPIUtil(dto, "/api/Business/WaterReplace");

            if (CollectionUtil.isNotEmpty(map)) {
                throw new ToflyDeniedException("调用换表同步接口失败");
            }

            log.info(String.valueOf(map.get("massage")));
            //0000代表添加成功
            if ("0000".equals(map.get("Code"))) {
                Bwhbdatail dbl = new Bwhbdatail();
                dbl.setAzfs(dto.getSbazfs())
                        .setBwBz(dto.getBz())
                        .setBwHblx(Long.valueOf(dto.getSblx()))
                        .setBwKhbh(dto.getYhbh())
                        .setBwHbrq(new Date())
                        .setBwHbry(dto.getHbr())
                        .setBwHbyy(dto.getHbyy())
                        .setBwJbcj(qfkhjbxx.getSbcj())
                        .setBwJblx(qfkhjbxx.getSblx())
                        .setBwJbbh(qfkhjbxx.getSbbh())
                        .setBwJbkj(qfkhjbxx.getSbkj())
                        .setBwJbzd(Long.valueOf(dto.getJbzd()))
                        .setBwJbqd(Long.valueOf(dto.getJbqd()))
                        .setBwKhdz(qfkhjbxx.getYhdz())
                        .setBwKhdh(qfkhjbxx.getLxdh())
                        .setBwKhxm(qfkhjbxx.getYhxm())
                        .setBwHblx(Long.valueOf(dto.getSblx()))
                        .setSqr(dto.getSqr())
                        .setHbsl(dto.getHbsl())
                        .setBwLsbh(date + System.currentTimeMillis())
                        .setJsfs(dto.getHbsljsfs())
                        .setBwXbqd(Long.valueOf(dto.getXbqd()))
                        .setSbm(dto.getSbsbm())
                        .setQfh(dto.getSbqfh())
                        .setBwXbcj(dto.getSbmp())
                        .setBwXblx(dto.getSblx())
                        .setSbxh(dto.getSbxh())
                        .setBwXbkj(dto.getSbkj())
                        .setBwXbbh(dto.getSbbh())
                        .setJbAzrq(qfkhjbxx.getAzrq())
                        .setSignature(signatureId)
                        .setImage(imageId)
                        .setBwHbbm(changeTableMapper.getDeptId(Long.valueOf(dto.getHbr())));
                long betweenDay = cn.hutool.core.date.DateUtil.between(qfkhjbxx.getAzrq(), dbl.getBwHbrq(), DateUnit.DAY);
                dbl.setSyDate(betweenDay);

                bwhbdatailService.save(dbl);

                Long departmentId = userClent.getById(Long.valueOf(dto.getHbr())).getResult().getDepartmentId();

                Bwrksbdatail bwrksbdatail = bwrksbdatailService.getOne(Wrappers.<Bwrksbdatail>lambdaQuery().eq(Bwrksbdatail::getBwSbbh, dto.getSbbh()));

                if (bwrksbdatail != null) {
                    bwrksbdatail.setBwKhbh(dto.getYhbh()).setBwKhxm(qfkhjbxx.getYhxm()).setBwKhdh(qfkhjbxx.getLxdh())
                            .setBwKhdz(qfkhjbxx.getYhdz()).setBwAzry(Long.valueOf(dto.getHbr())).setBwSbqd(Integer.valueOf(dto.getXbqd()))
                            .setAzfs(dto.getSbazfs()).setSfncys(qfkhjbxx.getSfncys()).setJyz(qfkhjbxx.getJyz()).setAzbm(departmentId)
                            .setGddh(qfkhjbxx.getLxdh()).setSbazdz(qfkhjbxx.getYhdz()).setIssyc("1").setMsg("换表成功").setBwSbzt("3").setBwAzrq(new Date());

                    bwrksbdatailService.updateById(bwrksbdatail);
                }

                //旧表改为换表状态
                bwrksbdatailService.update(Wrappers.<Bwrksbdatail>lambdaUpdate()
                        .eq(Bwrksbdatail::getBwSbbh, qfkhjbxx.getSbbh()).set(Bwrksbdatail::getBwSbzt, 9));

            } else {
                throw new ToflyDeniedException(map.get("message").toString());
            }
        } else {
            throw new ToflyDeniedException("用户不存在");
        }
        return true;
    }

    @Override
    public boolean changeTableStaging(ChangeTableDto dto, List<MultipartFile> signature, List<MultipartFile> images) throws Exception {
        if (StringUtils.isEmpty(dto.getYhbh())) {
            throw new ToflyDeniedException("用户编号为空");
        }

        Qfkhjbxx qfkhjbxx = qfkhjbxxService.getOne(Wrappers.<Qfkhjbxx>lambdaQuery()
                .eq(Qfkhjbxx::getYhbh, dto.getYhbh()));

        if (qfkhjbxx != null) {

            String signatureId = null;
            String imageId = null;
            if (CollectionUtil.isNotEmpty(signature)) {
                signatureId = fileUpLoadService.saveFileList(signature, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
            }
            if ((CollectionUtil.isNotEmpty(images))) {
                imageId = fileUpLoadService.saveFileList(images, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
            }

            String date = DateUtil.getDateFormat(new Date(), "yyyyMMdd-");


            Bwhbdatail dbl = new Bwhbdatail();
            dbl.setAzfs(dto.getSbazfs())
                    .setBwBz(dto.getBz())
                    .setBwHblx(Long.valueOf(dto.getSblx()))
                    .setBwKhbh(dto.getYhbh())
                    .setBwHbrq(new Date())
                    .setBwHbry(dto.getHbr())
                    .setBwHbyy(dto.getHbyy())
                    .setBwJbcj(qfkhjbxx.getSbcj())
                    .setBwJblx(qfkhjbxx.getSblx())
                    .setBwJbbh(qfkhjbxx.getSbbh())
                    .setBwJbkj(qfkhjbxx.getSbkj())
                    .setBwJbzd(Long.valueOf(dto.getJbzd()))
                    .setBwJbqd(Long.valueOf(dto.getJbqd()))
                    .setBwKhdz(qfkhjbxx.getYhdz())
                    .setBwKhdh(qfkhjbxx.getLxdh())
                    .setBwKhxm(qfkhjbxx.getYhxm())
                    .setBwHblx(Long.valueOf(dto.getSblx()))
                    .setSqr(dto.getSqr())
                    .setHbsl(dto.getHbsl())
                    .setBwLsbh(date + System.currentTimeMillis())
                    .setJsfs(dto.getHbsljsfs())
                    .setBwXbqd(Long.valueOf(dto.getXbqd()))
                    .setSbm(dto.getSbsbm())
                    .setQfh(dto.getSbqfh())
                    .setBwXbcj(dto.getSbmp())
                    .setBwXblx(dto.getSblx())
                    .setSbxh(dto.getSbxh())
                    .setBwXbkj(dto.getSbkj())
                    .setBwXbbh(dto.getSbbh())
                    .setJbAzrq(qfkhjbxx.getAzrq())
                    .setSignature(signatureId)
                    .setImage(imageId)
                    .setIsSync(0L)
                    .setJyz(qfkhjbxx.getJyz())
                    .setNcys(qfkhjbxx.getSfncys())
                    .setBwHbbm(changeTableMapper.getDeptId(Long.valueOf(dto.getHbr())));
            long betweenDay = cn.hutool.core.date.DateUtil.between(qfkhjbxx.getAzrq(), dbl.getBwHbrq(), DateUnit.DAY);
            dbl.setSyDate(betweenDay);

            bwhbdatailService.save(dbl);
        }
        return true;
    }

    /**
     * 批量换表
     *
     * @return Boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean changeTableBatch(String json, List<MultipartFile> signature, List<MultipartFile> images) {

        ChangeTableBatchDto changeTableBatchDto = JSONObject.parseObject(json, ChangeTableBatchDto.class);

        if (changeTableBatchDto == null || CollectionUtil.isEmpty(changeTableBatchDto.getInfos())) {
            throw new ToflyDeniedException("传入数据为空");
        }

        List<ChangeTableDto> dtoList = changeTableBatchDto.getInfos();

        String signatureId = null;
        String imageId = null;
        if (CollectionUtil.isNotEmpty(signature)) {
            signatureId = fileUpLoadService.saveFileList(signature, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
        }
        if ((CollectionUtil.isNotEmpty(images))) {
            imageId = fileUpLoadService.saveFileList(images, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
        }

        Bwhbdatail dbl;

        for (ChangeTableDto dto : dtoList) {
            if (StringUtils.isEmpty(dto.getYhbh())) {
                throw new ToflyDeniedException("用户编号为空");
            }

            dbl = new Bwhbdatail();

            Qfkhjbxx qfkhjbxx = qfkhjbxxService.getOne(Wrappers.<Qfkhjbxx>lambdaQuery()
                    .eq(Qfkhjbxx::getYhbh, dto.getYhbh()));

            String date = DateUtil.getDateFormat(new Date(), "yyyyMMdd-");

            dto.setAction("QUERY");

            log.info(dto.toString());

            Map map = null;
            try {
                map = CallAPIUtil.callAPIUtil(dto, "/api/Business/WaterReplace");
            } catch (Exception e) {
                log.info(e.toString());
                e.printStackTrace();
            }
            //同步默认信息
            dbl.setIsSync(0L);
            if (CollectionUtil.isNotEmpty(map)) {
                //0000代表添加成功
                if ("0000".equals(map.get("Code"))) {
                    dbl.setIsSync(1L);
                    Long userId = SecurityUtils.getUserId();
                    dbl.setAzfs(dto.getSbazfs())
                            .setBwBz(dto.getBz())
                            .setBwHblx(dto.getSblx() != null ? Long.valueOf(dto.getSblx()) : null)
                            .setBwKhbh(dto.getYhbh())
                            .setBwHbrq(new Date())
                            .setSignature(signatureId)
                            .setImage(imageId)
                            .setBwHbry(dto.getHbr())
                            .setBwHbyy(dto.getHbyy())
                            .setBwJbzd(dto.getJbzd() != null ? Long.valueOf(dto.getJbzd()) : null)
                            .setBwJbqd(dto.getJbqd() != null ? Long.valueOf(dto.getJbqd()) : null)
                            .setSqr(dto.getSqr())
                            .setHbsl(dto.getHbsl())
                            .setBwLsbh(date + System.currentTimeMillis())
                            .setJsfs(dto.getHbsljsfs())
                            .setBwXbqd(dto.getXbqd() != null ? Long.valueOf(dto.getXbqd()) : null)
                            .setSbm(dto.getSbsbm())
                            .setQfh(dto.getSbqfh())
                            .setBwXbcj(dto.getSbmp())
                            .setBwXblx(dto.getSblx())
                            .setSbxh(dto.getSbxh())
                            .setBwXbkj(dto.getSbkj())
                            .setBwXbbh(dto.getSbbh())
                            .setCreateUser(userId)
                            .setBwHbbm(dto.getHbr() != null ? changeTableMapper.getDeptId(Long.valueOf(dto.getHbr())) : null);

                    if (qfkhjbxx != null) {
                        long betweenDay = cn.hutool.core.date.DateUtil.between(qfkhjbxx.getAzrq(), dbl.getBwHbrq(), DateUnit.DAY);
                        dbl.setBwJbcj(qfkhjbxx.getSbcj())
                                .setBwJblx(qfkhjbxx.getSblx())
                                .setBwJbbh(qfkhjbxx.getSbbh())
                                .setBwJbkj(qfkhjbxx.getSbkj())
                                .setBwKhdz(qfkhjbxx.getYhdz())
                                .setSyDate(betweenDay)
                                .setBwKhdh(qfkhjbxx.getLxdh())
                                .setJbAzrq(qfkhjbxx.getAzrq())
                                .setBwKhxm(qfkhjbxx.getYhxm());
                    }

                    bwhbdatailService.save(dbl);

                    Long departmentId = 0L;
                    if (dto.getHbr() != null) {
                        departmentId = userClent.getById(Long.valueOf(dto.getHbr())).getResult().getDepartmentId();
                    }

                    Bwrksbdatail bwrksbdatail = bwrksbdatailService.getOne(Wrappers.<Bwrksbdatail>lambdaQuery().eq(Bwrksbdatail::getBwSbbh, dto.getSbbh()));

                    if (bwrksbdatail != null) {
                        bwrksbdatail.setBwKhbh(dto.getYhbh())
                                .setBwKhxm(qfkhjbxx != null ? qfkhjbxx.getYhxm() : "")
                                .setBwKhdh(qfkhjbxx != null ? qfkhjbxx.getLxdh() : "")
                                .setBwKhdz(qfkhjbxx != null ? qfkhjbxx.getYhdz() : "")
                                .setBwAzry(StringUtils.isEmpty(dto.getHbr()) ? 0L : Long.parseLong(dto.getHbr()))
                                .setBwSbqd(StringUtils.isEmpty(dto.getXbqd()) ? 0 : Integer.parseInt(dto.getXbqd()))
                                .setAzfs(dto.getSbazfs())
                                .setSfncys(qfkhjbxx != null ? qfkhjbxx.getSfncys() : "")
                                .setJyz(qfkhjbxx != null ? qfkhjbxx.getJyz() : "")
                                .setAzbm(departmentId)
                                .setGddh(qfkhjbxx != null ? qfkhjbxx.getLxdh() : "")
                                .setSbazdz(qfkhjbxx != null ? qfkhjbxx.getYhdz() : "")
                                .setBwSbzt("3").setBwAzrq(new Date());

                        bwrksbdatailService.updateById(bwrksbdatail);
                    }

                    String sbbh = "-1";
                    if (qfkhjbxx != null) {
                        sbbh = qfkhjbxx.getSbbh();
                    }
                    //旧表改为换表状态
                    bwrksbdatailService.update(Wrappers.<Bwrksbdatail>lambdaUpdate()
                            .eq(Bwrksbdatail::getBwSbbh, sbbh).set(Bwrksbdatail::getBwSbzt, 9));
                } else {
                    throw new ToflyDeniedException(map.get("message").toString());
                }
            } else {
                throw new ToflyDeniedException("调用换表同步接口失败");
            }

        }
        return true;
    }

    /**
     * 查询安装水表用户
     *
     * @param dto 查询条件
     * @return list
     */
    @Override
    public Page<Map> queryInfo(Page page, String dto, String sbkj, String sblx, String sbcj) {
        return qfkhjbxxService.page(page, Wrappers.<Qfkhjbxx>lambdaQuery()
                .ne(Qfkhjbxx::getYhzt, 9)
                .eq(!StringUtils.isEmpty(sbcj), Qfkhjbxx::getSbcj, sbcj)
                .eq(!StringUtils.isEmpty(sbkj), Qfkhjbxx::getSbkj, sbkj)
                .eq(!StringUtils.isEmpty(sblx), Qfkhjbxx::getSblx, sblx)
                .and(!StringUtils.isEmpty(dto), v -> v.like(Qfkhjbxx::getYhbh, dto)
                        .or().like(Qfkhjbxx::getYhxm, dto)
                        .or().like(Qfkhjbxx::getYhdz, dto)
                        .or().like(Qfkhjbxx::getLxdh, dto)
                        .or().like(Qfkhjbxx::getSbbh, dto)));
    }

    @Override
    public List<Map> countChangeTable(HttpServletResponse response, ChangeTableCondition dto) {
        List<Map> maps = changeTableMapper.countChangeTable(dto);
        if (dto.getExport() != null && dto.getExport() == 1) {
            exportCommonCodeService.exportCode(response, "countChangeTableService", maps);
        }
        return maps;
    }

    @Override
    public Page<Map> queryChangeTable(Page page, HttpServletResponse response, ChangeTableCondition dto) {
        Page<Map> mapPage = changeTableMapper.queryChangeTable(page, dto);
        List<Map> records = mapPage.getRecords();
        if (CollectionUtil.isNotEmpty(records)) {
            for (Map map : records) {
                String image = String.valueOf(map.get("image"));
                map.put("imagePath", !StringUtils.isEmpty(image) ? bwhbdatailMapper.getImagePath(image) : null);

            }
        }
        if (dto.getExport() == 1) {
            exportCommonCodeService.exportCode(response, "queryChangeTableService", records);
        }
        return mapPage;
    }

    /**
     * 批量添加暂存接口
     *
     * @param json      数据
     * @param signature 签名
     * @param images    图片
     * @return boolean
     */
    @Override
    public boolean batchInsertStaging(String json, List<MultipartFile> signature, List<MultipartFile> images) {

        ChangeTableBatchDto changeTableBatchDto = JSONObject.parseObject(json, ChangeTableBatchDto.class);

        if (changeTableBatchDto == null || CollectionUtil.isEmpty(changeTableBatchDto.getInfos())) {
            throw new ToflyDeniedException("传入数据为空");
        }

        List<ChangeTableDto> dtoList = changeTableBatchDto.getInfos();

        String signatureId = null;
        String imageId = null;
        if (CollectionUtil.isNotEmpty(signature)) {
            signatureId = fileUpLoadService.saveFileList(signature, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
        }
        if ((CollectionUtil.isNotEmpty(images))) {
            imageId = fileUpLoadService.saveFileList(images, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
        }

        Bwhbdatail dbl;

        for (ChangeTableDto dto : dtoList) {
            if (StringUtils.isEmpty(dto.getYhbh())) {
                throw new ToflyDeniedException("用户编号为空");
            }

            dbl = new Bwhbdatail();

            Qfkhjbxx qfkhjbxx = qfkhjbxxService.getOne(Wrappers.<Qfkhjbxx>lambdaQuery()
                    .eq(Qfkhjbxx::getYhbh, dto.getYhbh()));

            String date = DateUtil.getDateFormat(new Date(), "yyyyMMdd-");
            //同步默认信息
            //添加默认状态未同步
            dbl.setIsSync(0L);
            Long userId = SecurityUtils.getUserId();
            dbl.setAzfs(dto.getSbazfs())
                    .setBwBz(dto.getBz())
                    .setBwHblx(dto.getSblx() != null ? Long.valueOf(dto.getSblx()) : null)
                    .setBwKhbh(dto.getYhbh())
                    .setBwHbrq(new Date())
                    .setSignature(signatureId)
                    .setImage(imageId)
                    .setBwHbry(dto.getHbr())
                    .setBwHbyy(dto.getHbyy())
                    .setBwJbzd(dto.getJbzd() != null ? Long.valueOf(dto.getJbzd()) : null)
                    .setBwJbqd(dto.getJbqd() != null ? Long.valueOf(dto.getJbqd()) : null)
                    .setSqr(dto.getSqr())
                    .setHbsl(dto.getHbsl())
                    .setBwLsbh(date + System.currentTimeMillis())
                    .setJsfs(dto.getHbsljsfs())
                    .setBwXbqd(dto.getXbqd() != null ? Long.valueOf(dto.getXbqd()) : null)
                    .setSbm(dto.getSbsbm())
                    .setQfh(dto.getSbqfh())
                    .setBwXbcj(dto.getSbmp())
                    .setBwXblx(dto.getSblx())
                    .setSbxh(dto.getSbxh())
                    .setBwXbkj(dto.getSbkj())
                    .setBwXbbh(dto.getSbbh())
                    .setCreateUser(userId)
                    .setBwHbbm(dto.getHbr() != null ? changeTableMapper.getDeptId(Long.valueOf(dto.getHbr())) : null);

            if (qfkhjbxx != null) {
                long betweenDay = cn.hutool.core.date.DateUtil.between(qfkhjbxx.getAzrq(), dbl.getBwHbrq(), DateUnit.DAY);
                dbl.setBwJbcj(qfkhjbxx.getSbcj())
                        .setBwJblx(qfkhjbxx.getSblx())
                        .setBwJbbh(qfkhjbxx.getSbbh())
                        .setBwJbkj(qfkhjbxx.getSbkj())
                        .setBwKhdz(qfkhjbxx.getYhdz())
                        .setSyDate(betweenDay)
                        .setBwKhdh(qfkhjbxx.getLxdh())
                        .setJbAzrq(qfkhjbxx.getAzrq())
                        .setBwKhxm(qfkhjbxx.getYhxm());
            }

            bwhbdatailService.save(dbl);
        }
        return true;
    }

    /**
     * 新水表接口
     *
     * @param ids 换表id
     * @return boolean
     */
    @Override
    public boolean newChangeTable(String ids) {
        List<Bwhbdatail> changeTableList = bwhbdatailService.list(Wrappers.<Bwhbdatail>lambdaQuery()
                .in(Bwhbdatail::getId, Arrays.asList(ids.split(","))));

        String codeStr = changeTableList.stream().filter(Objects::nonNull).filter(h -> h.getIsSync() == 1)
                .map(Bwhbdatail::getBwXbbh).collect(Collectors.joining(","));

        if (!StringUtils.isEmpty(codeStr)) {
            throw new ToflyDeniedException("水表：" + codeStr + "已同步，无需再次同步");
        }

        Long userId = SecurityUtils.getUserId();


        ChangeTableDto dto;
        for (Bwhbdatail change : changeTableList) {
            dto = new ChangeTableDto();

            dto.setAction("query").setHbsl(change.getHbsl())
                    .setBz(change.getBwBz())
                    .setCzy(userId.toString())
                    .setHbr(change.getBwHbry())
                    .setHbrq(DateUtil.parseDateToStr("yyyyMMdd", change.getBwHbrq()))
                    .setHbsljsfs(change.getJsfs())
                    .setHbyy(change.getBwHbyy())
                    .setJbqd(String.valueOf(change.getBwJbqd()))
                    .setJbzd(String.valueOf(change.getBwJbzd()))
                    .setSbazfs(change.getAzfs())
                    .setSbbh(change.getBwXbbh())
                    .setSbkj(change.getBwXbkj())
                    .setSblx(change.getBwXblx())
                    .setSbmp(change.getBwXbcj())
                    .setSbqfh(change.getQfh())
                    .setSbsbm(change.getSbm())
                    .setSbxh(change.getSbxh())
                    .setSqr(change.getSqr())
                    .setXbqd(String.valueOf(change.getBwXbqd()))
                    .setYhbh(change.getBwKhbh());

            Map map = null;
            try {
                log.info("换表同步数据为：" + JSONObject.toJSONString(dto));
                map = CallAPIUtil.callAPIUtil(dto, "/api/Business/WaterReplace");
            } catch (Exception e) {
                e.printStackTrace();
            }

            if (CollectionUtil.isEmpty(map)) {
                throw new ToflyDeniedException("接口调用失败");
            } else {
                if ("0000".equals(map.get("Code"))) {
                    Long departmentId = 0L;
                    if (dto.getHbr() != null) {
                        departmentId = userClent.getById(Long.valueOf(dto.getHbr())).getResult().getDepartmentId();
                    }
                    Bwrksbdatail bwrksbdatail = bwrksbdatailService.getOne(Wrappers.<Bwrksbdatail>lambdaQuery().eq(Bwrksbdatail::getBwSbbh, dto.getSbbh()));

                    if (bwrksbdatail != null) {
                        bwrksbdatail.setBwKhbh(dto.getYhbh())
                                .setBwKhxm(change.getBwKhxm())
                                .setBwKhdh(change.getBwKhbh())
                                .setBwKhdz(change.getBwKhdz())
                                .setBwAzry(StringUtils.isEmpty(change.getBwHbry()) ? 0L : Long.parseLong(change.getBwHbry()))
                                .setBwSbqd(StringUtils.isEmpty(change.getBwXbqd()) ? 0 : Integer.parseInt(String.valueOf(change.getBwXbqd())))
                                .setAzfs(change.getAzfs())
                                .setSfncys(change.getNcys())
                                .setJyz(change.getJyz())
                                .setAzbm(departmentId)
                                .setGddh(change.getBwKhdh())
                                .setSbazdz(change.getBwKhdz())
                                .setBwSbzt("3").setBwAzrq(change.getBwHbrq());

                        bwrksbdatailService.updateById(bwrksbdatail);
                    }
                    change.setIsSync(1L);
                    change.setMsg(map.get("message").toString());
                } else {
                    change.setIsSync(0L);
                    change.setMsg(map.get("message").toString());
                }
                bwhbdatailService.update(Wrappers.<Bwhbdatail>lambdaUpdate()
                        .eq(Bwhbdatail::getId, change.getId())
                        .set(Bwhbdatail::getIsSync, change.getIsSync())
                        .set(Bwhbdatail::getMsg, change.getMsg()));
            }
        }

        return true;
    }

    /**
     * 水表换表信息修改
     *
     * @param signatures 签名
     * @param images     图片
     * @param datail     换表信息
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateChangeTable(List<MultipartFile> signatures, List<MultipartFile> images, Bwhbdatail datail) {

        if (!StringUtils.isEmpty(datail.getBwKhbh())) {
            Qfkhjbxx info = qfkhjbxxService.getOne(Wrappers.<Qfkhjbxx>lambdaQuery()
                    .eq(Qfkhjbxx::getYhbh, datail.getBwKhbh()));
            datail.setBwKhxm(info.getYhxm());
            datail.setBwKhdz(info.getYhdz());
            datail.setBwKhdh(info.getLxdh());
        }

        if (!StringUtils.isEmpty(datail.getBwXbbh())) {
            Bwrksbdatail tableInfo = bwrksbdatailService.getOne(Wrappers.<Bwrksbdatail>lambdaQuery()
                    .eq(Bwrksbdatail::getBwSbbh, datail.getBwXbbh()));
            datail.setBwXbcj(String.valueOf(tableInfo.getBwSbcj()));
            datail.setBwXbkj(String.valueOf(tableInfo.getBwSbkj()));
            datail.setBwXblx(String.valueOf(tableInfo.getBwSblx()));
        }

        if (!StringUtils.isEmpty(datail.getBwJbbh())) {
            Bwrksbdatail tableInfo = bwrksbdatailService.getOne(Wrappers.<Bwrksbdatail>lambdaQuery()
                    .eq(Bwrksbdatail::getBwSbbh, datail.getBwJbbh()));
            datail.setBwJbcj(String.valueOf(tableInfo.getBwSbcj()));
            datail.setBwJbkj(String.valueOf(tableInfo.getBwSbkj()));
            datail.setBwJblx(String.valueOf(tableInfo.getBwSblx()));
        }

        String signatureId = null;
        String imageId = null;
        if (CollectionUtil.isNotEmpty(signatures)) {
            signatureId = fileUpLoadService.saveFileList(signatures, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
        }
        if ((CollectionUtil.isNotEmpty(images))) {
            imageId = fileUpLoadService.saveFileList(images, "/xrty/changeTable", "TF_YWPN_BWHBDATAIL_W").toString();
        }

        if (!StringUtils.isEmpty(signatureId)) {
            datail.setSignature(signatureId);
        }

        String image = datail.getImage();
        String imageNewId = null;
        if (!StringUtils.isEmpty(datail.getImagePath()) && "null".equals(datail.getImagePath())) {
            log.info("图片路径为：" + datail.getImagePath());
            List<String> fileIdList = fileService.list(Wrappers.<File>lambdaQuery().eq(File::getFilePath, Arrays.asList(datail.getImagePath().split(","))))
                    .stream().filter(Objects::nonNull).map(f -> String.valueOf(f.getId())).collect(Collectors.toList());
            if (CollectionUtil.isNotEmpty(fileIdList)) {
                List<String> imageIdList = Arrays.asList(image.split(","));
                imageIdList.removeAll(fileIdList);
                imageNewId = imageIdList.stream().filter(Objects::nonNull).collect(Collectors.joining(","));
            }
            datail.setImagePath(null);
        }
        if (!StringUtils.isEmpty(imageId)) {
            if (StringUtils.isEmpty(image)) {
                datail.setImage(imageId);
            } else {
                datail.setImage(StringUtils.isEmpty(imageNewId) ? imageId : imageNewId + "," + imageId);
            }
        }
        return bwhbdatailService.updateById(datail);
    }

}
