package com.tofly.watermeter.handler;


import com.tofly.watermeter.custom.service.DmaCodeService;
import com.tofly.watermeter.custom.service.ReadMeterService;
import com.tofly.watermeter.custom.service.SbPlanService;
import com.tofly.watermeter.custom.service.TimedTaskService;
import com.tofly.watermeter.custom.service.meterBusiness.ConfigManageService;
import com.tofly.watermeter.service.CbqydwaterService;
import com.tofly.watermeter.service.QfcbplandetailsService;
import com.tofly.watermeter.service.YhbingqyService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.AllArgsConstructor;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

/**
 * @author FlashWJL
 * @date 2021/4/8 14:40
 */
@Component
@AllArgsConstructor
public class GeneratePlanHandler {

    private final ReadMeterService meterService;
    private final TimedTaskService timedTaskService;
    private final SbPlanService sbPlanService;
    private final QfcbplandetailsService qfcbplandetailsService;
    private final CbqydwaterService cbqydwaterService;
    private final DmaCodeService dmaCodeService;
    private final ConfigManageService configManageService;
    /**
     * 分区分区段
     */
    private final YhbingqyService yhbingqyService;


    /**
     * 每月获取客户信息  五星 *****
     * cron 时间表达式 ：1 1 22 26 * ?（每月26号晚上十点一点一分）
     * 2021-11-26 22:01:01
     * 2021-12-26 22:01:01
     *
     * @param param
     * @return
     */
    @XxlJob("getClientInfo")
    public ReturnT<String> getClientInfo(String param) {
        meterService.genClientInfo();
//        System.out.println("getReadData成功调用");

        return ReturnT.SUCCESS;
    }


//

    /**
     * 每晚根据客户档案信息同步水量信息；更新抄表任务中的上次读数
     * @param param
     * @return
     */
//    @XxlJob("initCbslEveyDay")
//    public ReturnT<String> initCbslEveyDay(String param) {
//        qfcbplandetailsService.initCbslEveyDay();
////        System.out.println("getReadData成功调用");
//
//        return ReturnT.SUCCESS;
//    }


    /**
     * 获取历史抄表数据  优先级最高  五星 *****
     * 1 1 0 27 * ? （27号凌晨0点抄表历史数据获取）
     * 2021-11-27 00:01:01
     * 2021-12-27 00:01:01
     * @param param
     * @return
     */
//    @XxlJob("getReadDataHis")
//    public ReturnT<String> getReadDataHis(String param) {
//        meterService.generateHistory();
////        System.out.println("getReadData成功调用");
//        return ReturnT.SUCCESS;
//    }

    /**
     * 每月获取抄表数据    四星 *****
     * 1 1 2 27 * ?  （27号凌晨2点获取森鑫炬最新抄表数据）
     * 2021-11-27 02:01:01
     * 2021-12-27 02:01:01
     * @param param
     * @return
     */
//    @XxlJob("getReadData")
//    public ReturnT<String> getReadData(String param) {
//        meterService.generateReadPlan();
////        System.out.println("getReadData成功调用");
//
//        return ReturnT.SUCCESS;
//    }


    /**
     * 定时同步历史用水数据
     * 初始同步从2018年1月1日开始到现在
     * 历史同步 以最大年月开始同步
     * 1 1 2 27 * ?  （27号凌晨2点获取森鑫炬最新抄表数据）
     * 2021-11-27 02:01:01
     * 2021-12-27 02:01:01
     *
     * @param param
     * @return
     */
    @XxlJob("historyWaterData")
    public ReturnT<String> getHistoryWaterData(String param) {
        meterService.getHistoryWaterData(false);

        return ReturnT.SUCCESS;
    }


    /**
     * 初始化每下个月抄表数据月计划  三星 *****
     * 1 1 4 27 * ?  （27号凌晨四点初始化抄表月计划）
     * 2021-11-27 04:01:01
     * 2021-12-27 04:01:01
     *
     * @return
     */
    @XxlJob("initCbbzAndDown")
    public ReturnT<String> initCbbzAndDown(String param) {
        qfcbplandetailsService.initCbData();
        return ReturnT.SUCCESS;
    }


    /**
     * 下月抄表计划补充换表信息，在初始化抄表计划后
     * 1 1 5 27 * ?  (27号凌晨五点初始化抄表计划换表信息；在初始化抄表计划后完成)
     * 2021-11-27 05:01:01
     * 2021-12-27 05:01:01
     *
     * @param param
     * @return
     */
    @XxlJob("initCbPlanHb")
    public ReturnT<String> initCbPlanHb(String param) {
        qfcbplandetailsService.initCbDataByHb();
        return ReturnT.SUCCESS;
    }

    /**
     * 初始化下个月前三次抄表水量、抄表日期，上次读数
     * 1 1 6 27 * ?  （27号凌晨六点）
     * 2021-11-27 06:01:01
     * 2021-12-27 06:01:01
     * @param param
     * @return
     */
//    @XxlJob("initFrontThreeData")
//    public ReturnT<String> initFrontThreeData(String param) {
//        qfcbplandetailsService.initFrontThreeData();
//        return ReturnT.SUCCESS;
//    }

    /**
     * 初始化同比数据
     * 27号七点
     *
     * @param param
     * @return
     */
    @XxlJob("initFrontThreeYearData")
    public ReturnT<String> initFrontThreeYearData(String param) {
        qfcbplandetailsService.initFrontThreeYearData();
        return ReturnT.SUCCESS;
    }

    /**
     * 每天晚上9点汇总抄表水量数据
     *
     * @param param
     * @return
     */
    @XxlJob("waterAddingEveryDay")
    public ReturnT<String> waterAddingEveryDay(String param) {
        cbqydwaterService.waterAdding();
        return ReturnT.SUCCESS;
    }

    /**
     * 每8秒获取森鑫炬修改、新增客户（我感觉森鑫炬提供的数据不对，而且8秒，时间太频繁了）
     * @param param
     * @return
     */
//    @XxlJob("getUpdateAndInsert")
//    public ReturnT<String> getUpdateAndInsert(String param) {
//        qfkhjbxxService.getUpdateAndInsert();
////        System.out.println("getReadData成功调用");
//        return ReturnT.SUCCESS;
//    }

    /**
     * 同步抄表片区、抄表本、厂家、类型编码
     */
    @XxlJob("getDmaCode")
    @Transactional(rollbackFor = Exception.class)
    public ReturnT<String> getDmaCode(String param) throws Exception {
        dmaCodeService.synchronizationArea();
        dmaCodeService.synchronizationMeterReading();
        dmaCodeService.synchronizationVendor();
        dmaCodeService.synchronizationType();
        return ReturnT.SUCCESS;
    }

    /**
     * 每月推算周检客户水表
     * 2 2 1 28 * ?  （28号推算客户水表周检时间）
     *
     * @param param
     * @return
     */
    @XxlJob("generateReckonZj")
    public ReturnT<String> generateReckonZj(String param) {
        sbPlanService.generateReckonZj();
//        System.out.println("getReadData成功调用");
        return ReturnT.SUCCESS;
    }

    /**
     * 同步用户区域区域段
     *
     * @return
     */
    @Scheduled(cron = "0 0/20 * * * ?")
    public void qyAndqyd() {
        yhbingqyService.synchroYhQyAndQyd();
    }


    @XxlJob("monthFill")
    public ReturnT<String> monthFill(String param) {
        timedTaskService.sendMsgAbout(0L, 618L);
        return ReturnT.SUCCESS;
    }

    @XxlJob("dayFill")
    public ReturnT<String> dayFill(String param) {
        timedTaskService.sendMsgAbout(1L, 614L);
        return ReturnT.SUCCESS;
    }

    @XxlJob("sendNoteFeign")
    public ReturnT<String> sendNoteFeign(String param) {
        timedTaskService.sendWarnMsgAndSMS();
        return ReturnT.SUCCESS;
    }

    /**
     * 同步水表厂家
     * @param param param
     * @return ReturnT
     */
    @XxlJob("synVendor")
    public ReturnT<String> synVendor(String param){
        configManageService.synVendor();
        return ReturnT.SUCCESS;
    }

    /**
     * dma异常信息绩效评价
     * @param param  param
     * @return ReturnT<String>
     */
    @XxlJob("performanceEvaluation")
    public ReturnT<String> performanceEvaluation(String param) {
        timedTaskService.performanceEvaluation();
        return ReturnT.SUCCESS;
    }
    @XxlJob("synInfo")
    public ReturnT<String> synInfo(String param) {
        timedTaskService.synchronizingInformation();
        return ReturnT.SUCCESS;
    }

}
